@@ -7,13 +7,16 @@ set -xe
 # python -m pip install --no-deps cityhash
 
 if [[ ${UPSTREAM_DEV} ]]; then
-    # conda uninstall --force numpy pandas
-    # python -m pip install --no-deps --pre \
-    #     -i https://pypi.anaconda.org/scipy-wheels-nightly/simple \
-    #     numpy
-    # python -m pip install --pre pandas==1.1.0rc0
+    mamba install -y -c arrow-nightlies "pyarrow>5.0"
 
-    conda update -y -c arrow-nightlies pyarrow
+    # FIXME https://github.com/mamba-org/mamba/issues/412
+    # mamba uninstall --force numpy pandas
+    conda uninstall --force numpy pandas fastparquet
+
+    python -m pip install --no-deps --pre \
+        -i https://pypi.anaconda.org/scipy-wheels-nightly/simple \
+        numpy \
+        pandas
 
     python -m pip install \
         --upgrade \
@@ -24,21 +27,17 @@ if [[ ${UPSTREAM_DEV} ]]; then
         git+https://github.com/dask/partd \
         git+https://github.com/dask/zict \
         git+https://github.com/dask/distributed \
+        git+https://github.com/dask/fastparquet \
         git+https://github.com/zarr-developers/zarr-python
 fi
 
 # Install dask
 python -m pip install --quiet --no-deps -e .[complete]
-echo conda list
-conda list
+echo mamba list
+mamba list
 
 # For debugging
 echo -e "--\n--Conda Environment (re-create this with \`conda env create --name <name> -f <output_file>\`)\n--"
-conda env export | grep -E -v '^prefix:.*$'
-
-# Ensure cytoolz is not installed in the CI environment when tests are run
-# We can add cytoolz back in once
-# https://github.com/conda-forge/cytoolz-feedstock/issues/36 is resolved
-conda uninstall --force cytoolz
+mamba env export | grep -E -v '^prefix:.*$'
 
 set +xe
