@@ -1,160 +1,23 @@
 import warnings
-from operator import getitem
-from itertools import product
-from numbers import Integral
-from tlz import merge, pipe, concat, partial, get
+from numbers import Integral, Number
+
+import numpy as np
+from tlz import concat, get, partial
 from tlz.curried import map
 
-from . import chunk
-from .core import (
-    Array,
-    map_blocks,
-    concatenate,
-    concatenate3,
-    reshapelist,
-    unify_chunks,
-)
-from .creation import empty_like, full_like
-from ..highlevelgraph import HighLevelGraph
 from ..base import tokenize
-from ..core import flatten
-from ..utils import concrete
+from ..highlevelgraph import HighLevelGraph
+from ..layers import ArrayOverlapLayer
+from ..utils import derived_from
+from . import chunk, numpy_compat
+from .core import Array, concatenate, map_blocks, unify_chunks
+from .creation import empty_like, full_like
 
 
-def fractional_slice(task, axes):
-    """
-
-    >>> fractional_slice(('x', 5.1), {0: 2})  # doctest: +SKIP
-    (getitem, ('x', 6), (slice(0, 2),))
-
-    >>> fractional_slice(('x', 3, 5.1), {0: 2, 1: 3})  # doctest: +SKIP
-    (getitem, ('x', 3, 5), (slice(None, None, None), slice(-3, None)))
-
-    >>> fractional_slice(('x', 2.9, 5.1), {0: 2, 1: 3})  # doctest: +SKIP
-    (getitem, ('x', 3, 5), (slice(0, 2), slice(-3, None)))
-    """
-    rounded = (task[0],) + tuple(int(round(i)) for i in task[1:])
-
-    index = []
-    for i, (t, r) in enumerate(zip(task[1:], rounded[1:])):
-        depth = axes.get(i, 0)
-        if isinstance(depth, tuple):
-            left_depth = depth[0]
-            right_depth = depth[1]
-        else:
-            left_depth = depth
-            right_depth = depth
-
-        if t == r:
-            index.append(slice(None, None, None))
-        elif t < r and right_depth:
-            index.append(slice(0, right_depth))
-        elif t > r and left_depth:
-            index.append(slice(-left_depth, None))
-        else:
-            index.append(slice(0, 0))
-    index = tuple(index)
-
-    if all(ind == slice(None, None, None) for ind in index):
-        return task
-    else:
-        return (getitem, rounded, index)
-
-
-def expand_key(k, dims, name=None, axes=None):
-    """Get all neighboring keys around center
-
-    Parameters
-    ----------
-    k: tuple
-        They key around which to generate new keys
-    dims: Sequence[int]
-        The number of chunks in each dimension
-    name: Option[str]
-        The name to include in the output keys, or none to include no name
-    axes: Dict[int, int]
-        The axes active in the expansion.  We don't expand on non-active axes
-
-    Examples
-    --------
-    >>> expand_key(('x', 2, 3), dims=[5, 5], name='y', axes={0: 1, 1: 1})  # doctest: +NORMALIZE_WHITESPACE
-    [[('y', 1.1, 2.1), ('y', 1.1, 3), ('y', 1.1, 3.9)],
-     [('y',   2, 2.1), ('y',   2, 3), ('y',   2, 3.9)],
-     [('y', 2.9, 2.1), ('y', 2.9, 3), ('y', 2.9, 3.9)]]
-
-    >>> expand_key(('x', 0, 4), dims=[5, 5], name='y', axes={0: 1, 1: 1})  # doctest: +NORMALIZE_WHITESPACE
-    [[('y',   0, 3.1), ('y',   0,   4)],
-     [('y', 0.9, 3.1), ('y', 0.9,   4)]]
-    """
-
-    def inds(i, ind):
-        rv = []
-        if ind - 0.9 > 0:
-            rv.append(ind - 0.9)
-        rv.append(ind)
-        if ind + 0.9 < dims[i] - 1:
-            rv.append(ind + 0.9)
-        return rv
-
-    shape = []
-    for i, ind in enumerate(k[1:]):
-        num = 1
-        if ind > 0:
-            num += 1
-        if ind < dims[i] - 1:
-            num += 1
-        shape.append(num)
-
-    args = [
-        inds(i, ind) if any((axes.get(i, 0),)) else [ind] for i, ind in enumerate(k[1:])
-    ]
-    if name is not None:
-        args = [[name]] + args
-    seq = list(product(*args))
-    shape2 = [d if any((axes.get(i, 0),)) else 1 for i, d in enumerate(shape)]
-    result = reshapelist(shape2, seq)
-    return result
-
-
-def overlap_internal(x, axes):
-    """Share boundaries between neighboring blocks
-
-    Parameters
-    ----------
-
-    x: da.Array
-        A dask array
-    axes: dict
-        The size of the shared boundary per axis
-
-    The axes input informs how many cells to overlap between neighboring blocks
-    {0: 2, 2: 5} means share two cells in 0 axis, 5 cells in 2 axis
-    """
-    dims = list(map(len, x.chunks))
-    expand_key2 = partial(expand_key, dims=dims, axes=axes)
-
-    # Make keys for each of the surrounding sub-arrays
-    interior_keys = pipe(
-        x.__dask_keys__(), flatten, map(expand_key2), map(flatten), concat, list
-    )
-
-    name = "overlap-" + tokenize(x, axes)
-    getitem_name = "getitem-" + tokenize(x, axes)
-    interior_slices = {}
-    overlap_blocks = {}
-    for k in interior_keys:
-        frac_slice = fractional_slice((x.name,) + k, axes)
-        if (x.name,) + k != frac_slice:
-            interior_slices[(getitem_name,) + k] = frac_slice
-        else:
-            interior_slices[(getitem_name,) + k] = (x.name,) + k
-            overlap_blocks[(name,) + k] = (
-                concatenate3,
-                (concrete, expand_key2((None,) + k, name=getitem_name)),
-            )
-
+def _overlap_internal_chunks(original_chunks, axes):
+    """Get new chunks for array with overlap."""
     chunks = []
-    for i, bds in enumerate(x.chunks):
+    for i, bds in enumerate(original_chunks):
         depth = axes.get(i, 0)
         if isinstance(depth, tuple):
             left_depth = depth[0]
@@ -172,9 +35,35 @@ def overlap_internal(x, axes):
             for bd in bds[1:-1]:
                 mid.append(bd + left_depth + right_depth)
             chunks.append(left + mid + right)
+    return chunks
 
-    dsk = merge(interior_slices, overlap_blocks)
-    graph = HighLevelGraph.from_collections(name, dsk, dependencies=[x])
+
+def overlap_internal(x, axes):
+    """Share boundaries between neighboring blocks
+
+    Parameters
+    ----------
+
+    x: da.Array
+        A dask array
+    axes: dict
+        The size of the shared boundary per axis
+
+    The axes input informs how many cells to overlap between neighboring blocks
+    {0: 2, 2: 5} means share two cells in 0 axis, 5 cells in 2 axis
+    """
+    token = tokenize(x, axes)
+    name = "overlap-" + token
+
+    graph = ArrayOverlapLayer(
+        name=x.name,
+        axes=axes,
+        chunks=x.chunks,
+        numblocks=x.numblocks,
+        token=token,
+    )
+    graph = HighLevelGraph.from_collections(name, graph, dependencies=[x])
+    chunks = _overlap_internal_chunks(x.chunks, axes)
 
     return Array(graph, name, chunks, meta=x)
 
@@ -359,7 +248,7 @@ def nearest(x, axis, depth):
 
 
 def constant(x, axis, depth, value):
-    """ Add constant slice to either side of array """
+    """Add constant slice to either side of array"""
     chunks = list(x.chunks)
     chunks[axis] = (depth,)
 
@@ -394,9 +283,9 @@ def boundaries(x, depth=None, kind=None):
     constant
     """
     if not isinstance(kind, dict):
-        kind = dict((i, kind) for i in range(x.ndim))
+        kind = {i: kind for i in range(x.ndim)}
     if not isinstance(depth, dict):
-        depth = dict((i, depth) for i in range(x.ndim))
+        depth = {i: depth for i in range(x.ndim)}
 
     for i in range(x.ndim):
         d = depth.get(i, 0)
@@ -532,14 +421,13 @@ def overlap(x, depth, boundary):
     new_chunks = tuple(
         ensure_minimum_chunksize(size, c) for size, c in zip(depths, x.chunks)
     )
-    x1 = x.rechunk(new_chunks)
+    x1 = x.rechunk(new_chunks)  # this is a no-op if x.chunks == new_chunks
 
     x2 = boundaries(x1, depth2, boundary2)
     x3 = overlap_internal(x2, depth2)
-    trim = dict(
-        (k, v * 2 if boundary2.get(k, "none") != "none" else 0)
-        for k, v in depth2.items()
-    )
+    trim = {
+        k: v * 2 if boundary2.get(k, "none") != "none" else 0 for k, v in depth2.items()
+    }
     x4 = chunk.trim(x3, trim)
     return x4
 
@@ -649,7 +537,7 @@ def map_overlap(
 
     >>> x = np.arange(16).reshape((4, 4))
     >>> d = da.from_array(x, chunks=(2, 2))
-    >>> d.map_overlap(lambda x: x + x.size, depth=1).compute()
+    >>> d.map_overlap(lambda x: x + x.size, depth=1, boundary='reflect').compute()
     array([[16, 17, 18, 19],
            [20, 21, 22, 23],
            [24, 25, 26, 27],
@@ -669,7 +557,7 @@ def map_overlap(
     >>> func = lambda x, y: x + y
     >>> x = da.arange(8).reshape(2, 4).rechunk((1, 2))
     >>> y = da.arange(4).rechunk(2)
-    >>> da.map_overlap(func, x, y, depth=1).compute() # doctest: +NORMALIZE_WHITESPACE
+    >>> da.map_overlap(func, x, y, depth=1, boundary='reflect').compute() # doctest: +NORMALIZE_WHITESPACE
     array([[ 0,  2,  4,  6],
            [ 4,  6,  8,  10]])
 
@@ -681,11 +569,11 @@ def map_overlap(
 
     >>> x = da.arange(8, chunks=4)
     >>> y = da.arange(8, chunks=2)
-    >>> r = da.map_overlap(func, x, y, depth=1, align_arrays=True)
+    >>> r = da.map_overlap(func, x, y, depth=1, boundary='reflect', align_arrays=True)
     >>> len(r.to_delayed())
     4
 
-    >>> da.map_overlap(func, x, y, depth=1, align_arrays=False).compute()
+    >>> da.map_overlap(func, x, y, depth=1, boundary='reflect', align_arrays=False).compute()
     Traceback (most recent call last):
         ...
     ValueError: Shapes do not align {'.0': {2, 4}}
@@ -699,9 +587,9 @@ def map_overlap(
     >>> block_args = dict(chunks=(), drop_axis=0)
     >>> da.map_blocks(func, x, **block_args).compute()
     10
-    >>> da.map_overlap(func, x, **block_args).compute()
+    >>> da.map_overlap(func, x, **block_args, boundary='reflect').compute()
     10
-    >>> da.map_overlap(func, x, **block_args, depth=1).compute()
+    >>> da.map_overlap(func, x, **block_args, depth=1, boundary='reflect').compute()
     12
 
     For functions that may not handle 0-d arrays, it's also possible to specify
@@ -711,7 +599,7 @@ def map_overlap(
 
     >>> x = np.arange(16).reshape((4, 4))
     >>> d = da.from_array(x, chunks=(2, 2))
-    >>> y = d.map_overlap(lambda x: x + x[2], depth=1, meta=np.array(()))
+    >>> y = d.map_overlap(lambda x: x + x[2], depth=1, boundary='reflect', meta=np.array(()))
     >>> y
     dask.array<_trim, shape=(4, 4), dtype=float64, chunksize=(2, 2), chunktype=numpy.ndarray>
     >>> y.compute()
@@ -725,7 +613,7 @@ def map_overlap(
     >>> import cupy  # doctest: +SKIP
     >>> x = cupy.arange(16).reshape((4, 4))  # doctest: +SKIP
     >>> d = da.from_array(x, chunks=(2, 2))  # doctest: +SKIP
-    >>> y = d.map_overlap(lambda x: x + x[2], depth=1, meta=cupy.array(()))  # doctest: +SKIP
+    >>> y = d.map_overlap(lambda x: x + x[2], depth=1, boundary='reflect', meta=cupy.array(()))  # doctest: +SKIP
     >>> y  # doctest: +SKIP
     dask.array<_trim, shape=(4, 4), dtype=float64, chunksize=(2, 2), chunktype=cupy.ndarray>
     >>> y.compute()  # doctest: +SKIP
@@ -749,6 +637,17 @@ def map_overlap(
         trim = get(sig.index("trim"), args, trim)
         func, args = args[0], [func]
 
+    if boundary is None:
+        # Default boundary value is set in the function named "coerce_boundary"
+        warnings.warn(
+            "Default 'boundary' argument value will change from 'reflect' "
+            "to 'none' in future versions from 2022.03.0 onwards. "
+            "Use 'boundary=\"none\"' to opt into the future behavior now "
+            "or set 'boundary=\"reflect\"' to maintain the current behavior "
+            "going forward.",
+            FutureWarning,
+        )
+
     if not callable(func):
         raise TypeError(
             "First argument must be callable function, not {}\n"
@@ -780,6 +679,10 @@ def map_overlap(
         inds = [list(reversed(range(x.ndim))) for x in args]
         _, args = unify_chunks(*list(concat(zip(args, inds))), warn=False)
 
+    # Escape to map_blocks if depth is zero (a more efficient computation)
+    if all([all(depth_val == 0 for depth_val in d.values()) for d in depth]):
+        return map_blocks(func, *args, **kwargs)
+
     for i, x in enumerate(args):
         for j in range(x.ndim):
             if isinstance(depth[i][j], tuple) and boundary[i][j] != "none":
@@ -803,7 +706,23 @@ def map_overlap(
         # Find index of array argument with maximum rank and break ties by choosing first provided
         i = sorted(enumerate(args), key=lambda v: (v[1].ndim, -v[0]))[-1][0]
         # Trim using depth/boundary setting for array of highest rank
-        return trim_internal(x, depth[i], boundary[i])
+        depth = depth[i]
+        boundary = boundary[i]
+        # remove any dropped axes from depth and boundary variables
+        drop_axis = kwargs.pop("drop_axis", None)
+        if drop_axis is not None:
+            if isinstance(drop_axis, Number):
+                drop_axis = [drop_axis]
+
+            # convert negative drop_axis to equivalent positive value
+            ndim_out = max(a.ndim for a in args if isinstance(a, Array))
+            drop_axis = [d % ndim_out for d in drop_axis]
+
+            kept_axes = tuple(ax for ax in range(args[i].ndim) if ax not in drop_axis)
+            # note that keys are relabeled to match values in range(x.ndim)
+            depth = {n: depth[ax] for n, ax in enumerate(kept_axes)}
+            boundary = {n: boundary[ax] for n, ax in enumerate(kept_axes)}
+        return trim_internal(x, depth, boundary)
     else:
         return x
 
@@ -817,9 +736,7 @@ def coerce_depth(ndim, depth):
     if isinstance(depth, tuple):
         depth = dict(zip(range(ndim), depth))
     if isinstance(depth, dict):
-        for i in range(ndim):
-            if i not in depth:
-                depth[i] = 0
+        depth = {ax: depth.get(ax, default) for ax in range(ndim)}
     return coerce_depth_type(ndim, depth)
 
 
@@ -841,7 +758,67 @@ def coerce_boundary(ndim, boundary):
     if isinstance(boundary, tuple):
         boundary = dict(zip(range(ndim), boundary))
     if isinstance(boundary, dict):
-        for i in range(ndim):
-            if i not in boundary:
-                boundary[i] = default
+        boundary = {ax: boundary.get(ax, default) for ax in range(ndim)}
     return boundary
+
+
+@derived_from(numpy_compat)
+def sliding_window_view(x, window_shape, axis=None):
+    from numpy.core.numeric import normalize_axis_tuple
+
+    window_shape = tuple(window_shape) if np.iterable(window_shape) else (window_shape,)
+
+    window_shape_array = np.array(window_shape)
+    if np.any(window_shape_array <= 0):
+        raise ValueError("`window_shape` must contain values > 0")
+
+    if axis is None:
+        axis = tuple(range(x.ndim))
+        if len(window_shape) != len(axis):
+            raise ValueError(
+                f"Since axis is `None`, must provide "
+                f"window_shape for all dimensions of `x`; "
+                f"got {len(window_shape)} window_shape elements "
+                f"and `x.ndim` is {x.ndim}."
+            )
+    else:
+        axis = normalize_axis_tuple(axis, x.ndim, allow_duplicate=True)
+        if len(window_shape) != len(axis):
+            raise ValueError(
+                f"Must provide matching length window_shape and "
+                f"axis; got {len(window_shape)} window_shape "
+                f"elements and {len(axis)} axes elements."
+            )
+
+    depths = [0] * x.ndim
+    for ax, window in zip(axis, window_shape):
+        depths[ax] += window - 1
+
+    # Ensure that each chunk is big enough to leave at least a size-1 chunk
+    # after windowing (this is only really necessary for the last chunk).
+    safe_chunks = tuple(
+        ensure_minimum_chunksize(d + 1, c) for d, c in zip(depths, x.chunks)
+    )
+    x = x.rechunk(safe_chunks)
+
+    # result.shape = x_shape_trimmed + window_shape,
+    # where x_shape_trimmed is x.shape with every entry
+    # reduced by one less than the corresponding window size.
+    # trim chunks to match x_shape_trimmed
+    newchunks = tuple(c[:-1] + (c[-1] - d,) for d, c in zip(depths, x.chunks)) + tuple(
+        (window,) for window in window_shape
+    )
+
+    return map_overlap(
+        numpy_compat.sliding_window_view,
+        x,
+        depth=tuple((0, d) for d in depths),  # Overlap on +ve side only
+        boundary="none",
+        meta=x._meta,
+        new_axis=range(x.ndim, x.ndim + len(axis)),
+        chunks=newchunks,
+        trim=False,
+        align_arrays=False,
+        window_shape=window_shape,
+        axis=axis,
+    )
