@@ -1,36 +1,35 @@
 import builtins
-from collections.abc import Iterable
+import contextlib
 import operator
+from collections.abc import Iterable
 from functools import partial
 from itertools import product, repeat
-from math import factorial, log, ceil
-
-import numpy as np
+from math import ceil, factorial, log, log2
 from numbers import Integral, Number
 
-from toolz import compose, partition_all, get, accumulate, pluck
+import numpy as np
+from tlz import accumulate, compose, drop, get, partition_all, pluck
 
-from . import chunk
-from .core import _concatenate2, Array, handle_out, implements
-from .blockwise import blockwise
-from ..blockwise import lol_tuples
-from .creation import arange, diagonal
-from .ufunc import sqrt
-from .utils import full_like_safe, validate_axis, compute_meta, is_arraylike
-from .wrap import zeros, ones
-from .numpy_compat import ma_divide, divide as np_divide
-from ..base import tokenize
-from ..highlevelgraph import HighLevelGraph
-from ..utils import ignoring, funcname, Dispatch, deepmap, getargspec, derived_from
 from .. import config
+from ..base import tokenize
+from ..blockwise import lol_tuples
+from ..highlevelgraph import HighLevelGraph
+from ..utils import deepmap, derived_from, funcname, getargspec, is_series_like
+from . import chunk
+from .blockwise import blockwise
+from .core import Array, _concatenate2, handle_out, implements, unknown_chunk_message
+from .creation import arange, diagonal
 
-# Generic functions to support chunks of different types
-empty_lookup = Dispatch("empty")
-empty_lookup.register((object, np.ndarray), np.empty)
-empty_lookup.register(np.ma.masked_array, np.ma.empty)
-divide_lookup = Dispatch("divide")
-divide_lookup.register((object, np.ndarray), np_divide)
-divide_lookup.register(np.ma.masked_array, ma_divide)
+# Keep empty_lookup here for backwards compatibility
+from .dispatch import divide_lookup, empty_lookup  # noqa: F401
+from .utils import (
+    asarray_safe,
+    compute_meta,
+    is_arraylike,
+    meta_from_array,
+    validate_axis,
+)
+from .wrap import ones, zeros
 
 
 def divide(a, b, dtype=None):
@@ -54,7 +53,7 @@ def reduction(
     output_size=1,
     meta=None,
 ):
-    """ General version of reductions
+    """General version of reductions
 
     Parameters
     ----------
@@ -78,8 +77,9 @@ def reduction(
     keepdims: boolean, optional
         Whether the reduction function should preserve the reduced axes,
         leaving them at size ``output_size``, or remove them.
-    dtype: np.dtype, optional
-        Force output dtype. Defaults to x.dtype if omitted.
+    dtype: np.dtype
+        data type of output. This argument was previously optional, but
+        leaving as ``None`` will now raise an exception.
     split_every: int >= 2 or dict(axis: int), optional
         Determines the depth of the recursive aggregation. If set to or more
         than the number of input chunks, the aggregation will be performed in
@@ -149,12 +149,14 @@ def reduction(
         chunk = partial(chunk, dtype=dtype)
     if "dtype" in getargspec(aggregate).args:
         aggregate = partial(aggregate, dtype=dtype)
+    if is_series_like(x):
+        x = x.values
 
     # Map chunk across all blocks
     inds = tuple(range(x.ndim))
     # The dtype of `tmp` doesn't actually matter, and may be incorrect.
     tmp = blockwise(
-        chunk, inds, x, inds, axis=axis, keepdims=True, dtype=dtype or float
+        chunk, inds, x, inds, axis=axis, keepdims=True, token=name, dtype=dtype or float
     )
     tmp._chunks = tuple(
         (output_size,) * len(c) if i in axis else c for i, c in enumerate(tmp.chunks)
@@ -207,14 +209,14 @@ def _tree_reduce(
     concatenate=True,
     reduced_meta=None,
 ):
-    """ Perform the tree reduction step of a reduction.
+    """Perform the tree reduction step of a reduction.
 
     Lower level, users should use ``reduction`` or ``arg_reduction`` directly.
     """
     # Normalize split_every
     split_every = split_every or config.get("split_every", 4)
     if isinstance(split_every, dict):
-        split_every = dict((k, split_every.get(k, 2)) for k in axis)
+        split_every = {k: split_every.get(k, 2) for k in axis}
     elif isinstance(split_every, Integral):
         n = builtins.max(int(split_every ** (1 / (len(axis) or 1))), 2)
         split_every = dict.fromkeys(axis, n)
@@ -256,7 +258,7 @@ def _tree_reduce(
 def partial_reduce(
     func, x, split_every, keepdims=False, dtype=None, name=None, reduced_meta=None
 ):
-    """ Partial reduction across multiple axes.
+    """Partial reduction across multiple axes.
 
     Parameters
     ----------
@@ -291,7 +293,7 @@ def partial_reduce(
         out_chunks = list(getter(out_chunks))
     dsk = {}
     for k, p in zip(keys, product(*parts)):
-        decided = dict((i, j[0]) for (i, j) in enumerate(p) if len(j) == 1)
+        decided = {i: j[0] for (i, j) in enumerate(p) if len(j) == 1}
         dummy = dict(i for i in enumerate(p) if i[0] not in decided)
         g = lol_tuples((x.name,), range(x.ndim), decided, dummy)
         dsk[(name,) + k] = (func, g)
@@ -303,7 +305,12 @@ def partial_reduce(
             meta = func(reduced_meta, computing_meta=True)
         # no meta keyword argument exists for func, and it isn't required
         except TypeError:
-            meta = func(reduced_meta)
+            try:
+                meta = func(reduced_meta)
+            except ValueError as e:
+                # min/max functions have no identity, don't apply function to meta
+                if "zero-size array to reduction operation" in str(e):
+                    meta = reduced_meta
         # when no work can be computed on the empty array (e.g., func is a ufunc)
         except ValueError:
             pass
@@ -320,7 +327,7 @@ def partial_reduce(
     if np.isscalar(meta):
         return Array(graph, name, out_chunks, dtype=dtype)
     else:
-        with ignoring(AttributeError):
+        with contextlib.suppress(AttributeError):
             meta = meta.astype(dtype)
         return Array(graph, name, out_chunks, meta=meta)
 
@@ -436,40 +443,88 @@ def nansum(a, axis=None, dtype=None, keepdims=False, split_every=None, out=None)
     )
 
 
-with ignoring(AttributeError):
+@derived_from(np)
+def nanprod(a, axis=None, dtype=None, keepdims=False, split_every=None, out=None):
+    if dtype is not None:
+        dt = dtype
+    else:
+        dt = getattr(chunk.nansum(np.empty((1,), dtype=a.dtype)), "dtype", object)
+    return reduction(
+        a,
+        chunk.nanprod,
+        chunk.prod,
+        axis=axis,
+        keepdims=keepdims,
+        dtype=dt,
+        split_every=split_every,
+        out=out,
+    )
 
-    @derived_from(np)
-    def nanprod(a, axis=None, dtype=None, keepdims=False, split_every=None, out=None):
-        if dtype is not None:
-            dt = dtype
-        else:
-            dt = getattr(chunk.nansum(np.empty((1,), dtype=a.dtype)), "dtype", object)
-        return reduction(
-            a,
-            chunk.nanprod,
-            chunk.prod,
-            axis=axis,
-            keepdims=keepdims,
-            dtype=dt,
-            split_every=split_every,
-            out=out,
-        )
 
-    @derived_from(np)
-    def nancumsum(x, axis, dtype=None, out=None):
-        return cumreduction(chunk.nancumsum, operator.add, 0, x, axis, dtype, out=out)
+@derived_from(np)
+def nancumsum(x, axis, dtype=None, out=None, *, method="sequential"):
+    """Dask added an additional keyword-only argument ``method``.
 
-    @derived_from(np)
-    def nancumprod(x, axis, dtype=None, out=None):
-        return cumreduction(chunk.nancumprod, operator.mul, 1, x, axis, dtype, out=out)
+    method : {'sequential', 'blelloch'}, optional
+        Choose which method to use to perform the cumsum.  Default is 'sequential'.
+
+        * 'sequential' performs the cumsum of each prior block before the current block.
+        * 'blelloch' is a work-efficient parallel cumsum.  It exposes parallelism by
+            first taking the sum of each block and combines the sums via a binary tree.
+            This method may be faster or more memory efficient depending on workload,
+            scheduler, and hardware.  More benchmarking is necessary.
+    """
+    return cumreduction(
+        chunk.nancumsum,
+        operator.add,
+        0,
+        x,
+        axis,
+        dtype,
+        out=out,
+        method=method,
+        preop=np.nansum,
+    )
+
+
+@derived_from(np)
+def nancumprod(x, axis, dtype=None, out=None, *, method="sequential"):
+    """Dask added an additional keyword-only argument ``method``.
+
+    method : {'sequential', 'blelloch'}, optional
+        Choose which method to use to perform the cumprod.  Default is 'sequential'.
+
+        * 'sequential' performs the cumprod of each prior block before the current block.
+        * 'blelloch' is a work-efficient parallel cumprod.  It exposes parallelism by first
+            taking the product of each block and combines the products via a binary tree.
+            This method may be faster or more memory efficient depending on workload,
+            scheduler, and hardware.  More benchmarking is necessary.
+    """
+    return cumreduction(
+        chunk.nancumprod,
+        operator.mul,
+        1,
+        x,
+        axis,
+        dtype,
+        out=out,
+        method=method,
+        preop=np.nanprod,
+    )
 
 
 @derived_from(np)
 def nanmin(a, axis=None, keepdims=False, split_every=None, out=None):
+    if np.isnan(a.size):
+        raise ValueError(f"Arrays chunk sizes are unknown. {unknown_chunk_message}")
+    if a.size == 0:
+        raise ValueError(
+            "zero-size array to reduction operation fmin which has no identity"
+        )
     return reduction(
         a,
-        chunk.nanmin,
-        chunk.nanmin,
+        _nanmin_skip,
+        _nanmin_skip,
         axis=axis,
         keepdims=keepdims,
         dtype=a.dtype,
@@ -478,12 +533,27 @@ def nanmin(a, axis=None, keepdims=False, split_every=None, out=None):
     )
 
 
+def _nanmin_skip(x_chunk, axis, keepdims):
+    if x_chunk.size > 0:
+        return np.nanmin(x_chunk, axis=axis, keepdims=keepdims)
+    else:
+        return asarray_safe(
+            np.array([], dtype=x_chunk.dtype), like=meta_from_array(x_chunk)
+        )
+
+
 @derived_from(np)
 def nanmax(a, axis=None, keepdims=False, split_every=None, out=None):
+    if np.isnan(a.size):
+        raise ValueError(f"Arrays chunk sizes are unknown. {unknown_chunk_message}")
+    if a.size == 0:
+        raise ValueError(
+            "zero-size array to reduction operation fmax which has no identity"
+        )
     return reduction(
         a,
-        chunk.nanmax,
-        chunk.nanmax,
+        _nanmax_skip,
+        _nanmax_skip,
         axis=axis,
         keepdims=keepdims,
         dtype=a.dtype,
@@ -492,8 +562,17 @@ def nanmax(a, axis=None, keepdims=False, split_every=None, out=None):
     )
 
 
+def _nanmax_skip(x_chunk, axis, keepdims):
+    if x_chunk.size > 0:
+        return np.nanmax(x_chunk, axis=axis, keepdims=keepdims)
+    else:
+        return asarray_safe(
+            np.array([], dtype=x_chunk.dtype), like=meta_from_array(x_chunk)
+        )
+
+
 def numel(x, **kwargs):
-    """ A reduction to count the number of elements """
+    """A reduction to count the number of elements"""
 
     if hasattr(x, "mask"):
         return chunk.sum(np.ones_like(x), **kwargs)
@@ -506,7 +585,7 @@ def numel(x, **kwargs):
     if axis is None:
         prod = np.prod(shape, dtype=dtype)
         return (
-            full_like_safe(x, prod, shape=(1,) * len(shape), dtype=dtype)
+            np.full_like(x, prod, shape=(1,) * len(shape), dtype=dtype)
             if keepdims is True
             else prod
         )
@@ -521,11 +600,11 @@ def numel(x, **kwargs):
         )
     else:
         new_shape = tuple(shape[dim] for dim in range(len(shape)) if dim not in axis)
-    return full_like_safe(x, prod, shape=new_shape, dtype=dtype)
+    return np.full_like(x, prod, shape=new_shape, dtype=dtype)
 
 
 def nannumel(x, **kwargs):
-    """ A reduction to count the number of elements """
+    """A reduction to count the number of elements"""
     return chunk.sum(~(np.isnan(x)), **kwargs)
 
 
@@ -548,7 +627,7 @@ def mean_combine(
     dtype="f8",
     axis=None,
     computing_meta=False,
-    **kwargs
+    **kwargs,
 ):
     if not isinstance(pairs, list):
         pairs = [pairs]
@@ -576,15 +655,18 @@ def mean_agg(pairs, dtype="f8", axis=None, computing_meta=False, **kwargs):
     totals = deepmap(lambda pair: pair["total"], pairs)
     total = _concatenate2(totals, axes=axis).sum(axis=axis, dtype=dtype, **kwargs)
 
-    return divide(total, n, dtype=dtype)
+    with np.errstate(divide="ignore", invalid="ignore"):
+        return divide(total, n, dtype=dtype)
 
 
 @derived_from(np)
 def mean(a, axis=None, dtype=None, keepdims=False, split_every=None, out=None):
     if dtype is not None:
         dt = dtype
+    elif a.dtype == object:
+        dt = object
     else:
-        dt = getattr(np.mean(np.empty(shape=(1,), dtype=a.dtype)), "dtype", object)
+        dt = getattr(np.mean(np.zeros(shape=(1,), dtype=a.dtype)), "dtype", object)
     return reduction(
         a,
         mean_chunk,
@@ -619,10 +701,6 @@ def nanmean(a, axis=None, dtype=None, keepdims=False, split_every=None, out=None
     )
 
 
-with ignoring(AttributeError):
-    nanmean = derived_from(np)(nanmean)
-
-
 def moment_chunk(
     A, order=2, sum=chunk.sum, numel=numel, dtype="f8", computing_meta=False, **kwargs
 ):
@@ -657,7 +735,7 @@ def moment_combine(
     sum=np.sum,
     axis=None,
     computing_meta=False,
-    **kwargs
+    **kwargs,
 ):
     if not isinstance(pairs, list):
         pairs = [pairs]
@@ -676,8 +754,10 @@ def moment_combine(
     Ms = _concatenate2(deepmap(lambda pair: pair["M"], pairs), axes=axis)
 
     total = totals.sum(axis=axis, **kwargs)
-    mu = divide(total, n, dtype=dtype)
-    inner_term = divide(totals, ns, dtype=dtype) - mu
+
+    with np.errstate(divide="ignore", invalid="ignore"):
+        mu = divide(total, n, dtype=dtype)
+        inner_term = divide(totals, ns, dtype=dtype) - mu
 
     xs = [
         _moment_helper(Ms, ns, inner_term, o, sum, axis, kwargs)
@@ -695,7 +775,7 @@ def moment_agg(
     sum=np.sum,
     axis=None,
     computing_meta=False,
-    **kwargs
+    **kwargs,
 ):
     if not isinstance(pairs, list):
         pairs = [pairs]
@@ -717,7 +797,9 @@ def moment_agg(
     Ms = _concatenate2(deepmap(lambda pair: pair["M"], pairs), axes=axis)
 
     mu = divide(totals.sum(axis=axis, **keepdim_kw), n, dtype=dtype)
-    inner_term = divide(totals, ns, dtype=dtype) - mu
+
+    with np.errstate(divide="ignore", invalid="ignore"):
+        inner_term = divide(totals, ns, dtype=dtype) - mu
 
     M = _moment_helper(Ms, ns, inner_term, order, sum, axis, kwargs)
 
@@ -727,7 +809,7 @@ def moment_agg(
     if isinstance(denominator, Number):
         if denominator < 0:
             denominator = np.nan
-    else:
+    elif denominator is not np.ma.masked:
         denominator[denominator < 0] = np.nan
 
     return divide(M, denominator, dtype=dtype)
@@ -743,9 +825,13 @@ def moment(
         reduced = a.sum(axis=axis)  # get reduced shape and chunks
         if order == 0:
             # When order equals 0, the result is 1, by definition.
-            return ones(reduced.shape, chunks=reduced.chunks, dtype="f8")
+            return ones(
+                reduced.shape, chunks=reduced.chunks, dtype="f8", meta=reduced._meta
+            )
         # By definition the first order about the mean is 0.
-        return zeros(reduced.shape, chunks=reduced.chunks, dtype="f8")
+        return zeros(
+            reduced.shape, chunks=reduced.chunks, dtype="f8", meta=reduced._meta
+        )
 
     if dtype is not None:
         dt = dtype
@@ -808,13 +894,30 @@ def nanvar(
     )
 
 
-with ignoring(AttributeError):
-    nanvar = derived_from(np)(nanvar)
+def _sqrt(a):
+    o = np.sqrt(a)
+    if isinstance(o, np.ma.masked_array) and not o.shape and o.mask.all():
+        return np.ma.masked
+    return o
+
+
+def safe_sqrt(a):
+    """A version of sqrt that properly handles scalar masked arrays.
+
+    To mimic ``np.ma`` reductions, we need to convert scalar masked arrays that
+    have an active mask to the ``np.ma.masked`` singleton. This is properly
+    handled automatically for reduction code, but not for ufuncs. We implement
+    a simple version here, since calling `np.ma.sqrt` everywhere is
+    significantly more expensive.
+    """
+    if hasattr(a, "_elemwise"):
+        return a._elemwise(_sqrt, a)
+    return _sqrt(a)
 
 
 @derived_from(np)
 def std(a, axis=None, dtype=None, keepdims=False, ddof=0, split_every=None, out=None):
-    result = sqrt(
+    result = safe_sqrt(
         var(
             a,
             axis=axis,
@@ -834,7 +937,7 @@ def std(a, axis=None, dtype=None, keepdims=False, ddof=0, split_every=None, out=
 def nanstd(
     a, axis=None, dtype=None, keepdims=False, ddof=0, split_every=None, out=None
 ):
-    result = sqrt(
+    result = safe_sqrt(
         nanvar(
             a,
             axis=axis,
@@ -850,13 +953,20 @@ def nanstd(
     return result
 
 
-with ignoring(AttributeError):
-    nanstd = derived_from(np)(nanstd)
-
-
 def _arg_combine(data, axis, argfunc, keepdims=False):
-    """ Merge intermediate results from ``arg_*`` functions"""
-    axis = None if len(axis) == data.ndim or data.ndim == 1 else axis[0]
+    """Merge intermediate results from ``arg_*`` functions"""
+    if isinstance(data, dict):
+        # Array type doesn't support structured arrays (e.g., CuPy),
+        # therefore `data` is stored in a `dict`.
+        assert data["vals"].ndim == data["arg"].ndim
+        axis = (
+            None
+            if len(axis) == data["vals"].ndim or data["vals"].ndim == 1
+            else axis[0]
+        )
+    else:
+        axis = None if len(axis) == data.ndim or data.ndim == 1 else axis[0]
+
     vals = data["vals"]
     arg = data["arg"]
     if axis is None:
@@ -895,9 +1005,14 @@ def arg_chunk(func, argfunc, x, axis, offset_info):
             fill_value = np.ma.maximum_fill_value(vals)
         vals = np.ma.filled(vals, fill_value)
 
-    result = np.empty(
-        shape=vals.shape, dtype=[("vals", vals.dtype), ("arg", arg.dtype)]
-    )
+    try:
+        result = np.empty_like(
+            vals, shape=vals.shape, dtype=[("vals", vals.dtype), ("arg", arg.dtype)]
+        )
+    except TypeError:
+        # Array type doesn't support structured arrays (e.g., CuPy)
+        result = dict()
+
     result["vals"] = vals
     result["arg"] = arg
     return result
@@ -905,9 +1020,15 @@ def arg_chunk(func, argfunc, x, axis, offset_info):
 
 def arg_combine(func, argfunc, data, axis=None, **kwargs):
     arg, vals = _arg_combine(data, axis, argfunc, keepdims=True)
-    result = np.empty(
-        shape=vals.shape, dtype=[("vals", vals.dtype), ("arg", arg.dtype)]
-    )
+
+    try:
+        result = np.empty_like(
+            vals, shape=vals.shape, dtype=[("vals", vals.dtype), ("arg", arg.dtype)]
+        )
+    except TypeError:
+        # Array type doesn't support structured arrays (e.g., CuPy).
+        result = dict()
+
     result["vals"] = vals
     result["arg"] = arg
     return result
@@ -925,7 +1046,7 @@ def nanarg_agg(func, argfunc, data, axis=None, **kwargs):
 
 
 def arg_reduction(x, chunk, combine, agg, axis=None, split_every=None, out=None):
-    """ Generic function for argreduction.
+    """Generic function for argreduction.
 
     Parameters
     ----------
@@ -947,7 +1068,7 @@ def arg_reduction(x, chunk, combine, agg, axis=None, split_every=None, out=None)
         axis = (axis,)
         ravel = x.ndim == 1
     else:
-        raise TypeError("axis must be either `None` or int, got '{0}'".format(axis))
+        raise TypeError(f"axis must be either `None` or int, got '{axis}'")
 
     for ax in axis:
         chunks = x.chunks[ax]
@@ -961,7 +1082,7 @@ def arg_reduction(x, chunk, combine, agg, axis=None, split_every=None, out=None)
             )
 
     # Map chunk across all blocks
-    name = "arg-reduce-{0}".format(tokenize(axis, x, chunk, combine, split_every))
+    name = f"arg-reduce-{tokenize(axis, x, chunk, combine, split_every)}"
     old = x.name
     keys = list(product(*map(range, x.numblocks)))
     offsets = list(product(*(accumulate(operator.add, bd[:-1], 0) for bd in x.chunks)))
@@ -971,20 +1092,28 @@ def arg_reduction(x, chunk, combine, agg, axis=None, split_every=None, out=None)
         offset_info = pluck(axis[0], offsets)
 
     chunks = tuple((1,) * len(c) if i in axis else c for (i, c) in enumerate(x.chunks))
-    dsk = dict(
-        ((name,) + k, (chunk, (old,) + k, axis, off))
+    dsk = {
+        (name,) + k: (chunk, (old,) + k, axis, off)
         for (k, off) in zip(keys, offset_info)
-    )
-    # The dtype of `tmp` doesn't actually matter, just need to provide something
+    }
+
+    dtype = np.argmin(asarray_safe([1], like=meta_from_array(x)))
+    meta = None
+    if is_arraylike(dtype):
+        # This case occurs on non-NumPy types (e.g., CuPy), where the returned
+        # value is an ndarray rather than a scalar.
+        meta = dtype
+        dtype = meta.dtype
+
     graph = HighLevelGraph.from_collections(name, dsk, dependencies=[x])
-    tmp = Array(graph, name, chunks, dtype=x.dtype)
-    dtype = np.argmin([1]).dtype
+    tmp = Array(graph, name, chunks, dtype=dtype, meta=meta)
+
     result = _tree_reduce(tmp, agg, axis, False, dtype, split_every, combine)
     return handle_out(out, result)
 
 
 def make_arg_reduction(func, argfunc, is_nan_func=False):
-    """ Create an argreduction callable
+    """Create an argreduction callable
 
     Parameters
     ----------
@@ -1030,8 +1159,184 @@ nanargmin = make_arg_reduction(chunk.nanmin, _nanargmin, True)
 nanargmax = make_arg_reduction(chunk.nanmax, _nanargmax, True)
 
 
-def cumreduction(func, binop, ident, x, axis=None, dtype=None, out=None):
-    """ Generic function for cumulative reduction
+def _prefixscan_combine(func, binop, pre, x, axis, dtype):
+    """Combine results of a parallel prefix scan such as cumsum
+
+    Parameters
+    ----------
+    func : callable
+        Cumulative function (e.g. ``np.cumsum``)
+    binop : callable
+        Associative function (e.g. ``add``)
+    pre : np.array
+        The value calculated in parallel from ``preop``.
+        For example, the sum of all the previous blocks.
+    x : np.array
+        Current block
+    axis : int
+    dtype : dtype
+
+    Returns
+    -------
+    np.array
+    """
+    # We could compute this in two tasks.
+    # This would allow us to do useful work (i.e., func), while waiting on `pre`.
+    # Using one task may guide the scheduler to do better and reduce scheduling overhead.
+    return binop(pre, func(x, axis=axis, dtype=dtype))
+
+
+def _prefixscan_first(func, x, axis, dtype):
+    """Compute the prefix scan (e.g., cumsum) on the first block
+
+    Parameters
+    ----------
+    func : callable
+        Cumulative function (e.g. ``np.cumsum``)
+    x : np.array
+        Current block
+    axis : int
+    dtype : dtype
+
+    Returns
+    -------
+    np.array
+    """
+    return func(x, axis=axis, dtype=dtype)
+
+
+def prefixscan_blelloch(func, preop, binop, x, axis=None, dtype=None, out=None):
+    """Generic function to perform parallel cumulative scan (a.k.a prefix scan)
+
+    The Blelloch prefix scan is work-efficient and exposes parallelism.
+    A parallel cumsum works by first taking the sum of each block, then do a binary tree
+    merge followed by a fan-out (i.e., the Brent-Kung pattern).  We then take the cumsum
+    of each block and add the sum of the previous blocks.
+
+    When performing a cumsum across N chunks, this method has 2 * lg(N) levels of dependencies.
+    In contrast, the sequential method has N levels of dependencies.
+
+    Floating point operations should be more accurate with this method compared to sequential.
+
+    Parameters
+    ----------
+    func : callable
+        Cumulative function (e.g. ``np.cumsum``)
+    preop : callable
+        Function to get the final value of a cumulative function (e.g., ``np.sum``)
+    binop : callable
+        Associative function (e.g. ``add``)
+    x : dask array
+    axis : int
+    dtype : dtype
+
+    Returns
+    -------
+    dask array
+    """
+    if axis is None:
+        x = x.flatten().rechunk(chunks=x.npartitions)
+        axis = 0
+    if dtype is None:
+        dtype = getattr(func(np.empty((0,), dtype=x.dtype)), "dtype", object)
+    assert isinstance(axis, Integral)
+    axis = validate_axis(axis, x.ndim)
+    name = f"{func.__name__}-{tokenize(func, axis, preop, binop, x, dtype)}"
+    base_key = (name,)
+
+    # Right now, the metadata for batches is incorrect, but this should be okay
+    batches = x.map_blocks(preop, axis=axis, keepdims=True, dtype=dtype)
+    # We don't need the last index until the end
+    *indices, last_index = full_indices = [
+        list(
+            product(
+                *[range(nb) if j != axis else [i] for j, nb in enumerate(x.numblocks)]
+            )
+        )
+        for i in range(x.numblocks[axis])
+    ]
+    prefix_vals = [[(batches.name,) + index for index in vals] for vals in indices]
+    dsk = {}
+    n_vals = len(prefix_vals)
+    level = 0
+    if n_vals >= 2:
+        # Upsweep
+        stride = 1
+        stride2 = 2
+        while stride2 <= n_vals:
+            for i in range(stride2 - 1, n_vals, stride2):
+                new_vals = []
+                for index, left_val, right_val in zip(
+                    indices[i], prefix_vals[i - stride], prefix_vals[i]
+                ):
+                    key = base_key + index + (level, i)
+                    dsk[key] = (binop, left_val, right_val)
+                    new_vals.append(key)
+                prefix_vals[i] = new_vals
+            stride = stride2
+            stride2 *= 2
+            level += 1
+
+        # Downsweep
+        # With `n_vals == 3`, we would have `stride = 1` and `stride = 0`, but we need
+        # to do a downsweep iteration, so make sure stride2 is at least 2.
+        stride2 = builtins.max(2, 2 ** ceil(log2(n_vals // 2)))
+        stride = stride2 // 2
+        while stride > 0:
+            for i in range(stride2 + stride - 1, n_vals, stride2):
+                new_vals = []
+                for index, left_val, right_val in zip(
+                    indices[i], prefix_vals[i - stride], prefix_vals[i]
+                ):
+                    key = base_key + index + (level, i)
+                    dsk[key] = (binop, left_val, right_val)
+                    new_vals.append(key)
+                prefix_vals[i] = new_vals
+            stride2 = stride
+            stride //= 2
+            level += 1
+
+    if full_indices:
+        for index in full_indices[0]:
+            dsk[base_key + index] = (
+                _prefixscan_first,
+                func,
+                (x.name,) + index,
+                axis,
+                dtype,
+            )
+        for indexes, vals in zip(drop(1, full_indices), prefix_vals):
+            for index, val in zip(indexes, vals):
+                dsk[base_key + index] = (
+                    _prefixscan_combine,
+                    func,
+                    binop,
+                    val,
+                    (x.name,) + index,
+                    axis,
+                    dtype,
+                )
+    if len(full_indices) < 2:
+        deps = [x]
+    else:
+        deps = [x, batches]
+    graph = HighLevelGraph.from_collections(name, dsk, dependencies=deps)
+    result = Array(graph, name, x.chunks, batches.dtype)
+    return handle_out(out, result)
+
+
+def cumreduction(
+    func,
+    binop,
+    ident,
+    x,
+    axis=None,
+    dtype=None,
+    out=None,
+    method="sequential",
+    preop=None,
+):
+    """Generic function for cumulative reduction
 
     Parameters
     ----------
@@ -1044,6 +1349,16 @@ def cumreduction(func, binop, ident, x, axis=None, dtype=None, out=None):
     x: dask Array
     axis: int
     dtype: dtype
+    method : {'sequential', 'blelloch'}, optional
+        Choose which method to use to perform the cumsum.  Default is 'sequential'.
+
+        * 'sequential' performs the scan of each prior block before the current block.
+        * 'blelloch' is a work-efficient parallel scan.  It exposes parallelism by first
+          calling ``preop`` on each block and combines the values via a binary tree.
+          This method may be faster or more memory efficient depending on workload,
+          scheduler, and hardware.  More benchmarking is necessary.
+    preop: callable, optional
+        Function used by 'blelloch' method like `np.cumsum->np.sum`` or ``np.cumprod->np.prod``
 
     Returns
     -------
@@ -1054,8 +1369,19 @@ def cumreduction(func, binop, ident, x, axis=None, dtype=None, out=None):
     cumsum
     cumprod
     """
+    if method == "blelloch":
+        if preop is None:
+            raise TypeError(
+                'cumreduction with "blelloch" method required `preop=` argument'
+            )
+        return prefixscan_blelloch(func, preop, binop, x, axis, dtype, out=out)
+    elif method != "sequential":
+        raise ValueError(
+            f'Invalid method for cumreduction.  Expected "sequential" or "blelloch".  Got: {method!r}'
+        )
+
     if axis is None:
-        x = x.flatten()
+        x = x.flatten().rechunk(chunks=x.npartitions)
         axis = 0
     if dtype is None:
         dtype = getattr(func(np.empty((0,), dtype=x.dtype)), "dtype", object)
@@ -1064,7 +1390,7 @@ def cumreduction(func, binop, ident, x, axis=None, dtype=None, out=None):
 
     m = x.map_blocks(func, axis=axis, dtype=dtype)
 
-    name = "{0}-{1}".format(func.__name__, tokenize(func, axis, binop, ident, x, dtype))
+    name = f"{func.__name__}-{tokenize(func, axis, binop, ident, x, dtype)}"
     n = x.numblocks[axis]
     full = slice(None, None, None)
     slc = (full,) * axis + (slice(-1, None),) + (full,) * (x.ndim - axis - 1)
@@ -1114,17 +1440,59 @@ def _cumprod_merge(a, b):
 
 
 @derived_from(np)
-def cumsum(x, axis=None, dtype=None, out=None):
-    return cumreduction(np.cumsum, _cumsum_merge, 0, x, axis, dtype, out=out)
+def cumsum(x, axis=None, dtype=None, out=None, method="sequential"):
+    """Dask added an additional keyword-only argument ``method``.
+
+    method : {'sequential', 'blelloch'}, optional
+        Choose which method to use to perform the cumsum.  Default is 'sequential'.
+
+        * 'sequential' performs the cumsum of each prior block before the current block.
+        * 'blelloch' is a work-efficient parallel cumsum.  It exposes parallelism by
+          first taking the sum of each block and combines the sums via a binary tree.
+          This method may be faster or more memory efficient depending on workload,
+          scheduler, and hardware.  More benchmarking is necessary.
+    """
+    return cumreduction(
+        np.cumsum,
+        _cumsum_merge,
+        0,
+        x,
+        axis,
+        dtype,
+        out=out,
+        method=method,
+        preop=np.sum,
+    )
 
 
 @derived_from(np)
-def cumprod(x, axis=None, dtype=None, out=None):
-    return cumreduction(np.cumprod, _cumprod_merge, 1, x, axis, dtype, out=out)
+def cumprod(x, axis=None, dtype=None, out=None, method="sequential"):
+    """Dask added an additional keyword-only argument ``method``.
+
+    method : {'sequential', 'blelloch'}, optional
+        Choose which method to use to perform the cumprod.  Default is 'sequential'.
+
+        * 'sequential' performs the cumprod of each prior block before the current block.
+        * 'blelloch' is a work-efficient parallel cumprod.  It exposes parallelism by first
+          taking the product of each block and combines the products via a binary tree.
+          This method may be faster or more memory efficient depending on workload,
+          scheduler, and hardware.  More benchmarking is necessary.
+    """
+    return cumreduction(
+        np.cumprod,
+        _cumprod_merge,
+        1,
+        x,
+        axis,
+        dtype,
+        out=out,
+        method=method,
+        preop=np.prod,
+    )
 
 
 def topk(a, k, axis=-1, split_every=None):
-    """ Extract the k largest elements from a on the given axis,
+    """Extract the k largest elements from a on the given axis,
     and return them sorted from largest to smallest.
     If k is negative, extract the -k smallest elements instead,
     and return them sorted from smallest to largest.
@@ -1183,7 +1551,7 @@ def topk(a, k, axis=-1, split_every=None):
 
 
 def argtopk(a, k, axis=-1, split_every=None):
-    """ Extract the indices of the k largest elements from a on the given axis,
+    """Extract the indices of the k largest elements from a on the given axis,
     and return them sorted from largest to smallest. If k is negative, extract
     the indices of the -k smallest elements instead, and return them sorted
     from smallest to largest.
@@ -1262,7 +1630,7 @@ def trace(a, offset=0, axis1=0, axis2=1, dtype=None):
 @derived_from(np)
 def median(a, axis=None, keepdims=False, out=None):
     """
-    This works by automatically chunking the reduced axes to a single chunk
+    This works by automatically chunking the reduced axes to a single chunk if necessary
     and then calling ``numpy.median`` function across the remaining dimensions
     """
     if axis is None:
@@ -1276,7 +1644,9 @@ def median(a, axis=None, keepdims=False, out=None):
 
     axis = [ax + a.ndim if ax < 0 else ax for ax in axis]
 
-    a = a.rechunk({ax: -1 if ax in axis else "auto" for ax in range(a.ndim)})
+    # rechunk if reduced axes are not contained in a single chunk
+    if builtins.any(a.numblocks[ax] > 1 for ax in axis):
+        a = a.rechunk({ax: -1 if ax in axis else "auto" for ax in range(a.ndim)})
 
     result = a.map_blocks(
         np.median,
@@ -1309,7 +1679,9 @@ def nanmedian(a, axis=None, keepdims=False, out=None):
 
     axis = [ax + a.ndim if ax < 0 else ax for ax in axis]
 
-    a = a.rechunk({ax: -1 if ax in axis else "auto" for ax in range(a.ndim)})
+    # rechunk if reduced axes are not contained in a single chunk
+    if builtins.any(a.numblocks[ax] > 1 for ax in axis):
+        a = a.rechunk({ax: -1 if ax in axis else "auto" for ax in range(a.ndim)})
 
     result = a.map_blocks(
         np.nanmedian,
