@@ -1,38 +1,39 @@
-import pytest
 import numpy as np
+import pytest
 
 import dask.array as da
-from dask.array.utils import assert_eq, IS_NEP18_ACTIVE
 from dask.array.numpy_compat import _numpy_120
+from dask.array.utils import assert_eq
 
 from .test_dispatch import EncapsulateNDArray, WrappedArray
 
 
-missing_arrfunc_cond = not IS_NEP18_ACTIVE
-missing_arrfunc_reason = "NEP-18 support is not available in NumPy"
-
-
-@pytest.mark.skipif(missing_arrfunc_cond, reason=missing_arrfunc_reason)
 @pytest.mark.parametrize(
     "func",
     [
+        lambda x: np.append(x, x),
         lambda x: np.concatenate([x, x, x]),
         lambda x: np.cov(x, x),
         lambda x: np.dot(x, x),
-        lambda x: np.dstack(x),
+        lambda x: np.dstack((x, x)),
         lambda x: np.flip(x, axis=0),
-        lambda x: np.hstack(x),
+        lambda x: np.hstack((x, x)),
         lambda x: np.matmul(x, x),
         lambda x: np.mean(x),
         lambda x: np.stack([x, x]),
         lambda x: np.block([x, x]),
         lambda x: np.sum(x),
         lambda x: np.var(x),
-        lambda x: np.vstack(x),
+        lambda x: np.vstack((x, x)),
         lambda x: np.linalg.norm(x),
         lambda x: np.min(x),
         lambda x: np.amin(x),
         lambda x: np.round(x),
+        lambda x: np.insert(x, 0, 3, axis=0),
+        lambda x: np.delete(x, 0, axis=0),
+        lambda x: np.select(
+            [x < 0.3, x < 0.6, x > 0.7], [x * 2, x, x / 2], default=0.65
+        ),
     ],
 )
 def test_array_function_dask(func):
@@ -45,7 +46,24 @@ def test_array_function_dask(func):
     assert_eq(res_y, res_x)
 
 
-@pytest.mark.skipif(missing_arrfunc_cond, reason=missing_arrfunc_reason)
+@pytest.mark.parametrize(
+    "func",
+    [
+        lambda x: np.dstack(x),
+        lambda x: np.hstack(x),
+        lambda x: np.vstack(x),
+    ],
+)
+def test_stack_functions_require_sequence_of_arrays(func):
+    x = np.random.random((100, 100))
+    y = da.from_array(x, chunks=(50, 50))
+
+    with pytest.raises(
+        NotImplementedError, match="expects a sequence of arrays as the first argument"
+    ):
+        func(y)
+
+
 @pytest.mark.parametrize("func", [np.fft.fft, np.fft.fft2])
 def test_array_function_fft(func):
     x = np.random.random((100, 100))
@@ -58,7 +76,6 @@ def test_array_function_fft(func):
     assert_eq(res_y, res_x)
 
 
-@pytest.mark.skipif(missing_arrfunc_cond, reason=missing_arrfunc_reason)
 @pytest.mark.parametrize(
     "func",
     [
@@ -77,7 +94,6 @@ def test_array_notimpl_function_dask(func):
         func(y)
 
 
-@pytest.mark.skipif(missing_arrfunc_cond, reason=missing_arrfunc_reason)
 @pytest.mark.parametrize(
     "func", [lambda x: np.real(x), lambda x: np.imag(x), lambda x: np.transpose(x)]
 )
@@ -91,8 +107,6 @@ def test_array_function_sparse(func):
     assert_eq(func(x), func(y))
 
 
-@pytest.mark.skipif(missing_arrfunc_cond, reason=missing_arrfunc_reason)
-@pytest.mark.xfail(_numpy_120, reason="sparse-383")
 def test_array_function_sparse_tensordot():
     sparse = pytest.importorskip("sparse")
     x = np.random.random((2, 3, 4))
@@ -108,7 +122,6 @@ def test_array_function_sparse_tensordot():
     )
 
 
-@pytest.mark.skipif(missing_arrfunc_cond, reason=missing_arrfunc_reason)
 @pytest.mark.parametrize("chunks", [(100, 100), (500, 100)])
 def test_array_function_cupy_svd(chunks):
     cupy = pytest.importorskip("cupy")
@@ -124,22 +137,21 @@ def test_array_function_cupy_svd(chunks):
     assert_eq(v, v_base)
 
 
-@pytest.mark.skipif(missing_arrfunc_cond, reason=missing_arrfunc_reason)
 @pytest.mark.parametrize(
     "func",
     [
         lambda x: np.concatenate([x, x, x]),
         lambda x: np.cov(x, x),
         lambda x: np.dot(x, x),
-        lambda x: np.dstack(x),
+        lambda x: np.dstack((x, x)),
         lambda x: np.flip(x, axis=0),
-        lambda x: np.hstack(x),
+        lambda x: np.hstack((x, x)),
         lambda x: np.matmul(x, x),
         lambda x: np.mean(x),
         lambda x: np.stack([x, x]),
         lambda x: np.sum(x),
         lambda x: np.var(x),
-        lambda x: np.vstack(x),
+        lambda x: np.vstack((x, x)),
         lambda x: np.linalg.norm(x),
     ],
 )
@@ -151,7 +163,7 @@ def test_unregistered_func(func):
     y = da.from_array(x, chunks=(50, 50))
 
     # Check if it's an equivalent array
-    assert_eq(x, y, check_meta=False)
+    assert_eq(x, y, check_meta=False, check_type=False)
 
     # Perform two NumPy functions, one on the
     # Encapsulated array
@@ -162,23 +174,19 @@ def test_unregistered_func(func):
     yy = func(y)
 
     # Check that they are equivalent arrays.
-    assert_eq(xx, yy, check_meta=False)
+    assert_eq(xx, yy, check_meta=False, check_type=False)
 
 
 def test_non_existent_func():
     # Regression test for __array_function__ becoming default in numpy 1.17
     # dask has no sort function, so ensure that this still calls np.sort
     x = da.from_array(np.array([1, 2, 4, 3]), chunks=(2,))
-    if IS_NEP18_ACTIVE:
-        with pytest.warns(
-            FutureWarning, match="The `numpy.sort` function is not implemented by Dask"
-        ):
-            assert list(np.sort(x)) == [1, 2, 3, 4]
-    else:
+    with pytest.warns(
+        FutureWarning, match="The `numpy.sort` function is not implemented by Dask"
+    ):
         assert list(np.sort(x)) == [1, 2, 3, 4]
 
 
-@pytest.mark.skipif(missing_arrfunc_cond, reason=missing_arrfunc_reason)
 @pytest.mark.parametrize(
     "func",
     [
@@ -206,9 +214,32 @@ def test_non_existent_func():
     ],
 )
 def test_binary_function_type_precedence(func, arr_upcast, arr_downcast):
-    """ Test proper dispatch on binary NumPy functions"""
+    """Test proper dispatch on binary NumPy functions"""
     assert (
         type(func(arr_upcast, arr_downcast))
         == type(func(arr_downcast, arr_upcast))
         == type(arr_upcast)
     )
+
+
+@pytest.mark.parametrize("func", [da.array, da.asarray, da.asanyarray, da.tri])
+def test_like_raises(func):
+    if _numpy_120:
+        assert_eq(func(1, like=func(1)), func(1))
+    else:
+        with pytest.raises(
+            RuntimeError, match="The use of ``like`` required NumPy >= 1.20"
+        ):
+            func(1, like=func(1))
+
+
+@pytest.mark.skipif(not _numpy_120, reason="NEP-35 is not available")
+@pytest.mark.parametrize("func", [np.array, np.asarray, np.asanyarray])
+def test_like_with_numpy_func(func):
+    assert_eq(func(1, like=da.array(1)), func(1))
+
+
+@pytest.mark.skipif(not _numpy_120, reason="NEP-35 is not available")
+@pytest.mark.parametrize("func", [np.array, np.asarray, np.asanyarray])
+def test_like_with_numpy_func_and_dtype(func):
+    assert_eq(func(1, dtype=float, like=da.array(1)), func(1, dtype=float))
