@@ -2,8 +2,7 @@ import numpy as np
 import pytest
 
 import dask.array as da
-from dask.array.numpy_compat import _numpy_120
-from dask.array.utils import meta_from_array, assert_eq
+from dask.array.utils import assert_eq, meta_from_array
 
 asarrays = [np.asarray]
 
@@ -24,9 +23,6 @@ except ImportError:
 
 @pytest.mark.parametrize("asarray", asarrays)
 def test_meta_from_array(asarray):
-    if "COO.from_numpy" in str(asarray) and _numpy_120:
-        raise pytest.xfail(reason="sparse-383")
-
     x = np.array(1)
     assert meta_from_array(x, ndim=1).shape == (0,)
 
@@ -40,6 +36,7 @@ def test_meta_from_array(asarray):
     assert meta_from_array(x, ndim=2).shape == (0, 0)
     assert meta_from_array(x, ndim=4).shape == (0, 0, 0, 0)
     assert meta_from_array(x, dtype="float64").dtype == "float64"
+    assert meta_from_array(x, dtype=float).dtype == "float64"
 
     x = da.ones((1,))
     assert isinstance(meta_from_array(x), np.ndarray)
@@ -49,7 +46,7 @@ def test_meta_from_array(asarray):
     assert meta_from_array(np.dtype("float32")) == np.dtype("float32")
 
 
-@pytest.mark.parametrize("meta", ["", "str", u"", u"str", b"", b"str"])
+@pytest.mark.parametrize("meta", ["", "str", "", "str", b"", b"str"])
 @pytest.mark.parametrize("dtype", [None, "bool", "int", "float"])
 def test_meta_from_array_literal(meta, dtype):
     if dtype is None:
@@ -77,3 +74,16 @@ def test_meta_from_array_type_inputs():
     assert_eq(x, x)
 
     assert da.from_array(np.ones(5).astype(np.int32), meta=np.ndarray).dtype == np.int32
+
+
+@pytest.mark.parametrize(
+    "a,b",
+    [
+        (da.array([1]), 1.0),
+        (da.array([1, 2]), [1.0, 2]),
+        (da.array([1, 2]), np.array([1.0, 2])),
+    ],
+)
+def test_assert_eq_checks_dtype(a, b):
+    with pytest.raises(AssertionError, match="a and b have different dtypes"):
+        assert_eq(a, b)
