@@ -1,20 +1,27 @@
 import collections
-import warnings
 from operator import add
 
-import pytest
 import numpy as np
+import pytest
 
 import dask
 import dask.array as da
-from dask.highlevelgraph import HighLevelGraph
-from dask.blockwise import Blockwise, rewrite_blockwise, optimize_blockwise, index_subs
 from dask.array.utils import assert_eq
-from dask.array.numpy_compat import _numpy_116
-from dask.utils_test import inc, dec
+from dask.blockwise import (
+    _BLOCKWISE_DEFAULT_PREFIX,
+    Blockwise,
+    _unique_dep,
+    index_subs,
+    optimize_blockwise,
+    rewrite_blockwise,
+)
+from dask.highlevelgraph import HighLevelGraph
+from dask.utils_test import dec, inc
 
 a, b, c, d, e, f, g = "abcdefg"
-_0, _1, _2, _3, _4, _5, _6, _7, _8, _9 = ["_%d" % i for i in range(10)]
+_0, _1, _2, _3, _4, _5, _6, _7, _8, _9 = (
+    _BLOCKWISE_DEFAULT_PREFIX + "%d" % i for i in range(10)
+)
 i, j, k = "ijk"
 
 
@@ -29,21 +36,40 @@ i, j, k = "ijk"
                 (c, "i", {c: (dec, _0)}, [(a, "i")]),
                 (d, "i", {d: (add, _0, _1, _2)}, [(a, "i"), (b, "i"), (c, "i")]),
             ],
-            (d, "i", {b: (inc, _0), c: (dec, _0), d: (add, _0, b, c)}, [(a, "i")]),
+            (
+                d,
+                "i",
+                {
+                    _unique_dep(b, "i"): (inc, _0),
+                    _unique_dep(c, "i"): (dec, _0),
+                    d: (add, _0, _unique_dep(b, "i"), _unique_dep(c, "i")),
+                },
+                [(a, "i")],
+            ),
         ],
         [
             [
                 (b, "i", {b: (inc, _0)}, [(a, "i")]),
                 (c, "j", {c: (inc, _0)}, [(b, "j")]),
             ],
-            (c, "j", {b: (inc, _0), c: (inc, b)}, [(a, "j")]),
+            (
+                c,
+                "j",
+                {_unique_dep(b, "j"): (inc, _0), c: (inc, _unique_dep(b, "j"))},
+                [(a, "j")],
+            ),
         ],
         [
             [
                 (b, "i", {b: (sum, _0)}, [(a, "ij")]),
                 (c, "k", {c: (inc, _0)}, [(b, "k")]),
             ],
-            (c, "k", {b: (sum, _0), c: (inc, b)}, [(a, "kA")]),
+            (
+                c,
+                "k",
+                {_unique_dep(b, "k"): (sum, _0), c: (inc, _unique_dep(b, "k"))},
+                [(a, "kA")],
+            ),
         ],
         [
             [
@@ -54,7 +80,11 @@ i, j, k = "ijk"
             (
                 g,
                 "ij",
-                {g: (add, c, d), c: (inc, _0), d: (inc, _1)},
+                {
+                    g: (add, _unique_dep(c, "i"), _unique_dep(d, "j")),
+                    _unique_dep(c, "i"): (inc, _0),
+                    _unique_dep(d, "j"): (inc, _1),
+                },
                 [(a, "i"), (b, "j")],
             ),
         ],
@@ -63,14 +93,27 @@ i, j, k = "ijk"
                 (b, "ji", {b: (np.transpose, _0)}, [(a, "ij")]),
                 (c, "ij", {c: (add, _0, _1)}, [(a, "ij"), (b, "ij")]),
             ],
-            (c, "ij", {c: (add, _0, b), b: (np.transpose, _1)}, [(a, "ij"), (a, "ji")]),
+            (
+                c,
+                "ij",
+                {
+                    c: (add, _0, _unique_dep(b, "ij")),
+                    _unique_dep(b, "ij"): (np.transpose, _1),
+                },
+                [(a, "ij"), (a, "ji")],
+            ),
         ],
         [
             [
                 (c, "i", {c: (add, _0, _1)}, [(a, "i"), (b, "i")]),
                 (d, "i", {d: (inc, _0)}, [(c, "i")]),
             ],
-            (d, "i", {d: (inc, c), c: (add, _0, _1)}, [(a, "i"), (b, "i")]),
+            (
+                d,
+                "i",
+                {d: (inc, _unique_dep(c, "i")), _unique_dep(c, "i"): (add, _0, _1)},
+                [(a, "i"), (b, "i")],
+            ),
         ],
         [
             [
@@ -80,7 +123,10 @@ i, j, k = "ijk"
             (
                 d,
                 "ij",
-                {d: (np.dot, b, _0), b: (np.transpose, _1)},
+                {
+                    d: (np.dot, _unique_dep(b, "ik"), _0),
+                    _unique_dep(b, "ik"): (np.transpose, _1),
+                },
                 [(c, "kj"), (a, "ki")],
             ),
         ],
@@ -93,7 +139,11 @@ i, j, k = "ijk"
             (
                 g,
                 "i",
-                {g: (add, c, f), f: (add, _2, _3), c: (add, _0, _1)},
+                {
+                    g: (add, _unique_dep(c, "i"), _unique_dep(f, "i")),
+                    _unique_dep(f, "i"): (add, _2, _3),
+                    _unique_dep(c, "i"): (add, _0, _1),
+                },
                 [(a, i), (b, i), (d, i), (e, i)],
             ),
         ],
@@ -106,7 +156,11 @@ i, j, k = "ijk"
             (
                 g,
                 "i",
-                {g: (add, c, f), f: (add, _0, _2), c: (add, _0, _1)},
+                {
+                    g: (add, _unique_dep(c, "i"), _unique_dep(f, "i")),
+                    _unique_dep(f, "i"): (add, _0, _2),
+                    _unique_dep(c, "i"): (add, _0, _1),
+                },
                 [(a, "i"), (b, "i"), (e, "i")],
             ),
         ],
@@ -115,14 +169,24 @@ i, j, k = "ijk"
                 (b, "i", {b: (sum, _0)}, [(a, "ij")]),
                 (c, "i", {c: (inc, _0)}, [(b, "i")]),
             ],
-            (c, "i", {c: (inc, b), b: (sum, _0)}, [(a, "iA")]),
+            (
+                c,
+                "i",
+                {c: (inc, _unique_dep(b, "i")), _unique_dep(b, "i"): (sum, _0)},
+                [(a, "iA")],
+            ),
         ],
         [
             [
                 (c, "i", {c: (inc, _0)}, [(b, "i")]),
                 (d, "i", {d: (add, _0, _1, _2)}, [(a, "i"), (b, "i"), (c, "i")]),
             ],
-            (d, "i", {d: (add, _0, _1, c), c: (inc, _1)}, [(a, "i"), (b, "i")]),
+            (
+                d,
+                "i",
+                {d: (add, _0, _1, _unique_dep(c, "i")), _unique_dep(c, "i"): (inc, _1)},
+                [(a, "i"), (b, "i")],
+            ),
         ],
         # Include literals
         [
@@ -137,10 +201,89 @@ i, j, k = "ijk"
             (
                 c,
                 "j",
-                {b: (add, _1, _2), c: (add, b, _0)},
+                {_unique_dep(b, "j"): (add, _1, _2), c: (add, _unique_dep(b, "j"), _0)},
                 [(456, None), (a, "j"), (123, None)],
             ),
         ],
+        # Literals that compare equal (e.g. 0 and False) aren't deduplicated
+        [
+            [
+                (b, "i", {b: (add, _0, _1)}, [(a, "i"), (0, None)]),
+                (c, "j", {c: (add, _0, _1)}, [(b, "j"), (False, None)]),
+            ],
+            (
+                c,
+                "j",
+                {_unique_dep(b, "j"): (add, _1, _2), c: (add, _unique_dep(b, "j"), _0)},
+                [(False, None), (a, "j"), (0, None)],
+            ),
+        ],
+        # Literals are deduplicated
+        [
+            [
+                (b, "i", {b: (add, _0, _1)}, [(a, "i"), (123, None)]),
+                (c, "j", {c: (add, _0, _1)}, [(b, "j"), (123, None)]),
+            ],
+            (
+                c,
+                "j",
+                {_unique_dep(b, "j"): (add, _1, _0), c: (add, _unique_dep(b, "j"), _0)},
+                [(123, None), (a, "j")],
+            ),
+        ],
+        # Check cases where two distinct indices are used
+        # for the same dependency name, and where the same
+        # dependency-index combination is repeated
+        # (See: https://github.com/dask/dask/issues/8535)
+        [
+            [
+                (b, "jk", {b: (add, _0, _1)}, [(a, "jk"), (2, None)]),
+                (c, "ijk", {c: (add, _0, _1)}, [(b, "ij"), (b, "jk")]),
+                (d, "ijk", {d: (inc, _0, _1)}, [(c, "ijk"), (123, None)]),
+            ],
+            (
+                "d",
+                "ijk",
+                {
+                    "d": (inc, _unique_dep(c, "ijk"), _0),
+                    _unique_dep(c, "ijk"): (
+                        add,
+                        _unique_dep(b, "ij"),
+                        _unique_dep(b, "jk"),
+                    ),
+                    _unique_dep(b, "ij"): (add, _1, _2),
+                    _unique_dep(b, "jk"): (add, _3, _2),
+                },
+                [(123, None), (a, "ij"), (2, None), (a, "jk")],
+            ),
+        ],
+        [
+            [
+                (b, "jk", {b: (add, _0, _1)}, [(a, "jk"), (2, None)]),
+                (c, "ijk", {c: (add, _0, _1)}, [(b, "ij"), (b, "jk")]),
+                (d, "ijk", {d: (add, _0, _1, _2)}, [(b, "ij"), (c, "ij"), (b, "ij")]),
+            ],
+            (
+                "d",
+                "ijk",
+                {
+                    "d": (
+                        add,
+                        _unique_dep(b, "ij"),
+                        _unique_dep(c, "ij"),
+                        _unique_dep(b, "ij"),
+                    ),
+                    _unique_dep(c, "ij"): (
+                        add,
+                        _unique_dep(b, "ij"),
+                        _unique_dep(b, "jk"),
+                    ),
+                    _unique_dep(b, "ij"): (add, _0, _1),
+                    _unique_dep(b, "jk"): (add, _2, _1),
+                },
+                [(a, "ij"), (2, None), (a, "jk")],
+            ),
+        ],
     ],
 )
 def test_rewrite(inputs, expected):
@@ -176,11 +319,42 @@ def test_optimize_blockwise():
     assert isinstance(dsk, HighLevelGraph)
 
     assert (
-        len([layer for layer in dsk.dicts.values() if isinstance(layer, Blockwise)])
+        len([layer for layer in dsk.layers.values() if isinstance(layer, Blockwise)])
         == 1
     )
 
 
+def test_optimize_blockwise_annotations():
+    a = da.ones(10, chunks=(5,))
+    b = a + 1
+
+    with dask.annotate(qux="foo"):
+        c = b + 2
+        d = c + 3
+
+    with dask.annotate(qux="baz"):
+        e = d + 4
+        f = e + 5
+
+    g = f + 6
+
+    dsk = da.optimization.optimize_blockwise(g.dask)
+
+    annotations = (
+        layer.annotations
+        for layer in dsk.layers.values()
+        if isinstance(layer, Blockwise)
+    )
+    annotations = collections.Counter(
+        tuple(a.items()) if type(a) is dict else a for a in annotations
+    )
+
+    assert len(annotations) == 3
+    assert annotations[None] == 2
+    assert annotations[(("qux", "baz"),)] == 1
+    assert annotations[(("qux", "foo"),)] == 1
+
+
 def test_blockwise_diamond_fusion():
     x = da.ones(10, chunks=(5,))
     y = ((x + 1) + 2) + 3
@@ -193,7 +367,7 @@ def test_blockwise_diamond_fusion():
     assert isinstance(dsk, HighLevelGraph)
 
     assert (
-        len([layer for layer in dsk.dicts.values() if isinstance(layer, Blockwise)])
+        len([layer for layer in dsk.layers.values() if isinstance(layer, Blockwise)])
         == 1
     )
 
@@ -204,15 +378,15 @@ def test_blockwise_non_blockwise_output():
     w = y.sum()
     z = ((y * 2) * 3) * 4
 
-    z_top_before = tuple(z.dask.dicts[z.name].indices)
+    z_top_before = tuple(z.dask.layers[z.name].indices)
     (zz,) = dask.optimize(z)
-    z_top_after = tuple(z.dask.dicts[z.name].indices)
+    z_top_after = tuple(z.dask.layers[z.name].indices)
     assert z_top_before == z_top_after, "z_top mutated"
 
     dsk = optimize_blockwise(z.dask, keys=list(dask.core.flatten(z.__dask_keys__())))
     assert isinstance(dsk, HighLevelGraph)
     assert (
-        len([layer for layer in dsk.dicts.values() if isinstance(layer, Blockwise)])
+        len([layer for layer in dsk.layers.values() if isinstance(layer, Blockwise)])
         == 1
     )
 
@@ -222,7 +396,7 @@ def test_blockwise_non_blockwise_output():
     )
     assert isinstance(dsk, HighLevelGraph)
     assert (
-        len([layer for layer in z.dask.dicts.values() if isinstance(layer, Blockwise)])
+        len([layer for layer in z.dask.layers.values() if isinstance(layer, Blockwise)])
         >= 1
     )
 
@@ -231,7 +405,7 @@ def test_top_len():
     x = da.ones(10, chunks=(5,))
     y = x[:, None] * x[None, :]
 
-    d = y.dask.dicts[y.name]
+    d = y.dask.layers[y.name]
     assert len(d) == 4
 
 
@@ -467,23 +641,16 @@ def test_blockwise_chunks():
 
 
 def test_blockwise_numpy_arg():
-    with warnings.catch_warnings():
-        if not _numpy_116:
-            # Not sure why, but this DeprecationWarning is no longer
-            # showing up for NumPy >=1.16. So we only filter here
-            # for 1.15 and earlier
-            warnings.simplefilter("ignore", DeprecationWarning)
+    x = da.arange(10, chunks=(5,))
+    y = np.arange(1000)
 
-        x = da.arange(10, chunks=(5,))
-        y = np.arange(1000)
-
-        x = x.map_blocks(lambda x, y: x, 1.0)
-        x = x.map_blocks(lambda x, y: x, "abc")
-        x = x.map_blocks(lambda x, y: x, y)
-        x = x.map_blocks(lambda x, y: x, "abc")
-        x = x.map_blocks(lambda x, y: x, 1.0)
-        x = x.map_blocks(lambda x, y, z: x, "abc", np.array(["a", "b"], dtype=object))
-        assert_eq(x, np.arange(10))
+    x = x.map_blocks(lambda x, y: x, 1.0)
+    x = x.map_blocks(lambda x, y: x, "abc")
+    x = x.map_blocks(lambda x, y: x, y)
+    x = x.map_blocks(lambda x, y: x, "abc")
+    x = x.map_blocks(lambda x, y: x, 1.0)
+    x = x.map_blocks(lambda x, y, z: x, "abc", np.array(["a", "b"], dtype=object))
+    assert_eq(x, np.arange(10))
 
 
 def test_bag_array_conversion():
@@ -491,7 +658,7 @@ def test_bag_array_conversion():
 
     b = db.range(10, npartitions=1)
     (x,) = b.map_partitions(np.asarray).to_delayed()
-    (x,) = [da.from_delayed(a, shape=(10,), dtype=int) for a in [x]]
+    (x,) = (da.from_delayed(a, shape=(10,), dtype=int) for a in [x])
     z = da.concatenate([x])
     assert_eq(z, np.arange(10), check_graph=False)
 
@@ -554,7 +721,7 @@ def test_dont_merge_before_reductions():
 
     dsk = optimize_blockwise(w.dask)
 
-    assert len([d for d in dsk.dicts.values() if isinstance(d, Blockwise)]) == 2
+    assert len([d for d in dsk.layers.values() if isinstance(d, Blockwise)]) == 2
 
     z.compute()
 
@@ -566,3 +733,12 @@ def test_atop_legacy():
     z = da.blockwise(inc, "i", x, "i", dtype=x.dtype)
     assert_eq(y, z)
     assert y.name == z.name
+
+
+def test_non_hlg():
+    # Regression test for https://github.com/dask/dask/issues/5850
+    a = da.from_array(np.ones(1, np.float64), chunks=(1,))
+    a.dask = dict(a.dask)  # Convert from HighLevelGraph to plain dict
+    b = da.from_array(np.zeros(1, np.float64), chunks=(1,))
+    x = a + b
+    assert_eq(x, a)
