@@ -1,3 +1,5 @@
+import sys
+
 import pytest
 
 pytest.importorskip("numpy")
@@ -5,9 +7,11 @@ pytest.importorskip("scipy")
 
 import numpy as np
 import scipy.linalg
+from packaging.version import parse as parse_version
 
 import dask.array as da
-from dask.array.linalg import tsqr, sfqr, svd_compressed, qr, svd
+from dask.array.linalg import qr, sfqr, svd, svd_compressed, tsqr
+from dask.array.numpy_compat import _np_version
 from dask.array.utils import assert_eq, same_keys, svd_flip
 
 
@@ -807,11 +811,17 @@ def test_cholesky(shape, chunk):
 
     A = _get_symmat(shape)
     dA = da.from_array(A, (chunk, chunk))
-    assert_eq(da.linalg.cholesky(dA), scipy.linalg.cholesky(A), check_graph=False)
+    assert_eq(
+        da.linalg.cholesky(dA).compute(),
+        scipy.linalg.cholesky(A),
+        check_graph=False,
+        check_chunks=False,
+    )
     assert_eq(
         da.linalg.cholesky(dA, lower=True),
         scipy.linalg.cholesky(A, lower=True),
         check_graph=False,
+        check_chunks=False,
     )
 
 
@@ -970,6 +980,11 @@ def test_svd_incompatible_dimensions(ndim):
         da.linalg.svd(x)
 
 
+@pytest.mark.xfail(
+    sys.platform == "darwin" and _np_version < parse_version("1.22"),
+    reason="https://github.com/dask/dask/issues/7189",
+    strict=False,
+)
 @pytest.mark.parametrize(
     "shape, chunks, axis",
     [[(5,), (2,), None], [(5,), (2,), 0], [(5,), (2,), (0,)], [(5, 6), (2, 2), None]],
@@ -987,6 +1002,11 @@ def test_norm_any_ndim(shape, chunks, axis, norm, keepdims):
 
 
 @pytest.mark.slow
+@pytest.mark.xfail(
+    sys.platform == "darwin" and _np_version < parse_version("1.22"),
+    reason="https://github.com/dask/dask/issues/7189",
+    strict=False,
+)
 @pytest.mark.parametrize(
     "shape, chunks",
     [
