@@ -3,14 +3,21 @@ import pytest
 pytest.importorskip("numpy")
 
 import numpy as np
+
 import dask
 import dask.array as da
+from dask.array.core import getter, getter_nofancy
+from dask.array.optimization import (
+    fuse_slice,
+    getitem,
+    optimize,
+    optimize_blockwise,
+    optimize_slices,
+)
+from dask.array.utils import assert_eq
 from dask.highlevelgraph import HighLevelGraph
 from dask.optimization import fuse
 from dask.utils import SerializableLock
-from dask.array.core import getter, getter_nofancy
-from dask.array.optimization import getitem, optimize, optimize_slices, fuse_slice
-from dask.array.utils import assert_eq
 
 
 def test_fuse_getitem():
@@ -293,7 +300,7 @@ def test_fuse_slices_with_alias():
     keys = [("dx2", 0)]
     dsk2 = optimize(dsk, keys)
     assert len(dsk2) == 3
-    fused_key = set(dsk2).difference(["x", ("dx2", 0)]).pop()
+    fused_key = (dsk2.keys() - {"x", ("dx2", 0)}).pop()
     assert dsk2[fused_key] == (getter, "x", (slice(0, 4), 0))
 
 
@@ -388,6 +395,22 @@ def test_disable_lowlevel_fusion():
         assert_eq(y, [1] * 3)
 
 
+def test_array_creation_blockwise_fusion():
+    """
+    Check that certain array creation routines work with blockwise and can be
+    fused with other blockwise operations.
+    """
+    x = da.ones(3, chunks=(3,))
+    y = da.zeros(3, chunks=(3,))
+    z = da.full(3, fill_value=2, chunks=(3,))
+    a = x + y + z
+    dsk1 = a.__dask_graph__()
+    assert len(dsk1) == 5
+    dsk2 = optimize_blockwise(dsk1)
+    assert len(dsk2) == 1
+    assert_eq(a, np.full(3, 3.0))
+
+
 def test_gh3937():
     # test for github issue #3937
     x = da.from_array([1, 2, 3.0], (2,))
@@ -428,7 +451,20 @@ def test_fuse_roots_annotations():
 
     z = (x + 1) + (2 * y)
     hlg = dask.blockwise.optimize_blockwise(z.dask)
-    assert len(hlg.layers) == 4
+    assert len(hlg.layers) == 3
     assert {"foo": "bar"} in [l.annotations for l in hlg.layers.values()]
     za = da.Array(hlg, z.name, z.chunks, z.dtype)
     assert_eq(za, z)
+
+
+@pytest.mark.parametrize("optimize_graph", [True, False])
+def test_optimize_blockwise_duplicate_dependency(optimize_graph):
+    # Two blockwise operations in a row with duplicate name
+    # (See: https://github.com/dask/dask/issues/8535)
+    xx = da.from_array(np.array([[1, 1], [2, 2]]), chunks=1)
+    xx = xx * 2
+    z = da.matmul(xx, xx)
+
+    # Compare to known answer
+    result = z.compute(optimize_graph=optimize_graph)
+    assert assert_eq(result, [[12, 12], [24, 24]])
