@@ -1,5 +1,6 @@
 import os
 import warnings
+from itertools import zip_longest
 
 import pytest
 
@@ -8,9 +9,10 @@ np = pytest.importorskip("numpy")
 import itertools
 
 import dask.array as da
-from dask.array.utils import assert_eq as _assert_eq, same_keys
-from dask.core import get_deps
 import dask.config as config
+from dask.array.utils import assert_eq as _assert_eq
+from dask.array.utils import same_keys
+from dask.core import get_deps
 
 
 def assert_eq(a, b):
@@ -47,8 +49,43 @@ def test_numel(dtype, keepdims):
             )
 
 
+def reduction_0d_test(da_func, darr, np_func, narr):
+    expected = np_func(narr)
+    actual = da_func(darr)
+
+    assert_eq(actual, expected)
+    assert_eq(da_func(narr), expected)  # Ensure Dask reductions work with NumPy arrays
+    assert actual.size == 1
+
+
+def test_reductions_0D():
+    x = np.int_(3)  # np.int_ has a dtype attribute, np.int does not.
+    a = da.from_array(x, chunks=(1,))
+
+    reduction_0d_test(da.sum, a, np.sum, x)
+    reduction_0d_test(da.prod, a, np.prod, x)
+    reduction_0d_test(da.mean, a, np.mean, x)
+    reduction_0d_test(da.var, a, np.var, x)
+    reduction_0d_test(da.std, a, np.std, x)
+    reduction_0d_test(da.min, a, np.min, x)
+    reduction_0d_test(da.max, a, np.max, x)
+    reduction_0d_test(da.any, a, np.any, x)
+    reduction_0d_test(da.all, a, np.all, x)
+
+    reduction_0d_test(da.nansum, a, np.nansum, x)
+    reduction_0d_test(da.nanprod, a, np.nanprod, x)
+    reduction_0d_test(da.nanmean, a, np.mean, x)
+    reduction_0d_test(da.nanvar, a, np.var, x)
+    reduction_0d_test(da.nanstd, a, np.std, x)
+    reduction_0d_test(da.nanmin, a, np.nanmin, x)
+    reduction_0d_test(da.nanmax, a, np.nanmax, x)
+
+
 def reduction_1d_test(da_func, darr, np_func, narr, use_dtype=True, split_every=True):
     assert_eq(da_func(darr), np_func(narr))
+    assert_eq(
+        da_func(narr), np_func(narr)
+    )  # Ensure Dask reductions work with NumPy arrays
     assert_eq(da_func(darr, keepdims=True), np_func(narr, keepdims=True))
     assert_eq(da_func(darr, axis=()), np_func(narr, axis=()))
     assert same_keys(da_func(darr), da_func(darr))
@@ -413,7 +450,8 @@ def test_nan_object(func):
             warnings.simplefilter("default", RuntimeWarning)
 
         assert_eq(getattr(np, func)(x, axis=1), getattr(da, func)(d, axis=1))
-        assert_eq(getattr(np, func)(x), getattr(da, func)(d))
+        # wrap the scalar in a numpy array since the dask version cannot know dtype
+        assert_eq(np.array(getattr(np, func)(x)).astype(object), getattr(da, func)(d))
 
 
 def test_0d_array():
@@ -510,7 +548,16 @@ def test_reduction_names():
     assert x.mean().name.startswith("mean")
 
 
-@pytest.mark.filterwarnings("ignore:`argmax` is not implemented by dask")
+def test_general_reduction_names():
+    dtype = int
+    a = da.reduction(
+        da.ones(10, dtype, chunks=2), np.sum, np.sum, dtype=dtype, name="foo"
+    )
+    names, tokens = list(zip_longest(*[key[0].rsplit("-", 1) for key in a.dask]))
+    assert set(names) == {"ones_like", "foo", "foo-partial", "foo-aggregate"}
+    assert all(tokens)
+
+
 @pytest.mark.parametrize("func", [np.sum, np.argmax])
 def test_array_reduction_out(func):
     x = da.arange(10, chunks=(5,))
@@ -522,18 +569,23 @@ def test_array_reduction_out(func):
 @pytest.mark.parametrize("func", ["cumsum", "cumprod", "nancumsum", "nancumprod"])
 @pytest.mark.parametrize("use_nan", [False, True])
 @pytest.mark.parametrize("axis", [None, 0, 1, -1])
-def test_array_cumreduction_axis(func, use_nan, axis):
+@pytest.mark.parametrize("method", ["sequential", "blelloch"])
+def test_array_cumreduction_axis(func, use_nan, axis, method):
     np_func = getattr(np, func)
     da_func = getattr(da, func)
 
     s = (10, 11, 12)
-    a = np.arange(np.prod(s)).reshape(s)
+    a = np.arange(np.prod(s), dtype=float).reshape(s)
     if use_nan:
         a[1] = np.nan
     d = da.from_array(a, chunks=(4, 5, 6))
+    if func in ["cumprod", "nancumprod"] and method == "blelloch" and axis is None:
+        with pytest.warns(RuntimeWarning):
+            da_func(d, axis=axis, method=method).compute()
+            return
 
     a_r = np_func(a, axis=axis)
-    d_r = da_func(d, axis=axis)
+    d_r = da_func(d, axis=axis, method=method)
 
     assert_eq(a_r, d_r)
 
@@ -630,11 +682,16 @@ def test_topk_argtopk3():
     "func",
     [da.cumsum, da.cumprod, da.argmin, da.argmax, da.min, da.max, da.nansum, da.nanmax],
 )
-def test_regres_3940(func):
+@pytest.mark.parametrize("method", ["sequential", "blelloch"])
+def test_regres_3940(func, method):
+    if func in {da.cumsum, da.cumprod}:
+        kwargs = {"method": method}
+    else:
+        kwargs = {}
     a = da.ones((5, 2), chunks=(2, 2))
-    assert func(a).name != func(a + 1).name
-    assert func(a, axis=0).name != func(a).name
-    assert func(a, axis=0).name != func(a, axis=1).name
+    assert func(a, **kwargs).name != func(a + 1, **kwargs).name
+    assert func(a, axis=0, **kwargs).name != func(a, **kwargs).name
+    assert func(a, axis=0, **kwargs).name != func(a, axis=1, **kwargs).name
     if func not in {da.cumsum, da.cumprod, da.argmin, da.argmax}:
         assert func(a, axis=()).name != func(a).name
         assert func(a, axis=()).name != func(a, axis=0).name
@@ -665,13 +722,90 @@ def test_trace():
     _assert(a, b, offset=1, axis1=0, axis2=2, dtype=float)
 
 
+@pytest.mark.parametrize("func", ["median", "nanmedian"])
 @pytest.mark.parametrize("axis", [0, [0, 1], 1, -1])
 @pytest.mark.parametrize("keepdims", [True, False])
-def test_median(axis, keepdims):
+def test_median(axis, keepdims, func):
     x = np.arange(100).reshape((2, 5, 10))
     d = da.from_array(x, chunks=2)
-
     assert_eq(
-        da.median(d, axis=axis, keepdims=keepdims),
-        np.median(x, axis=axis, keepdims=keepdims),
+        getattr(da, func)(d, axis=axis, keepdims=keepdims),
+        getattr(np, func)(x, axis=axis, keepdims=keepdims),
     )
+
+
+@pytest.mark.parametrize("func", ["median", "nanmedian"])
+@pytest.mark.parametrize("axis", [0, [0, 2], 1])
+def test_median_does_not_rechunk_if_whole_axis_in_one_chunk(axis, func):
+    x = np.arange(100).reshape((2, 5, 10))
+    d = da.from_array(x, chunks=(2, 1, 10))
+
+    actual = getattr(da, func)(d, axis=axis)
+    expected = getattr(np, func)(x, axis=axis)
+    assert_eq(actual, expected)
+    does_rechunk = "rechunk" in str(dict(actual.__dask_graph__()))
+    if axis == 1:
+        assert does_rechunk
+    else:
+        assert not does_rechunk
+
+
+@pytest.mark.parametrize("method", ["sum", "mean", "prod"])
+def test_object_reduction(method):
+    arr = da.ones(1).astype(object)
+    result = getattr(arr, method)().compute()
+    assert result == 1
+
+
+@pytest.mark.parametrize("func", ["nanmin", "nanmax"])
+def test_empty_chunk_nanmin_nanmax(func):
+    # see https://github.com/dask/dask/issues/8352
+    x = np.arange(10).reshape(2, 5)
+    d = da.from_array(x, chunks=2)
+    x = x[x > 4]
+    d = d[d > 4]
+    block_lens = np.array([len(x.compute()) for x in d.blocks])
+    assert 0 in block_lens
+    with pytest.raises(ValueError) as err:
+        getattr(da, func)(d)
+    assert "Arrays chunk sizes are unknown" in str(err)
+    d = d.compute_chunk_sizes()
+    assert_eq(getattr(da, func)(d), getattr(np, func)(x))
+
+
+@pytest.mark.parametrize("func", ["nanmin", "nanmax"])
+def test_empty_chunk_nanmin_nanmax_raise(func):
+    # see https://github.com/dask/dask/issues/8352
+    x = np.arange(10).reshape(2, 5)
+    d = da.from_array(x, chunks=2)
+    d = d[d > 9]
+    x = x[x > 9]
+    d = d.compute_chunk_sizes()
+    with pytest.raises(ValueError) as err_np:
+        getattr(np, func)(x)
+    with pytest.raises(ValueError) as err_da:
+        d = getattr(da, func)(d)
+        d.compute()
+    assert str(err_np.value) == str(err_da.value)
+
+
+def test_mean_func_does_not_warn():
+    # non-regression test for https://github.com/pydata/xarray/issues/5151
+    xr = pytest.importorskip("xarray")
+    a = xr.DataArray(da.from_array(np.full((10, 10), np.nan)))
+
+    with pytest.warns(None) as rec:
+        a.mean().compute()
+    assert not rec  # did not warn
+
+
+@pytest.mark.parametrize("func", ["nanvar", "nanstd"])
+def test_nan_func_does_not_warn(func):
+    # non-regression test for #6105
+    x = np.ones((10,)) * np.nan
+    x[0] = 1
+    x[1] = 2
+    d = da.from_array(x, chunks=2)
+    with pytest.warns(None) as rec:
+        getattr(da, func)(d).compute()
+    assert not rec  # did not warn
