@@ -11,16 +11,25 @@ from dask.array.ufunc import da_frompyfunc
 from dask.array.utils import assert_eq
 from dask.base import tokenize
 
+DISCLAIMER = """
+This docstring was copied from numpy.{name}.
 
-def test_ufunc_meta():
-    assert da.log.__name__ == "log"
-    assert da.log.__doc__.replace("  # doctest: +SKIP", "") == np.log.__doc__
+Some inconsistencies with the Dask version may exist.
+"""
 
-    assert da.modf.__name__ == "modf"
-    assert da.modf.__doc__.replace("  # doctest: +SKIP", "") == np.modf.__doc__
 
-    assert da.frexp.__name__ == "frexp"
-    assert da.frexp.__doc__.replace("  # doctest: +SKIP", "") == np.frexp.__doc__
+@pytest.mark.parametrize("name", ["log", "modf", "frexp"])
+def test_ufunc_meta(name):
+    disclaimer = DISCLAIMER.format(name=name)
+    skip_test = "  # doctest: +SKIP"
+    ufunc = getattr(da, name)
+    assert ufunc.__name__ == name
+    assert disclaimer in ufunc.__doc__
+
+    assert (
+        ufunc.__doc__.replace(disclaimer, "").replace(skip_test, "")
+        == getattr(np, name).__doc__
+    )
 
 
 def test_ufunc():
@@ -465,3 +474,106 @@ def test_divmod():
     expected = divmod(arr1, arr2)
     assert_eq(result[0], expected[0])
     assert_eq(result[1], expected[1])
+
+
+@pytest.mark.parametrize("dt", ["float64", "float32", "int32", "int64"])
+def test_dtype_kwarg(dt):
+    arr1 = np.array([1, 2, 3])
+    arr2 = np.array([4, 5, 6])
+
+    darr1 = da.from_array(arr1)
+    darr2 = da.from_array(arr2)
+
+    expected = np.add(arr1, arr2, dtype=dt)
+    result = np.add(darr1, darr2, dtype=dt)
+    assert_eq(expected, result)
+
+    result = da.add(darr1, darr2, dtype=dt)
+    assert_eq(expected, result)
+
+
+@pytest.mark.parametrize("dtype", [None, "f8"])
+@pytest.mark.parametrize("left_is_da", [False, True])
+@pytest.mark.parametrize("right_is_da", [False, True])
+@pytest.mark.parametrize("where_kind", [True, False, "numpy", "dask"])
+def test_ufunc_where(dtype, left_is_da, right_is_da, where_kind):
+    left = np.arange(12).reshape((3, 4))
+    right = np.arange(4)
+    out = np.zeros_like(left, dtype=dtype)
+    d_out = da.zeros_like(left, dtype=dtype)
+
+    if where_kind in (True, False):
+        d_where = where = where_kind
+    else:
+        d_where = where = np.array([False, True, True, False])
+        if where_kind == "dask":
+            d_where = da.from_array(where, chunks=2)
+
+    d_left = da.from_array(left, chunks=2) if left_is_da else left
+    d_right = da.from_array(right, chunks=2) if right_is_da else right
+
+    expected = np.add(left, right, where=where, out=out, dtype=dtype)
+    result = da.add(d_left, d_right, where=d_where, out=d_out, dtype=dtype)
+    assert result is d_out
+    assert_eq(expected, result)
+
+
+@pytest.mark.parametrize("left_is_da", [False, True])
+@pytest.mark.parametrize("right_is_da", [False, True])
+@pytest.mark.parametrize("where_is_da", [False, True])
+def test_ufunc_where_broadcasts(left_is_da, right_is_da, where_is_da):
+    left = np.arange(4)
+    right = np.arange(4, 8)
+    where = np.array([[0, 1, 1, 0], [1, 0, 0, 1], [0, 1, 0, 1]]).astype("bool")
+    out = np.zeros(where.shape, dtype=left.dtype)
+
+    d_out = da.zeros(where.shape, dtype=left.dtype)
+    d_where = da.from_array(where, chunks=2) if where_is_da else where
+    d_left = da.from_array(left, chunks=2) if left_is_da else left
+    d_right = da.from_array(right, chunks=2) if right_is_da else right
+
+    expected = np.add(left, right, where=where, out=out)
+    result = da.add(d_left, d_right, where=d_where, out=d_out)
+    assert result is d_out
+    assert_eq(expected, result)
+
+
+def test_ufunc_where_no_out():
+    left = np.arange(4)
+    right = np.arange(4, 8)
+    where = np.array([[0, 1, 1, 0], [1, 0, 0, 1], [0, 1, 0, 1]]).astype("bool")
+
+    d_where = da.from_array(where, chunks=2)
+    d_left = da.from_array(left, chunks=2)
+    d_right = da.from_array(right, chunks=2)
+
+    expected = np.add(left, right, where=where)
+    result = da.add(d_left, d_right, where=d_where)
+
+    # If no `out` is provided, numpy leaves elements that don't match `where`
+    # uninitialized, so they effectively may be any random value.  We test that
+    # the set values match, and that the unset values aren't equal to if
+    # `where` wasn't provided (to test that `where` was actually passed).
+
+    expected_masked = np.where(where, expected, 0)
+    result_masked = np.where(where, expected, 0)
+    assert_eq(expected_masked, result_masked)
+
+    expected_no_where = np.add(left, right)
+    assert not np.equal(result.compute(), expected_no_where).all()
+
+
+def test_ufunc_where_doesnt_mutate_out():
+    """Dask array's are immutable, ensure that the backing numpy array for
+    `out` isn't actually mutated"""
+    left = da.from_array(np.arange(4, dtype="i8"), chunks=2)
+    right = da.from_array(np.arange(4, 8, dtype="i8"), chunks=2)
+    where = da.from_array(np.array([1, 0, 0, 1], dtype="bool"), chunks=2)
+    out_np = np.zeros(4, dtype="i8")
+    out = da.from_array(out_np, chunks=2)
+    result = da.add(left, right, where=where, out=out)
+    assert out is result
+    assert_eq(out, np.array([4, 0, 0, 10], dtype="i8"))
+
+    # Check that original `out` array isn't mutated
+    assert np.equal(out_np, 0).all()
