@@ -1,36 +1,27 @@
-from operator import getitem
 from functools import partial
+from operator import getitem
 
 import numpy as np
-from toolz import curry
 
-from .core import Array, elemwise, blockwise, apply_infer_dtype, asarray
-from .utils import empty_like_safe, IS_NEP18_ACTIVE
-from ..base import is_dask_collection, normalize_function
 from .. import core
+from ..base import is_dask_collection, normalize_function
 from ..highlevelgraph import HighLevelGraph
 from ..utils import (
-    skip_doctest,
-    funcname,
     derived_from,
+    funcname,
     is_dataframe_like,
-    is_series_like,
     is_index_like,
+    is_series_like,
 )
+from .core import Array, apply_infer_dtype, asarray, blockwise, elemwise
 
 
 def __array_wrap__(numpy_ufunc, x, *args, **kwargs):
     return x.__array_wrap__(numpy_ufunc(x, *args, **kwargs))
 
 
-@curry
-def copy_docstring(target, source=None):
-    target.__doc__ = skip_doctest(source.__doc__)
-    return target
-
-
-def wrap_elemwise(numpy_ufunc, array_wrap=False):
-    """ Wrap up numpy function into dask.array """
+def wrap_elemwise(numpy_ufunc, array_wrap=False, source=np):
+    """Wrap up numpy function into dask.array"""
 
     def wrapped(*args, **kwargs):
         dsk = [arg for arg in args if hasattr(arg, "_elemwise")]
@@ -40,7 +31,7 @@ def wrap_elemwise(numpy_ufunc, array_wrap=False):
                 or is_series_like(dsk[0])
                 or is_index_like(dsk[0])
             )
-            if array_wrap and (is_dataframe or not IS_NEP18_ACTIVE):
+            if array_wrap and is_dataframe:
                 return dsk[0]._elemwise(__array_wrap__, numpy_ufunc, *args, **kwargs)
             else:
                 return dsk[0]._elemwise(numpy_ufunc, *args, **kwargs)
@@ -49,11 +40,10 @@ def wrap_elemwise(numpy_ufunc, array_wrap=False):
 
     # functools.wraps cannot wrap ufunc in Python 2.x
     wrapped.__name__ = numpy_ufunc.__name__
-    wrapped.__doc__ = skip_doctest(numpy_ufunc.__doc__)
-    return wrapped
+    return derived_from(source)(wrapped)
 
 
-class da_frompyfunc(object):
+class da_frompyfunc:
     """A serializable `frompyfunc` object"""
 
     def __init__(self, func, nin, nout):
@@ -79,7 +69,7 @@ class da_frompyfunc(object):
     def __getattr__(self, a):
         if not a.startswith("_"):
             return getattr(self._ufunc, a)
-        raise AttributeError("%r object has no attribute %r" % (type(self).__name__, a))
+        raise AttributeError(f"{type(self).__name__!r} object has no attribute {a!r}")
 
     def __dir__(self):
         o = set(dir(type(self)))
@@ -95,7 +85,7 @@ def frompyfunc(func, nin, nout):
     return ufunc(da_frompyfunc(func, nin, nout))
 
 
-class ufunc(object):
+class ufunc:
     _forward_attrs = {
         "nin",
         "nargs",
@@ -114,14 +104,13 @@ class ufunc(object):
             )
         self._ufunc = ufunc
         self.__name__ = ufunc.__name__
-        copy_docstring(self, ufunc)
+        if isinstance(ufunc, np.ufunc):
+            derived_from(np)(self)
 
     def __getattr__(self, key):
         if key in self._forward_attrs:
             return getattr(self._ufunc, key)
-        raise AttributeError(
-            "%r object has no attribute %r" % (type(self).__name__, key)
-        )
+        raise AttributeError(f"{type(self).__name__!r} object has no attribute {key!r}")
 
     def __dir__(self):
         return list(self._forward_attrs.union(dir(type(self)), self.__dict__))
@@ -142,7 +131,7 @@ class ufunc(object):
         else:
             return self._ufunc(*args, **kwargs)
 
-    @copy_docstring(source=np.ufunc.outer)
+    @derived_from(np.ufunc)
     def outer(self, A, B, **kwargs):
         if self.nin != 2:
             raise ValueError("outer product only supported for binary functions")
@@ -189,7 +178,7 @@ class ufunc(object):
             B_inds,
             dtype=dtype,
             token=self.__name__ + ".outer",
-            **kwargs
+            **kwargs,
         )
 
 
@@ -304,7 +293,7 @@ sinc = wrap_elemwise(np.sinc, array_wrap=True)
 nan_to_num = wrap_elemwise(np.nan_to_num, array_wrap=True)
 
 
-@copy_docstring(source=np.angle)
+@derived_from(np)
 def angle(x, deg=0):
     deg = bool(deg)
     if hasattr(x, "_elemwise"):
@@ -312,7 +301,7 @@ def angle(x, deg=0):
     return np.angle(x, deg=deg)
 
 
-@copy_docstring(source=np.frexp)
+@derived_from(np)
 def frexp(x):
     # Not actually object dtype, just need to specify something
     tmp = elemwise(np.frexp, x, dtype=object)
@@ -327,7 +316,7 @@ def frexp(x):
         for key in core.flatten(tmp.__dask_keys__())
     }
 
-    a = empty_like_safe(getattr(x, "_meta", x), shape=(1,) * x.ndim, dtype=x.dtype)
+    a = np.empty_like(getattr(x, "_meta", x), shape=(1,) * x.ndim, dtype=x.dtype)
     l, r = np.frexp(a)
 
     graph = HighLevelGraph.from_collections(left, ldsk, dependencies=[tmp])
@@ -337,7 +326,7 @@ def frexp(x):
     return L, R
 
 
-@copy_docstring(source=np.modf)
+@derived_from(np)
 def modf(x):
     # Not actually object dtype, just need to specify something
     tmp = elemwise(np.modf, x, dtype=object)
@@ -352,7 +341,7 @@ def modf(x):
         for key in core.flatten(tmp.__dask_keys__())
     }
 
-    a = empty_like_safe(getattr(x, "_meta", x), shape=(1,) * x.ndim, dtype=x.dtype)
+    a = np.empty_like(getattr(x, "_meta", x), shape=(1,) * x.ndim, dtype=x.dtype)
     l, r = np.modf(a)
 
     graph = HighLevelGraph.from_collections(left, ldsk, dependencies=[tmp])
@@ -362,7 +351,7 @@ def modf(x):
     return L, R
 
 
-@copy_docstring(source=np.divmod)
+@derived_from(np)
 def divmod(x, y):
     res1 = x // y
     res2 = x % y
