@@ -1,17 +1,22 @@
 import os
-import pytest
 import subprocess
 import sys
 import time
-import fsspec
-from distutils.version import LooseVersion
 
-from dask.bytes.core import open_files
-from dask.bytes._compatibility import FSSPEC_042
+import fsspec
+import pytest
+from fsspec.core import open_files
+from packaging.version import parse as parse_version
+
+import dask.bag as db
 from dask.utils import tmpdir
 
 files = ["a", "b"]
 requests = pytest.importorskip("requests")
+errs = requests.exceptions.RequestException, FileNotFoundError
+if parse_version(fsspec.__version__) > parse_version("0.7.4"):
+    aiohttp = pytest.importorskip("aiohttp")
+    errs = errs + (aiohttp.client_exceptions.ClientResponseError,)
 
 
 @pytest.fixture(scope="module")
@@ -28,11 +33,11 @@ def dir_server():
             try:
                 requests.get("http://localhost:8999")
                 break
-            except requests.exceptions.ConnectionError:
+            except requests.exceptions.ConnectionError as e:
                 time.sleep(0.1)
                 timeout -= 0.1
                 if timeout < 0:
-                    raise RuntimeError("Server did not appear")
+                    raise RuntimeError("Server did not appear") from e
         yield d
         p.terminate()
 
@@ -43,23 +48,25 @@ def test_simple(dir_server):
     f = open_files(root + fn)[0]
     with f as f:
         data = f.read()
-    assert data == open(os.path.join(dir_server, fn), "rb").read()
+    with open(os.path.join(dir_server, fn), "rb") as expected:
+        assert data == expected.read()
 
 
 def test_loc(dir_server):
     root = "http://localhost:8999/"
     fn = files[0]
     f = open_files(root + fn)[0]
-    expected = open(os.path.join(dir_server, fn), "rb").read()
-    with f as f:
-        data = f.read(2)
-        assert data == expected[:2]
-        assert f.loc == 2
-        f.seek(0)
-        data = f.read(3)
-        assert data == expected[:3]
-        f.seek(1, 1)
-        assert f.loc == 4
+    with open(os.path.join(dir_server, fn), "rb") as expected:
+        expected = expected.read()
+        with f as f:
+            data = f.read(2)
+            assert data == expected[:2]
+            assert f.loc == 2
+            f.seek(0)
+            data = f.read(3)
+            assert data == expected[:3]
+            f.seek(1, 1)
+            assert f.loc == 4
 
 
 def test_fetch_range_with_headers(dir_server):
@@ -70,7 +77,8 @@ def test_fetch_range_with_headers(dir_server):
     f = open_files(root + fn, headers=headers)[0]
     with f as f:
         data = f.read(length=1) + f.read(length=-1)
-    assert data == open(os.path.join(dir_server, fn), "rb").read()
+    with open(os.path.join(dir_server, fn), "rb") as expected:
+        assert data == expected.read()
 
 
 @pytest.mark.parametrize("block_size", [None, 99999])
@@ -78,49 +86,52 @@ def test_ops(dir_server, block_size):
     root = "http://localhost:8999/"
     fn = files[0]
     f = open_files(root + fn)[0]
-    data = open(os.path.join(dir_server, fn), "rb").read()
-    with f as f:
-        # these pass because the default
-        assert f.read(10) == data[:10]
-        f.seek(0)
-        assert f.read(10) == data[:10]
-        assert f.read(10) == data[10:20]
-        f.seek(-10, 2)
-        assert f.read() == data[-10:]
+    with open(os.path.join(dir_server, fn), "rb") as expected:
+        expected = expected.read()
+        with f as f:
+            # these pass because the default
+            assert f.read(10) == expected[:10]
+            f.seek(0)
+            assert f.read(10) == expected[:10]
+            assert f.read(10) == expected[10:20]
+            f.seek(-10, 2)
+            assert f.read() == expected[-10:]
 
 
 def test_ops_blocksize(dir_server):
     root = "http://localhost:8999/"
     fn = files[0]
     f = open_files(root + fn, block_size=2)[0]
-    data = open(os.path.join(dir_server, fn), "rb").read()
-    with f as f:
-        # it's OK to read the whole file
-        assert f.read() == data
-        # and now the file magically has a size
-        assert f.size == len(data)
+    with open(os.path.join(dir_server, fn), "rb") as expected:
+        expected = expected.read()
+        with f as f:
+            # it's OK to read the whole file
+            assert f.read() == expected
+            # and now the file magically has a size
+            assert f.size == len(expected)
 
-    # note that if we reuse f from above, because it is tokenized, we get
-    # the same open file - where is this cached?
-    fn = files[1]
-    f = open_files(root + fn, block_size=2)[0]
-    with f as f:
-        # fails because we want only 12 bytes
-        with pytest.raises(ValueError):
-            assert f.read(10) == data[:10]
+        # note that if we reuse f from above, because it is tokenized, we get
+        # the same open file - where is this cached?
+        fn = files[1]
+        f = open_files(root + fn, block_size=2)[0]
+        with f as f:
+            if parse_version(fsspec.__version__) < parse_version("2021.11.1"):
+                # fails because we want only 12 bytes
+                with pytest.raises(ValueError):
+                    assert f.read(10) == expected[:10]
+            else:
+                # fixed in https://github.com/fsspec/filesystem_spec/pull/830
+                assert f.read(10) == expected[:10]
 
 
 def test_errors(dir_server):
     f = open_files("http://localhost:8999/doesnotexist")[0]
-    with pytest.raises(requests.exceptions.RequestException):
+    with pytest.raises(errs):
         with f as f:
             f.read()
     f = open_files("http://nohost/")[0]
 
-    if FSSPEC_042:
-        expected = FileNotFoundError
-    else:
-        expected = requests.exceptions.RequestException
+    expected = FileNotFoundError
 
     with pytest.raises(expected):
         with f as f:
@@ -142,7 +153,8 @@ def test_files(dir_server):
     fs = open_files([root + f for f in files])
     for f, f2 in zip(fs, files):
         with f as f:
-            assert f.read() == open(os.path.join(dir_server, f2), "rb").read()
+            with open(os.path.join(dir_server, f2), "rb") as expected:
+                assert f.read() == expected.read()
 
 
 def test_open_glob(dir_server):
@@ -169,11 +181,12 @@ def test_parquet():
     assert df.columns.tolist() == ["n_nationkey", "n_name", "n_regionkey", "n_comment"]
 
 
-@pytest.mark.xfail(reason="https://github.com/dask/dask/issues/3696", strict=False)
+@pytest.mark.flaky(
+    reruns=10, reruns_delay=5, reason="https://github.com/dask/dask/issues/3696"
+)
 @pytest.mark.network
 def test_bag():
     # This test pulls from different hosts
-    db = pytest.importorskip("dask.bag")
     urls = [
         "https://raw.githubusercontent.com/weierophinney/pastebin/"
         "master/public/js-src/dojox/data/tests/stores/patterns.csv",
@@ -184,10 +197,6 @@ def test_bag():
     b.compute()
 
 
-@pytest.mark.xfail(
-    LooseVersion(fsspec.__version__) <= "0.4.1",
-    reason="https://github.com/dask/dask/pull/5231",
-)
 @pytest.mark.network
 def test_read_csv():
     dd = pytest.importorskip("dask.dataframe")
