@@ -1,6 +1,7 @@
+from functools import partial
+
 import numpy as np
 import pandas as pd
-from functools import partial
 
 from ..utils import derived_from
 
@@ -13,7 +14,7 @@ def maybe_wrap_pandas(obj, x):
     return x
 
 
-class Accessor(object):
+class Accessor:
     """
     Base class for pandas Accessor objects cat, dt, and str.
 
@@ -50,7 +51,7 @@ class Accessor(object):
 
     def _property_map(self, attr):
         meta = self._delegate_property(self._series._meta, self._accessor_name, attr)
-        token = "%s-%s" % (self._accessor_name, attr)
+        token = f"{self._accessor_name}-{attr}"
         return self._series.map_partitions(
             self._delegate_property, self._accessor_name, attr, token=token, meta=meta
         )
@@ -62,7 +63,7 @@ class Accessor(object):
             meta = self._delegate_method(
                 self._series._meta_nonempty, self._accessor_name, attr, args, kwargs
             )
-        token = "%s-%s" % (self._accessor_name, attr)
+        token = f"{self._accessor_name}-{attr}"
         return self._series.map_partitions(
             self._delegate_method,
             self._accessor_name,
@@ -75,7 +76,7 @@ class Accessor(object):
 
     @property
     def _delegates(self):
-        return set(dir(self._meta)).difference(self._not_implemented)
+        return set(dir(self._meta)) - self._not_implemented
 
     def __dir__(self):
         o = self._delegates
@@ -138,10 +139,18 @@ class StringAccessor(Accessor):
 
     @derived_from(pd.core.strings.StringMethods)
     def cat(self, others=None, sep=None, na_rep=None):
-        from .core import Series, Index
+        from .core import Index, Series
 
         if others is None:
-            raise NotImplementedError("x.str.cat() with `others == None`")
+
+            def str_cat_none(x):
+
+                if isinstance(x, (Series, Index)):
+                    x = x.compute()
+
+                return x.str.cat(sep=sep, na_rep=na_rep)
+
+            return self._series.reduction(chunk=str_cat_none, aggregate=str_cat_none)
 
         valid_types = (Series, Index, pd.Series, pd.Index)
         if isinstance(others, valid_types):
@@ -168,7 +177,7 @@ def str_extractall(series, pat, flags):
 
 
 def str_get(series, index):
-    """ Implements series.str[index] """
+    """Implements series.str[index]"""
     return series.str[index]
 
 
