@@ -1,22 +1,25 @@
+from . import demo
+from .csv import read_csv, read_fwf, read_table, to_csv
+from .hdf import read_hdf, to_hdf
 from .io import (
+    dataframe_from_ctable,
     from_array,
     from_bcolz,
-    from_array,
-    from_bcolz,
-    from_pandas,
     from_dask_array,
     from_delayed,
-    dataframe_from_ctable,
+    from_pandas,
     to_bag,
     to_records,
 )
-from .csv import read_csv, to_csv, read_table, read_fwf
-from .hdf import read_hdf, to_hdf
-from .sql import read_sql_table, to_sql
 from .json import read_json, to_json
-from . import demo
+from .sql import read_sql_table, to_sql
 
 try:
     from .parquet import read_parquet, to_parquet
 except ImportError:
     pass
+
+try:
+    from .orc import read_orc, to_orc
+except ImportError:
+    pass
