@@ -1,117 +1,143 @@
 from collections.abc import Mapping
 from io import BytesIO
-from warnings import warn, catch_warnings, simplefilter
+from warnings import catch_warnings, simplefilter, warn
+
+from ...highlevelgraph import HighLevelGraph
+from ...layers import DataFrameIOLayer
 
 try:
     import psutil
 except ImportError:
     psutil = None
 
+import fsspec.implementations.local
 import numpy as np
 import pandas as pd
+from fsspec.compression import compr
+from fsspec.core import get_fs_token_paths
+from fsspec.core import open as open_file
+from fsspec.core import open_files
+from fsspec.utils import infer_compression
 from pandas.api.types import (
-    is_integer_dtype,
-    is_float_dtype,
-    is_object_dtype,
-    is_datetime64_any_dtype,
     CategoricalDtype,
+    is_datetime64_any_dtype,
+    is_float_dtype,
+    is_integer_dtype,
+    is_object_dtype,
 )
 
 from ...base import tokenize
-
-# this import checks for the importability of fsspec
-from ...bytes import read_bytes, open_file, open_files
-from ..core import new_dd_object
+from ...bytes import read_bytes
 from ...core import flatten
 from ...delayed import delayed
 from ...utils import asciitable, parse_bytes
+from ..core import new_dd_object
 from ..utils import clear_known_categories
 
-import fsspec.implementations.local
-from fsspec.compression import compr
-from fsspec.core import get_fs_token_paths
-from fsspec.utils import infer_compression
 
-
-class CSVSubgraph(Mapping):
+class CSVFunctionWrapper:
     """
-    Subgraph for reading CSV files.
+    CSV Function-Wrapper Class
+    Reads CSV data from disk to produce a partition (given a key).
     """
 
     def __init__(
         self,
-        name,
-        reader,
-        blocks,
-        is_first,
+        full_columns,
+        columns,
+        colname,
         head,
         header,
-        kwargs,
+        reader,
         dtypes,
-        columns,
         enforce,
-        path,
+        kwargs,
     ):
-        self.name = name
-        self.reader = reader
-        self.blocks = blocks
-        self.is_first = is_first
-        self.head = head  # example pandas DF for metadata
-        self.header = header  # prepend to all blocks
-        self.kwargs = kwargs
-        self.dtypes = dtypes
+        self.full_columns = full_columns
         self.columns = columns
+        self.colname = colname
+        self.head = head
+        self.header = header
+        self.reader = reader
+        self.dtypes = dtypes
         self.enforce = enforce
-        self.colname, self.paths = path or (None, None)
+        self.kwargs = kwargs
 
-    def __getitem__(self, key):
-        try:
-            name, i = key
-        except ValueError:
-            # too many / few values to unpack
-            raise KeyError(key) from None
+    def project_columns(self, columns):
+        """Return a new CSVFunctionWrapper object with
+        a sub-column projection.
+        """
+        # Make sure columns is ordered correctly
+        columns = [c for c in self.head.columns if c in columns]
+        if columns == self.columns:
+            return self
+        return CSVFunctionWrapper(
+            self.full_columns,
+            columns,
+            self.colname,
+            self.head[columns],
+            self.header,
+            self.reader,
+            {c: self.dtypes[c] for c in columns},
+            self.enforce,
+            self.kwargs,
+        )
 
-        if name != self.name:
-            raise KeyError(key)
+    def __call__(self, part):
 
-        if i < 0 or i >= len(self.blocks):
-            raise KeyError(key)
+        # Part will be a 3-element tuple
+        block, path, is_first, is_last = part
 
-        block = self.blocks[i]
-        if self.paths is not None:
+        # Construct `path_info`
+        if path is not None:
             path_info = (
                 self.colname,
-                self.paths[i],
+                path,
                 sorted(list(self.head[self.colname].cat.categories)),
             )
         else:
             path_info = None
 
+        # Deal with arguments that are special
+        # for the first block of each file
         write_header = False
         rest_kwargs = self.kwargs.copy()
-        if not self.is_first[i]:
+        if not is_first:
             write_header = True
             rest_kwargs.pop("skiprows", None)
+            if rest_kwargs.get("header", 0) is not None:
+                rest_kwargs.pop("header", None)
+        if not is_last:
+            rest_kwargs.pop("skipfooter", None)
 
-        return (
-            pandas_read_text,
+        # Deal with column projection
+        columns = self.full_columns
+        project_after_read = False
+        if self.columns is not None:
+            if self.kwargs:
+                # To be safe, if any kwargs are defined, avoid
+                # changing `usecols` here. Instead, we can just
+                # select columns after the read
+                project_after_read = True
+            else:
+                columns = self.columns
+                rest_kwargs["usecols"] = columns
+
+        # Call `pandas_read_text`
+        df = pandas_read_text(
             self.reader,
             block,
             self.header,
             rest_kwargs,
             self.dtypes,
-            self.columns,
+            columns,
             write_header,
             self.enforce,
             path_info,
         )
-
-    def __len__(self):
-        return len(self.blocks)
-
-    def __iter__(self):
-        for i in range(len(self)):
-            yield (self.name, i)
+        if project_after_read:
+            return df[self.columns]
+        return df
 
 
 def pandas_read_text(
@@ -138,7 +164,7 @@ def pandas_read_text(
     kwargs : dict
         A dictionary of keyword arguments to be passed to ``reader``
     dtypes : dict
-        DTypes to assign to columns
+        dtypes to assign to columns
     path : tuple
         A tuple containing path column name, path to file, and an ordered list of paths.
 
@@ -157,8 +183,6 @@ def pandas_read_text(
 
     if enforce and columns and (list(df.columns) != list(columns)):
         raise ValueError("Columns do not match", df.columns, columns)
-    elif columns:
-        df.columns = columns
     if path:
         colname, path, paths = path
         code = paths.index(path)
@@ -203,8 +227,7 @@ def coerce_dtypes(df, dtypes):
     if bad_dtypes:
         if errors:
             ex = "\n".join(
-                "- %s\n  %r" % (c, e)
-                for c, e in sorted(errors, key=lambda x: str(x[0]))
+                f"- {c}\n  {e!r}" for c, e in sorted(errors, key=lambda x: str(x[0]))
             )
             exceptions = (
                 "The following columns also raised exceptions on "
@@ -223,7 +246,7 @@ def coerce_dtypes(df, dtypes):
         bad_dtypes = sorted(bad_dtypes, key=lambda x: str(x[0]))
         table = asciitable(["Column", "Found", "Expected"], bad_dtypes)
         dtype_kw = "dtype={%s}" % ",\n       ".join(
-            "%r: '%s'" % (k, v) for (k, v, _) in bad_dtypes
+            f"{k!r}: '{v}'" for (k, v, _) in bad_dtypes
         )
 
         dtype_msg = (
@@ -270,6 +293,7 @@ def text_blocks_to_pandas(
     specified_dtypes=None,
     path=None,
     blocksize=None,
+    urlpath=None,
 ):
     """Convert blocks of bytes to a dask.dataframe
 
@@ -305,8 +329,6 @@ def text_blocks_to_pandas(
     # 2. contain 'category' for data inferred types
     categoricals = head.select_dtypes(include=["category"]).columns
 
-    known_categoricals = []
-    unknown_categoricals = categoricals
     if isinstance(specified_dtypes, Mapping):
         known_categoricals = [
             k
@@ -315,11 +337,7 @@ def text_blocks_to_pandas(
             and specified_dtypes.get(k).categories is not None
         ]
         unknown_categoricals = categoricals.difference(known_categoricals)
-    elif (
-        isinstance(specified_dtypes, CategoricalDtype)
-        and specified_dtypes.categories is None
-    ):
-        known_categoricals = []
+    else:
         unknown_categoricals = categoricals
 
     # Fixup the dtypes
@@ -331,8 +349,7 @@ def text_blocks_to_pandas(
     blocks = tuple(flatten(block_lists))
     # Create mask of first blocks from nested block_lists
     is_first = tuple(block_mask(block_lists))
-
-    name = "read-csv-" + tokenize(reader, columns, enforce, head, blocksize)
+    is_last = tuple(block_mask_last(block_lists))
 
     if path:
         colname, path_converter = path
@@ -351,21 +368,35 @@ def text_blocks_to_pandas(
     if len(unknown_categoricals):
         head = clear_known_categories(head, cols=unknown_categoricals)
 
-    subgraph = CSVSubgraph(
-        name,
-        reader,
-        blocks,
-        is_first,
-        head,
-        header,
-        kwargs,
-        dtypes,
-        columns,
-        enforce,
-        path,
-    )
+    # Define parts
+    parts = []
+    colname, paths = path or (None, None)
+    for i in range(len(blocks)):
+        parts.append([blocks[i], paths[i] if paths else None, is_first[i], is_last[i]])
 
-    return new_dd_object(subgraph, name, head, (None,) * (len(blocks) + 1))
+    # Create Blockwise layer
+    label = "read-csv-"
+    name = label + tokenize(reader, urlpath, columns, enforce, head, blocksize)
+    layer = DataFrameIOLayer(
+        name,
+        columns,
+        parts,
+        CSVFunctionWrapper(
+            columns,
+            None,
+            colname,
+            head,
+            header,
+            reader,
+            dtypes,
+            enforce,
+            kwargs,
+        ),
+        label=label,
+        produces_tasks=True,
+    )
+    graph = HighLevelGraph({name: layer}, {name: set()})
+    return new_dd_object(graph, name, head, (None,) * (len(blocks) + 1))
 
 
 def block_mask(block_lists):
@@ -383,21 +414,43 @@ def block_mask(block_lists):
         yield from (False for _ in block[1:])
 
 
+def block_mask_last(block_lists):
+    """
+    Yields a flat iterable of booleans to mark the last element of the
+    nested input ``block_lists`` in a flattened output.
+
+    >>> list(block_mask_last([[1, 2], [3, 4], [5]]))
+    [False, True, False, True, True]
+    """
+    for block in block_lists:
+        if not block:
+            continue
+        yield from (False for _ in block[:-1])
+        yield True
+
+
 def auto_blocksize(total_memory, cpu_count):
     memory_factor = 10
     blocksize = int(total_memory // cpu_count / memory_factor)
     return min(blocksize, int(64e6))
 
 
+def _infer_block_size():
+    default = 2 ** 25
+    if psutil is not None:
+        with catch_warnings():
+            simplefilter("ignore", RuntimeWarning)
+            mem = psutil.virtual_memory().total
+            cpu = psutil.cpu_count()
+
+        if mem and cpu:
+            return auto_blocksize(mem, cpu)
+
+    return default
+
+
 # guess blocksize if psutil is installed or use acceptable default one if not
-if psutil is not None:
-    with catch_warnings():
-        simplefilter("ignore", RuntimeWarning)
-        TOTAL_MEM = psutil.virtual_memory().total
-        CPU_COUNT = psutil.cpu_count()
-        AUTO_BLOCKSIZE = auto_blocksize(TOTAL_MEM, CPU_COUNT)
-else:
-    AUTO_BLOCKSIZE = 2 ** 25
+AUTO_BLOCKSIZE = _infer_block_size()
 
 
 def read_pandas(
@@ -407,6 +460,7 @@ def read_pandas(
     lineterminator=None,
     compression="infer",
     sample=256000,
+    sample_rows=10,
     enforce=False,
     assume_missing=False,
     storage_options=None,
@@ -423,12 +477,11 @@ def read_pandas(
     if "index" in kwargs or "index_col" in kwargs:
         raise ValueError(
             "Keywords 'index' and 'index_col' not supported. "
-            "Use dd.{0}(...).set_index('my-index') "
-            "instead".format(reader_name)
+            f"Use dd.{reader_name}(...).set_index('my-index') instead"
         )
     for kw in ["iterator", "chunksize"]:
         if kw in kwargs:
-            raise ValueError("{0} not supported for dd.{1}".format(kw, reader_name))
+            raise ValueError(f"{kw} not supported for dd.{reader_name}")
     if kwargs.get("nrows", None):
         raise ValueError(
             "The 'nrows' keyword is not supported by "
@@ -449,9 +502,7 @@ def read_pandas(
         # find the firstrow that is not skipped, for use as header
         firstrow = min(set(range(len(skiprows) + 1)) - set(skiprows))
     if isinstance(kwargs.get("header"), list):
-        raise TypeError(
-            "List of header rows not supported for dd.{0}".format(reader_name)
-        )
+        raise TypeError(f"List of header rows not supported for dd.{reader_name}")
     if isinstance(kwargs.get("converters"), dict) and include_path_column:
         path_converter = kwargs.get("converters").get(include_path_column, None)
     else:
@@ -521,7 +572,23 @@ def read_pandas(
     names = kwargs.get("names", None)
     header = kwargs.get("header", "infer" if names is None else None)
     need = 1 if header is None else 2
-    parts = b_sample.split(b_lineterminator, lastskiprow + need)
+
+    if kwargs.get("comment"):
+        # if comment is provided, step through lines of b_sample and strip out comments
+        parts = []
+        for part in b_sample.split(b_lineterminator):
+            split_comment = part.decode().split(kwargs.get("comment"))
+            if len(split_comment) > 1:
+                # if line starts with comment, don't include that line in parts.
+                if len(split_comment[0]) > 0:
+                    parts.append(split_comment[0].strip().encode())
+            else:
+                parts.append(part)
+            if len(parts) > need:
+                break
+    else:
+        parts = b_sample.split(b_lineterminator, lastskiprow + need)
+
     # If the last partition is empty, don't count it
     nparts = 0 if not parts else len(parts) - int(not parts[-1])
 
@@ -532,10 +599,23 @@ def read_pandas(
             "in `sample` in the call to `read_csv`/`read_table`"
         )
 
+    if isinstance(header, int):
+        firstrow += header
     header = b"" if header is None else parts[firstrow] + b_lineterminator
 
     # Use sample to infer dtypes and check for presence of include_path_column
-    head = reader(BytesIO(b_sample), **kwargs)
+    head_kwargs = kwargs.copy()
+    head_kwargs.pop("skipfooter", None)
+    try:
+        head = reader(BytesIO(b_sample), nrows=sample_rows, **head_kwargs)
+    except pd.errors.ParserError as e:
+        if "EOF" in str(e):
+            raise ValueError(
+                "EOF encountered while reading header. \n"
+                "Pass argument `sample_rows` and make sure the value of `sample` "
+                "is large enough to accommodate that many rows of data"
+            ) from e
+        raise
     if include_path_column and (include_path_column in head.columns):
         raise ValueError(
             "Files already contain the column name: %s, so the "
@@ -553,7 +633,7 @@ def read_pandas(
             if is_integer_dtype(head[c].dtype) and c not in specified_dtypes:
                 head[c] = head[c].astype(float)
 
-    values = [[dsk.dask.values() for dsk in block] for block in values]
+    values = [[list(dsk.dask.values()) for dsk in block] for block in values]
 
     return text_blocks_to_pandas(
         reader,
@@ -565,6 +645,7 @@ def read_pandas(
         specified_dtypes=specified_dtypes,
         path=path,
         blocksize=blocksize,
+        urlpath=urlpath,
     )
 
 
@@ -602,7 +683,7 @@ urlpath : string or list
 blocksize : str, int or None, optional
     Number of bytes by which to cut up larger files. Default value is computed
     based on available physical memory and the number of cores, up to a maximum
-    of 64MB. Can be a number like ``64000000` or a string like ``"64MB"``. If
+    of 64MB. Can be a number like ``64000000`` or a string like ``"64MB"``. If
     ``None``, a single block is used for each file.
 sample : int, optional
     Number of bytes to use when determining dtypes
@@ -650,6 +731,7 @@ def make_reader(reader, reader_name, file_type):
         lineterminator=None,
         compression="infer",
         sample=256000,
+        sample_rows=10,
         enforce=False,
         assume_missing=False,
         storage_options=None,
@@ -663,6 +745,7 @@ def make_reader(reader, reader_name, file_type):
             lineterminator=lineterminator,
             compression=compression,
             sample=sample,
+            sample_rows=sample_rows,
             enforce=enforce,
             assume_missing=assume_missing,
             storage_options=storage_options,
@@ -868,7 +951,9 @@ def to_csv(
         if scheduler is not None and compute_kwargs.get("scheduler") is None:
             compute_kwargs["scheduler"] = scheduler
 
-        delayed(values).compute(**compute_kwargs)
+        import dask
+
+        dask.compute(*values, **compute_kwargs)
         return [f.path for f in files]
     else:
         return values
