@@ -3,9 +3,10 @@ import pandas as pd
 
 import dask
 from dask.delayed import tokenize
-from .io import from_delayed, from_pandas
+
 from ... import delayed
 from .. import methods
+from .io import from_delayed, from_pandas
 
 
 def read_sql_table(
@@ -124,10 +125,8 @@ def read_sql_table(
         if columns
         else list(table.columns)
     )
-    if index_col not in columns:
-        columns.append(
-            table.columns[index_col] if isinstance(index_col, str) else index_col
-        )
+    if index not in columns:
+        columns.append(index)
 
     if isinstance(index_col, str):
         kwargs["index_col"] = index_col
@@ -140,7 +139,7 @@ def read_sql_table(
         q = sql.select(columns).limit(head_rows).select_from(table)
         head = pd.read_sql(q, engine, **kwargs)
 
-        if head.empty:
+        if len(head) == 0:
             # no results at all
             name = table.name
             schema = table.schema
@@ -223,8 +222,12 @@ def _read_sql_chunk(q, uri, meta, engine_kwargs=None, **kwargs):
     engine = sa.create_engine(uri, **engine_kwargs)
     df = pd.read_sql(q, engine, **kwargs)
     engine.dispose()
-    if df.empty:
+    if len(df) == 0:
         return meta
+    elif len(meta.dtypes.to_dict()) == 0:
+        # only index column in loaded
+        # required only for pandas < 1.0.0
+        return df
     else:
         return df.astype(meta.dtypes.to_dict(), copy=False)
 
@@ -341,13 +344,13 @@ def to_sql(
     Dask Name: from_pandas, 2 tasks
 
     >>> from dask.utils import tmpfile
-    >>> from sqlalchemy import create_engine    # doctest: +SKIP
-    >>> with tmpfile() as f:                    # doctest: +SKIP
-    ...     db = 'sqlite:///%s' %f              # doctest: +SKIP
-    ...     ddf.to_sql('test', db)              # doctest: +SKIP
-    ...     engine = create_engine(db, echo=False) # doctest: +SKIP
-    ...     result = engine.execute("SELECT * FROM test").fetchall() # doctest: +SKIP
-    >>> result                                  # doctest: +SKIP
+    >>> from sqlalchemy import create_engine
+    >>> with tmpfile() as f:
+    ...     db = 'sqlite:///%s' %f
+    ...     ddf.to_sql('test', db)
+    ...     engine = create_engine(db, echo=False)
+    ...     result = engine.execute("SELECT * FROM test").fetchall()
+    >>> result
     [(0, 0, '00'), (1, 1, '11'), (2, 2, '22'), (3, 3, '33')]
     """
     if not isinstance(uri, str):
