@@ -1,15 +1,17 @@
 import datetime
-
-import pandas as pd
-from pandas.core.window import Rolling as pd_Rolling
+import inspect
 from numbers import Integral
 
+import pandas as pd
+from pandas.api.types import is_datetime64_any_dtype
+from pandas.core.window import Rolling as pd_Rolling
+
 from ..base import tokenize
-from ..utils import M, funcname, derived_from
 from ..highlevelgraph import HighLevelGraph
-from .core import _emulate
-from .utils import make_meta, PANDAS_VERSION
+from ..utils import M, derived_from, funcname, has_keyword
 from . import methods
+from .core import _emulate
+from .utils import make_meta
 
 
 def overlap_chunk(
@@ -75,7 +77,7 @@ def map_overlap(func, df, before, after, *args, **kwargs):
     dd.DataFrame.map_overlap
     """
     if isinstance(before, datetime.timedelta) or isinstance(after, datetime.timedelta):
-        if not df.index._meta_nonempty.is_all_dates:
+        if not is_datetime64_any_dtype(df.index._meta_nonempty.inferred_type):
             raise TypeError(
                 "Must have a `DatetimeIndex` when using string offset "
                 "for `before` and `after`"
@@ -100,9 +102,9 @@ def map_overlap(func, df, before, after, *args, **kwargs):
         meta = kwargs.pop("meta")
     else:
         meta = _emulate(func, df, *args, **kwargs)
-    meta = make_meta(meta, index=df._meta.index)
+    meta = make_meta(meta, index=df._meta.index, parent_meta=df._meta)
 
-    name = "{0}-{1}".format(func_name, token)
+    name = f"{func_name}-{token}"
     name_a = "overlap-prepend-" + tokenize(df, before)
     name_b = "overlap-append-" + tokenize(df, after)
     df_name = df._name
@@ -115,13 +117,13 @@ def map_overlap(func, df, before, after, *args, **kwargs):
     )
 
     if before and isinstance(before, Integral):
-        dsk.update(
-            {
-                (name_a, i): (M.tail, (df_name, i), before)
-                for i in range(df.npartitions - 1)
-            }
-        )
-        prevs = [None] + [(name_a, i) for i in range(df.npartitions - 1)]
+
+        prevs = [None]
+        for i in range(df.npartitions - 1):
+            key = (name_a, i)
+            dsk[key] = (M.tail, (df_name, i), before)
+            prevs.append(key)
+
     elif isinstance(before, datetime.timedelta):
         # Assumes monotonic (increasing?) index
         divs = pd.Series(df.divisions)
@@ -133,6 +135,7 @@ def map_overlap(func, df, before, after, *args, **kwargs):
 
         if (before > deltas).any():
             pt_z = divs[0]
+            prevs = [None]
             for i in range(df.npartitions - 1):
                 # Select all indexes of relevant partitions between the current partition and
                 # the partition with the highest division outside the rolling window (before)
@@ -146,56 +149,48 @@ def map_overlap(func, df, before, after, *args, **kwargs):
                     first = first - deltas[j]
                     j = j - 1
 
-                dsk.update(
-                    {
-                        (name_a, i): (
-                            _tail_timedelta,
-                            [(df_name, k) for k in range(j, i + 1)],
-                            (df_name, i + 1),
-                            before,
-                        )
-                    }
+                key = (name_a, i)
+                dsk[key] = (
+                    _tail_timedelta,
+                    [(df_name, k) for k in range(j, i + 1)],
+                    (df_name, i + 1),
+                    before,
                 )
-
-            prevs = [None] + [(name_a, i) for i in range(df.npartitions - 1)]
+                prevs.append(key)
 
         else:
-            dsk.update(
-                {
-                    (name_a, i): (
-                        _tail_timedelta,
-                        [(df_name, i)],
-                        (df_name, i + 1),
-                        before,
-                    )
-                    for i in range(df.npartitions - 1)
-                }
-            )
-            prevs = [None] + [(name_a, i) for i in range(df.npartitions - 1)]
+            prevs = [None]
+            for i in range(df.npartitions - 1):
+                key = (name_a, i)
+                dsk[key] = (
+                    _tail_timedelta,
+                    [(df_name, i)],
+                    (df_name, i + 1),
+                    before,
+                )
+                prevs.append(key)
     else:
         prevs = [None] * df.npartitions
 
     if after and isinstance(after, Integral):
-        dsk.update(
-            {
-                (name_b, i): (M.head, (df_name, i), after)
-                for i in range(1, df.npartitions)
-            }
-        )
-        nexts = [(name_b, i) for i in range(1, df.npartitions)] + [None]
+        nexts = []
+        for i in range(1, df.npartitions):
+            key = (name_b, i)
+            dsk[key] = (M.head, (df_name, i), after)
+            nexts.append(key)
+        nexts.append(None)
     elif isinstance(after, datetime.timedelta):
         # TODO: Do we have a use-case for this? Pandas doesn't allow negative rolling windows
         deltas = pd.Series(df.divisions).diff().iloc[1:-1]
         if (after > deltas).any():
             raise ValueError(timedelta_partition_message)
 
-        dsk.update(
-            {
-                (name_b, i): (_head_timedelta, (df_name, i - 0), (df_name, i), after)
-                for i in range(1, df.npartitions)
-            }
-        )
-        nexts = [(name_b, i) for i in range(1, df.npartitions)] + [None]
+        nexts = []
+        for i in range(1, df.npartitions):
+            key = (name_b, i)
+            dsk[key] = (_head_timedelta, (df_name, i - 0), (df_name, i), after)
+            nexts.append(key)
+        nexts.append(None)
     else:
         nexts = [None] * df.npartitions
 
@@ -253,12 +248,7 @@ def _tail_timedelta(prevs, current, before):
     return selected
 
 
-def pandas_rolling_method(df, rolling_kwargs, name, *args, **kwargs):
-    rolling = df.rolling(**rolling_kwargs)
-    return getattr(rolling, name)(*args, **kwargs)
-
-
-class Rolling(object):
+class Rolling:
     """Provides rolling window calculations."""
 
     def __init__(
@@ -271,15 +261,13 @@ class Rolling(object):
         self.axis = axis
         self.win_type = win_type
         # Allow pandas to raise if appropriate
-        pd_roll = obj._meta.rolling(**self._rolling_kwargs())
+        obj._meta.rolling(**self._rolling_kwargs())
         # Using .rolling(window='2s'), pandas will convert the
         # offset str to a window in nanoseconds. But pandas doesn't
         # accept the integer window with win_type='freq', so we store
         # that information here.
         # See https://github.com/pandas-dev/pandas/issues/15969
-        self._window = pd_roll.window
-        self._win_type = pd_roll.win_type
-        self._min_periods = pd_roll.min_periods
+        self._win_type = None if isinstance(self.window, int) else "freq"
 
     def _rolling_kwargs(self):
         return {
@@ -302,22 +290,27 @@ class Rolling(object):
             or self.obj.npartitions == 1
         )
 
+    @staticmethod
+    def pandas_rolling_method(df, rolling_kwargs, name, *args, **kwargs):
+        rolling = df.rolling(**rolling_kwargs)
+        return getattr(rolling, name)(*args, **kwargs)
+
     def _call_method(self, method_name, *args, **kwargs):
         rolling_kwargs = self._rolling_kwargs()
-        meta = pandas_rolling_method(
+        meta = self.pandas_rolling_method(
             self.obj._meta_nonempty, rolling_kwargs, method_name, *args, **kwargs
         )
 
         if self._has_single_partition:
             # There's no overlap just use map_partitions
             return self.obj.map_partitions(
-                pandas_rolling_method,
+                self.pandas_rolling_method,
                 rolling_kwargs,
                 method_name,
                 *args,
                 token=method_name,
                 meta=meta,
-                **kwargs
+                **kwargs,
             )
         # Convert window to overlap
         if self.center:
@@ -330,7 +323,7 @@ class Rolling(object):
             before = self.window - 1
             after = 0
         return map_overlap(
-            pandas_rolling_method,
+            self.pandas_rolling_method,
             self.obj,
             before,
             after,
@@ -339,7 +332,7 @@ class Rolling(object):
             *args,
             token=method_name,
             meta=meta,
-            **kwargs
+            **kwargs,
         )
 
     @derived_from(pd_Rolling)
@@ -391,20 +384,29 @@ class Rolling(object):
         return self._call_method("quantile", quantile)
 
     @derived_from(pd_Rolling)
-    def apply(self, func, args=(), kwargs={}, **kwds):
-        # TODO: In a future version of pandas this will change to
-        # raw=False. Think about inspecting the function signature and setting
-        # to that?
-        if PANDAS_VERSION >= "0.23.0":
-            kwds.setdefault("raw", None)
-        else:
-            if kwargs:
-                msg = (
-                    "Invalid argument to 'apply'. Keyword arguments "
-                    "should be given as a dict to the 'kwargs' argument. "
-                )
-                raise TypeError(msg)
-        return self._call_method("apply", func, args=args, kwargs=kwargs, **kwds)
+    def apply(
+        self,
+        func,
+        raw=None,
+        engine="cython",
+        engine_kwargs=None,
+        args=None,
+        kwargs=None,
+    ):
+        compat_kwargs = {}
+        kwargs = kwargs or {}
+        args = args or ()
+        meta = self.obj._meta.rolling(0)
+        if has_keyword(meta.apply, "engine"):
+            # PANDAS_GT_100
+            compat_kwargs = dict(engine=engine, engine_kwargs=engine_kwargs)
+        if raw is None:
+            # PANDAS_GT_100: The default changed from None to False
+            raw = inspect.signature(meta.apply).parameters["raw"]
+
+        return self._call_method(
+            "apply", func, raw=raw, args=args, kwargs=kwargs, **compat_kwargs
+        )
 
     @derived_from(pd_Rolling)
     def aggregate(self, func, args=(), kwargs={}, **kwds):
@@ -425,13 +427,72 @@ class Rolling(object):
             return _order[k]
 
         rolling_kwargs = self._rolling_kwargs()
-        # pandas translates the '2S' offset to nanoseconds
-        rolling_kwargs["window"] = self._window
+        rolling_kwargs["window"] = self.window
         rolling_kwargs["win_type"] = self._win_type
         return "Rolling [{}]".format(
             ",".join(
-                "{}={}".format(k, v)
+                f"{k}={v}"
                 for k, v in sorted(rolling_kwargs.items(), key=order)
                 if v is not None
             )
         )
+
+
+class RollingGroupby(Rolling):
+    def __init__(
+        self,
+        groupby,
+        window=None,
+        min_periods=None,
+        center=False,
+        win_type=None,
+        axis=0,
+    ):
+        self._groupby_kwargs = groupby._groupby_kwargs
+        self._groupby_slice = groupby._slice
+
+        obj = groupby.obj
+        if self._groupby_slice is not None:
+            if isinstance(self._groupby_slice, str):
+                sliced_plus = [self._groupby_slice]
+            else:
+                sliced_plus = list(self._groupby_slice)
+            if isinstance(groupby.by, str):
+                sliced_plus.append(groupby.by)
+            else:
+                sliced_plus.extend(groupby.by)
+            obj = obj[sliced_plus]
+
+        super().__init__(
+            obj,
+            window=window,
+            min_periods=min_periods,
+            center=center,
+            win_type=win_type,
+            axis=axis,
+        )
+
+    @staticmethod
+    def pandas_rolling_method(
+        df,
+        rolling_kwargs,
+        name,
+        *args,
+        groupby_kwargs=None,
+        groupby_slice=None,
+        **kwargs,
+    ):
+        groupby = df.groupby(**groupby_kwargs)
+        if groupby_slice:
+            groupby = groupby[groupby_slice]
+        rolling = groupby.rolling(**rolling_kwargs)
+        return getattr(rolling, name)(*args, **kwargs).sort_index(level=-1)
+
+    def _call_method(self, method_name, *args, **kwargs):
+        return super()._call_method(
+            method_name,
+            *args,
+            groupby_kwargs=self._groupby_kwargs,
+            groupby_slice=self._groupby_slice,
+            **kwargs,
+        )
