@@ -1,9 +1,9 @@
-import dask.dataframe as dd
 import numpy as np
 import pandas as pd
 import pytest
 
-from dask.dataframe.utils import assert_eq, PANDAS_VERSION
+import dask.dataframe as dd
+from dask.dataframe.utils import assert_eq
 
 
 # Fixtures
@@ -66,6 +66,16 @@ def ddf_right_single(df_right):
     return dd.from_pandas(df_right, npartitions=1, sort=False)
 
 
+@pytest.fixture
+def ddf_right_double(df_right):
+    return dd.from_pandas(df_right, npartitions=2, sort=False)
+
+
+@pytest.fixture
+def ddf_left_double(df_left):
+    return dd.from_pandas(df_left, npartitions=2, sort=False)
+
+
 @pytest.fixture(params=["inner", "left", "right", "outer"])
 def how(request):
     return request.param
@@ -78,16 +88,14 @@ def on(request):
 
 # Tests
 # =====
-@pytest.mark.skipif(
-    PANDAS_VERSION < "0.23.0",
-    reason="Need pandas col+index merge support (pandas-dev/pandas#14355)",
-)
-def test_merge_known_to_known(df_left, df_right, ddf_left, ddf_right, on, how):
+def test_merge_known_to_known(
+    df_left, df_right, ddf_left, ddf_right, on, how, shuffle_method
+):
     # Compute expected
     expected = df_left.merge(df_right, on=on, how=how)
 
     # Perform merge
-    result = ddf_left.merge(ddf_right, on=on, how=how, shuffle="tasks")
+    result = ddf_left.merge(ddf_right, on=on, how=how, shuffle=shuffle_method)
 
     # Assertions
     assert_eq(result, expected)
@@ -95,94 +103,144 @@ def test_merge_known_to_known(df_left, df_right, ddf_left, ddf_right, on, how):
     assert len(result.__dask_graph__()) < 80
 
 
-@pytest.mark.skipif(
-    PANDAS_VERSION < "0.23.0",
-    reason="Need pandas col+index merge support (pandas-dev/pandas#14355)",
-)
 @pytest.mark.parametrize("how", ["inner", "left"])
-def test_merge_known_to_single(df_left, df_right, ddf_left, ddf_right_single, on, how):
+def test_merge_known_to_single(
+    df_left, df_right, ddf_left, ddf_right_single, on, how, shuffle_method
+):
     # Compute expected
     expected = df_left.merge(df_right, on=on, how=how)
 
     # Perform merge
-    result = ddf_left.merge(ddf_right_single, on=on, how=how, shuffle="tasks")
+    result = ddf_left.merge(ddf_right_single, on=on, how=how, shuffle=shuffle_method)
 
     # Assertions
     assert_eq(result, expected)
-    assert_eq(result.divisions, ddf_left.divisions)
+    assert result.divisions == ddf_left.divisions
     assert len(result.__dask_graph__()) < 30
 
 
-@pytest.mark.skipif(
-    PANDAS_VERSION < "0.23.0",
-    reason="Need pandas col+index merge support (pandas-dev/pandas#14355)",
-)
 @pytest.mark.parametrize("how", ["inner", "right"])
-def test_merge_single_to_known(df_left, df_right, ddf_left_single, ddf_right, on, how):
+def test_merge_single_to_known(
+    df_left, df_right, ddf_left_single, ddf_right, on, how, shuffle_method
+):
     # Compute expected
     expected = df_left.merge(df_right, on=on, how=how)
 
     # Perform merge
-    result = ddf_left_single.merge(ddf_right, on=on, how=how, shuffle="tasks")
+    result = ddf_left_single.merge(ddf_right, on=on, how=how, shuffle=shuffle_method)
 
     # Assertions
     assert_eq(result, expected)
-    assert_eq(result.divisions, ddf_right.divisions)
+    assert result.divisions == ddf_right.divisions
     assert len(result.__dask_graph__()) < 30
 
 
-@pytest.mark.skipif(
-    PANDAS_VERSION < "0.23.0",
-    reason="Need pandas col+index merge support (pandas-dev/pandas#14355)",
-)
 def test_merge_known_to_unknown(
-    df_left, df_right, ddf_left, ddf_right_unknown, on, how
+    df_left, df_right, ddf_left, ddf_right_unknown, on, how, shuffle_method
 ):
     # Compute expected
     expected = df_left.merge(df_right, on=on, how=how)
 
     # Perform merge
-    result = ddf_left.merge(ddf_right_unknown, on=on, how=how, shuffle="tasks")
+    result = ddf_left.merge(ddf_right_unknown, on=on, how=how, shuffle=shuffle_method)
 
     # Assertions
     assert_eq(result, expected)
     assert_eq(result.divisions, tuple(None for _ in range(11)))
-    assert len(result.__dask_graph__()) >= 400
 
 
-@pytest.mark.skipif(
-    PANDAS_VERSION < "0.23.0",
-    reason="Need pandas col+index merge support (pandas-dev/pandas#14355)",
-)
 def test_merge_unknown_to_known(
-    df_left, df_right, ddf_left_unknown, ddf_right, on, how
+    df_left, df_right, ddf_left_unknown, ddf_right, on, how, shuffle_method
 ):
     # Compute expected
     expected = df_left.merge(df_right, on=on, how=how)
 
     # Perform merge
-    result = ddf_left_unknown.merge(ddf_right, on=on, how=how, shuffle="tasks")
+    result = ddf_left_unknown.merge(ddf_right, on=on, how=how, shuffle=shuffle_method)
 
     # Assertions
     assert_eq(result, expected)
     assert_eq(result.divisions, tuple(None for _ in range(11)))
-    assert len(result.__dask_graph__()) > 400
 
 
-@pytest.mark.skipif(
-    PANDAS_VERSION < "0.23.0",
-    reason="Need pandas col+index merge support (pandas-dev/pandas#14355)",
-)
 def test_merge_unknown_to_unknown(
-    df_left, df_right, ddf_left_unknown, ddf_right_unknown, on, how
+    df_left,
+    df_right,
+    ddf_left_unknown,
+    ddf_right_unknown,
+    on,
+    how,
+    shuffle_method,
 ):
     # Compute expected
     expected = df_left.merge(df_right, on=on, how=how)
 
     # Merge unknown to unknown
-    result = ddf_left_unknown.merge(ddf_right_unknown, on=on, how=how, shuffle="tasks")
+    result = ddf_left_unknown.merge(
+        ddf_right_unknown, on=on, how=how, shuffle=shuffle_method
+    )
 
     # Assertions
     assert_eq(result, expected)
     assert_eq(result.divisions, tuple(None for _ in range(11)))
-    assert len(result.__dask_graph__()) > 400
+
+
+@pytest.mark.parametrize("how", ["inner", "left"])
+def test_merge_known_to_double_bcast_right(
+    df_left, df_right, ddf_left, ddf_right_double, on, how, shuffle_method
+):
+    # Compute expected
+    expected = df_left.merge(df_right, on=on, how=how)
+
+    # Perform merge
+    result = ddf_left.merge(
+        ddf_right_double, on=on, how=how, shuffle=shuffle_method, broadcast=True
+    )
+
+    # Assertions
+    assert_eq(result, expected)
+    # Hash join used in disk-shuffling doesn't preserve divisions.
+    if shuffle_method == "task":
+        assert_eq(result.divisions, ddf_left.divisions)
+
+
+@pytest.mark.parametrize("how", ["inner", "right"])
+@pytest.mark.parametrize("broadcast", [True, 0.75])
+def test_merge_known_to_double_bcast_left(
+    df_left, df_right, ddf_left_double, ddf_right, on, shuffle_method, how, broadcast
+):
+    # Compute expected
+    expected = df_left.merge(df_right, on=on, how=how)
+
+    # Perform merge
+    result = ddf_left_double.merge(
+        ddf_right, on=on, how=how, broadcast=broadcast, shuffle=shuffle_method
+    )
+
+    # Assertions
+    assert_eq(result, expected)
+    # Hash join used in disk-shuffling doesn't preserve divisions.
+    if shuffle_method == "task":
+        assert_eq(result.divisions, ddf_right.divisions)
+
+    # Check that culling works
+    result.head(1)
+
+
+@pytest.mark.parametrize("repartition", [None, 4])
+def test_merge_column_with_nulls(repartition):
+    # See: https://github.com/dask/dask/issues/7558
+
+    df1 = pd.DataFrame({"a": ["0", "0", None, None, None, None, "5", "7", "15", "33"]})
+    df2 = pd.DataFrame({"c": ["1", "2", "3", "4"], "b": ["0", "5", "7", "15"]})
+    df1_d = dd.from_pandas(df1, npartitions=4)
+    df2_d = dd.from_pandas(df2, npartitions=3).set_index("b")
+    if repartition:
+        df2_d = df2_d.repartition(repartition)
+
+    pandas_result = df1.merge(
+        df2.set_index("b"), how="left", left_on="a", right_index=True
+    )
+    dask_result = df1_d.merge(df2_d, how="left", left_on="a", right_index=True)
+
+    assert_eq(dask_result, pandas_result)
