@@ -1,29 +1,29 @@
 import warnings
 
-import dask.dataframe as dd
 import numpy as np
 import pandas as pd
+import pytest
 
+import dask.dataframe as dd
 from dask.base import compute_as_if_collection
 from dask.dataframe._compat import tm
 from dask.dataframe.core import _Frame
 from dask.dataframe.methods import concat
 from dask.dataframe.multi import (
-    align_partitions,
-    merge_indexed_dataframes,
-    hash_join,
-    concat_indexed_dataframes,
     _maybe_align_partitions,
+    align_partitions,
+    concat_indexed_dataframes,
+    hash_join,
+    merge_indexed_dataframes,
 )
 from dask.dataframe.utils import (
-    assert_eq,
     assert_divisions,
-    make_meta,
-    has_known_categories,
+    assert_eq,
     clear_known_categories,
+    has_known_categories,
+    make_meta,
 )
-
-import pytest
+from dask.utils_test import hlg_layer_topological
 
 
 def test_align_partitions():
@@ -224,8 +224,7 @@ def list_eq(aa, bb):
 
 
 @pytest.mark.parametrize("how", ["inner", "left", "right", "outer"])
-@pytest.mark.parametrize("shuffle", ["disk", "tasks"])
-def test_hash_join(how, shuffle):
+def test_hash_join(how, shuffle_method):
     A = pd.DataFrame({"x": [1, 2, 3, 4, 5, 6], "y": [1, 1, 2, 2, 3, 4]})
     a = dd.repartition(A, [0, 4, 5])
 
@@ -239,7 +238,7 @@ def test_hash_join(how, shuffle):
     list_eq(result, expected)
 
     # Different columns and npartitions
-    c = hash_join(a, "x", b, "z", "outer", npartitions=3, shuffle=shuffle)
+    c = hash_join(a, "x", b, "z", "outer", npartitions=3, shuffle=shuffle_method)
     assert c.npartitions == 3
 
     result = c.compute(scheduler="single-threaded")
@@ -248,12 +247,12 @@ def test_hash_join(how, shuffle):
     list_eq(result, expected)
 
     assert (
-        hash_join(a, "y", b, "y", "inner", shuffle=shuffle)._name
-        == hash_join(a, "y", b, "y", "inner", shuffle=shuffle)._name
+        hash_join(a, "y", b, "y", "inner", shuffle=shuffle_method)._name
+        == hash_join(a, "y", b, "y", "inner", shuffle=shuffle_method)._name
     )
     assert (
-        hash_join(a, "y", b, "y", "inner", shuffle=shuffle)._name
-        != hash_join(a, "y", b, "y", "outer", shuffle=shuffle)._name
+        hash_join(a, "y", b, "y", "inner", shuffle=shuffle_method)._name
+        != hash_join(a, "y", b, "y", "outer", shuffle=shuffle_method)._name
     )
 
 
@@ -336,7 +335,7 @@ def test_merge_asof_left_on_right_index(
     b = dd.from_pandas(B, npartitions=2)
 
     if unknown_divisions:
-        a.divisions = [None] * len(a.divisions)
+        a.divisions = (None,) * len(a.divisions)
 
     C = pd.merge_asof(
         A,
@@ -369,7 +368,7 @@ def test_merge_asof_left_on_right_index(
             b = dd.from_pandas(B, npartitions=nparts)
 
             if unknown_divisions:
-                a.divisions = [None] * len(a.divisions)
+                a.divisions = (None,) * len(a.divisions)
 
             C = pd.merge_asof(
                 A,
@@ -583,6 +582,58 @@ def test_merge_asof_unsorted_raises():
         result.compute()
 
 
+def test_merge_asof_with_empty():
+    good_df = pd.DataFrame({"index_col": list(range(10)), "good_val": list(range(10))})
+    good_dd = dd.from_pandas(good_df, npartitions=2)
+    empty_df = (
+        good_df[good_df.index_col < 0].copy().rename(columns={"good_val": "empty_val"})
+    )
+    empty_dd = dd.from_pandas(empty_df, npartitions=2)
+
+    # left good, right empty
+    result_dd = dd.merge_asof(
+        good_dd.set_index("index_col"),
+        empty_dd.set_index("index_col"),
+        left_index=True,
+        right_index=True,
+    )
+    result_df = pd.merge_asof(
+        good_df.set_index("index_col"),
+        empty_df.set_index("index_col"),
+        left_index=True,
+        right_index=True,
+    )
+    assert_eq(result_dd, result_df, check_index=False)
+    # left empty, right good
+    result_dd = dd.merge_asof(
+        empty_dd.set_index("index_col"),
+        good_dd.set_index("index_col"),
+        left_index=True,
+        right_index=True,
+    )
+    result_df = pd.merge_asof(
+        empty_df.set_index("index_col"),
+        good_df.set_index("index_col"),
+        left_index=True,
+        right_index=True,
+    )
+    assert_eq(result_dd, result_df, check_index=False)
+    # left/right both empty
+    result_dd = dd.merge_asof(
+        empty_dd.set_index("index_col"),
+        empty_dd.set_index("index_col"),
+        left_index=True,
+        right_index=True,
+    )
+    result_df = pd.merge_asof(
+        empty_df.set_index("index_col"),
+        empty_df.set_index("index_col"),
+        left_index=True,
+        right_index=True,
+    )
+    assert_eq(result_dd, result_df, check_index=False)
+
+
 @pytest.mark.parametrize("join", ["inner", "outer"])
 def test_indexed_concat(join):
     A = pd.DataFrame(
@@ -718,8 +769,7 @@ def test_merge_columns_dtypes(how, on_index):
 
 
 @pytest.mark.parametrize("how", ["inner", "outer", "left", "right"])
-@pytest.mark.parametrize("shuffle", ["disk", "tasks"])
-def test_merge(how, shuffle):
+def test_merge(how, shuffle_method):
     A = pd.DataFrame({"x": [1, 2, 3, 4, 5, 6], "y": [1, 1, 2, 2, 3, 4]})
     a = dd.repartition(A, [0, 4, 5])
 
@@ -727,7 +777,9 @@ def test_merge(how, shuffle):
     b = dd.repartition(B, [0, 2, 5])
 
     assert_eq(
-        dd.merge(a, b, left_index=True, right_index=True, how=how, shuffle=shuffle),
+        dd.merge(
+            a, b, left_index=True, right_index=True, how=how, shuffle=shuffle_method
+        ),
         pd.merge(A, B, left_index=True, right_index=True, how=how),
     )
 
@@ -736,7 +788,7 @@ def test_merge(how, shuffle):
     assert all(d is None for d in result.divisions)
 
     list_eq(
-        dd.merge(a, b, left_on="x", right_on="z", how=how, shuffle=shuffle),
+        dd.merge(a, b, left_on="x", right_on="z", how=how, shuffle=shuffle_method),
         pd.merge(A, B, left_on="x", right_on="z", how=how),
     )
     list_eq(
@@ -747,18 +799,20 @@ def test_merge(how, shuffle):
             right_on="z",
             how=how,
             suffixes=("1", "2"),
-            shuffle=shuffle,
+            shuffle=shuffle_method,
         ),
         pd.merge(A, B, left_on="x", right_on="z", how=how, suffixes=("1", "2")),
     )
 
-    list_eq(dd.merge(a, b, how=how, shuffle=shuffle), pd.merge(A, B, how=how))
-    list_eq(dd.merge(a, B, how=how, shuffle=shuffle), pd.merge(A, B, how=how))
-    list_eq(dd.merge(A, b, how=how, shuffle=shuffle), pd.merge(A, B, how=how))
-    list_eq(dd.merge(A, B, how=how, shuffle=shuffle), pd.merge(A, B, how=how))
+    list_eq(dd.merge(a, b, how=how, shuffle=shuffle_method), pd.merge(A, B, how=how))
+    list_eq(dd.merge(a, B, how=how, shuffle=shuffle_method), pd.merge(A, B, how=how))
+    list_eq(dd.merge(A, b, how=how, shuffle=shuffle_method), pd.merge(A, B, how=how))
+    list_eq(dd.merge(A, B, how=how, shuffle=shuffle_method), pd.merge(A, B, how=how))
 
     list_eq(
-        dd.merge(a, b, left_index=True, right_index=True, how=how, shuffle=shuffle),
+        dd.merge(
+            a, b, left_index=True, right_index=True, how=how, shuffle=shuffle_method
+        ),
         pd.merge(A, B, left_index=True, right_index=True, how=how),
     )
     list_eq(
@@ -769,13 +823,13 @@ def test_merge(how, shuffle):
             right_index=True,
             how=how,
             suffixes=("1", "2"),
-            shuffle=shuffle,
+            shuffle=shuffle_method,
         ),
         pd.merge(A, B, left_index=True, right_index=True, how=how, suffixes=("1", "2")),
     )
 
     list_eq(
-        dd.merge(a, b, left_on="x", right_index=True, how=how, shuffle=shuffle),
+        dd.merge(a, b, left_on="x", right_index=True, how=how, shuffle=shuffle_method),
         pd.merge(A, B, left_on="x", right_index=True, how=how),
     )
     list_eq(
@@ -786,7 +840,7 @@ def test_merge(how, shuffle):
             right_index=True,
             how=how,
             suffixes=("1", "2"),
-            shuffle=shuffle,
+            shuffle=shuffle_method,
         ),
         pd.merge(A, B, left_on="x", right_index=True, how=how, suffixes=("1", "2")),
     )
@@ -801,13 +855,13 @@ def test_merge(how, shuffle):
 @pytest.mark.parametrize(
     "engine",
     [
-        "cudf",
         pytest.param(
             "pandas",
             marks=pytest.mark.xfail(
                 reason="Pandas does not support leftsemi or leftanti"
             ),
         ),
+        pytest.param("cudf", marks=pytest.mark.gpu),
     ],
 )
 def test_merge_tasks_semi_anti_cudf(engine, how, parts):
@@ -861,9 +915,8 @@ def test_merge_tasks_passes_through():
 
 
 @pytest.mark.slow
-@pytest.mark.parametrize("shuffle", ["disk", "tasks"])
 @pytest.mark.parametrize("how", ["inner", "outer", "left", "right"])
-def test_merge_by_index_patterns(how, shuffle):
+def test_merge_by_index_patterns(how, shuffle_method):
 
     pdf1l = pd.DataFrame({"a": [1, 2, 3, 4, 5, 6, 7], "b": [7, 6, 5, 4, 3, 2, 1]})
     pdf1r = pd.DataFrame({"c": [1, 2, 3, 4, 5, 6, 7], "d": [7, 6, 5, 4, 3, 2, 1]})
@@ -931,7 +984,7 @@ def test_merge_by_index_patterns(how, shuffle):
                     how=how,
                     left_index=True,
                     right_index=True,
-                    shuffle=shuffle,
+                    shuffle=shuffle_method,
                 ),
                 pd_merge(pdl, pdr, how=how, left_index=True, right_index=True),
             )
@@ -942,7 +995,7 @@ def test_merge_by_index_patterns(how, shuffle):
                     how=how,
                     left_index=True,
                     right_index=True,
-                    shuffle=shuffle,
+                    shuffle=shuffle_method,
                 ),
                 pd_merge(pdr, pdl, how=how, left_index=True, right_index=True),
             )
@@ -954,7 +1007,7 @@ def test_merge_by_index_patterns(how, shuffle):
                     how=how,
                     left_index=True,
                     right_index=True,
-                    shuffle=shuffle,
+                    shuffle=shuffle_method,
                     indicator=True,
                 ),
                 pd_merge(
@@ -968,7 +1021,7 @@ def test_merge_by_index_patterns(how, shuffle):
                     how=how,
                     left_index=True,
                     right_index=True,
-                    shuffle=shuffle,
+                    shuffle=shuffle_method,
                     indicator=True,
                 ),
                 pd_merge(
@@ -978,24 +1031,36 @@ def test_merge_by_index_patterns(how, shuffle):
 
             assert_eq(
                 ddr.merge(
-                    ddl, how=how, left_index=True, right_index=True, shuffle=shuffle
+                    ddl,
+                    how=how,
+                    left_index=True,
+                    right_index=True,
+                    shuffle=shuffle_method,
                 ),
                 pdr.merge(pdl, how=how, left_index=True, right_index=True),
             )
             assert_eq(
                 ddl.merge(
-                    ddr, how=how, left_index=True, right_index=True, shuffle=shuffle
+                    ddr,
+                    how=how,
+                    left_index=True,
+                    right_index=True,
+                    shuffle=shuffle_method,
                 ),
                 pdl.merge(pdr, how=how, left_index=True, right_index=True),
             )
 
             # hash join
             list_eq(
-                dd.merge(ddl, ddr, how=how, left_on="a", right_on="c", shuffle=shuffle),
+                dd.merge(
+                    ddl, ddr, how=how, left_on="a", right_on="c", shuffle=shuffle_method
+                ),
                 pd.merge(pdl, pdr, how=how, left_on="a", right_on="c"),
             )
             list_eq(
-                dd.merge(ddl, ddr, how=how, left_on="b", right_on="d", shuffle=shuffle),
+                dd.merge(
+                    ddl, ddr, how=how, left_on="b", right_on="d", shuffle=shuffle_method
+                ),
                 pd.merge(pdl, pdr, how=how, left_on="b", right_on="d"),
             )
 
@@ -1006,7 +1071,7 @@ def test_merge_by_index_patterns(how, shuffle):
                     how=how,
                     left_on="c",
                     right_on="a",
-                    shuffle=shuffle,
+                    shuffle=shuffle_method,
                     indicator=True,
                 ),
                 pd.merge(pdr, pdl, how=how, left_on="c", right_on="a", indicator=True),
@@ -1018,43 +1083,54 @@ def test_merge_by_index_patterns(how, shuffle):
                     how=how,
                     left_on="d",
                     right_on="b",
-                    shuffle=shuffle,
+                    shuffle=shuffle_method,
                     indicator=True,
                 ),
                 pd.merge(pdr, pdl, how=how, left_on="d", right_on="b", indicator=True),
             )
 
             list_eq(
-                dd.merge(ddr, ddl, how=how, left_on="c", right_on="a", shuffle=shuffle),
+                dd.merge(
+                    ddr, ddl, how=how, left_on="c", right_on="a", shuffle=shuffle_method
+                ),
                 pd.merge(pdr, pdl, how=how, left_on="c", right_on="a"),
             )
             list_eq(
-                dd.merge(ddr, ddl, how=how, left_on="d", right_on="b", shuffle=shuffle),
+                dd.merge(
+                    ddr, ddl, how=how, left_on="d", right_on="b", shuffle=shuffle_method
+                ),
                 pd.merge(pdr, pdl, how=how, left_on="d", right_on="b"),
             )
 
             list_eq(
-                ddl.merge(ddr, how=how, left_on="a", right_on="c", shuffle=shuffle),
+                ddl.merge(
+                    ddr, how=how, left_on="a", right_on="c", shuffle=shuffle_method
+                ),
                 pdl.merge(pdr, how=how, left_on="a", right_on="c"),
             )
             list_eq(
-                ddl.merge(ddr, how=how, left_on="b", right_on="d", shuffle=shuffle),
+                ddl.merge(
+                    ddr, how=how, left_on="b", right_on="d", shuffle=shuffle_method
+                ),
                 pdl.merge(pdr, how=how, left_on="b", right_on="d"),
             )
 
             list_eq(
-                ddr.merge(ddl, how=how, left_on="c", right_on="a", shuffle=shuffle),
+                ddr.merge(
+                    ddl, how=how, left_on="c", right_on="a", shuffle=shuffle_method
+                ),
                 pdr.merge(pdl, how=how, left_on="c", right_on="a"),
             )
             list_eq(
-                ddr.merge(ddl, how=how, left_on="d", right_on="b", shuffle=shuffle),
+                ddr.merge(
+                    ddl, how=how, left_on="d", right_on="b", shuffle=shuffle_method
+                ),
                 pdr.merge(pdl, how=how, left_on="d", right_on="b"),
             )
 
 
 @pytest.mark.parametrize("how", ["inner", "outer", "left", "right"])
-@pytest.mark.parametrize("shuffle", ["disk", "tasks"])
-def test_join_by_index_patterns(how, shuffle):
+def test_join_by_index_patterns(how, shuffle_method):
 
     # Similar test cases as test_merge_by_index_patterns,
     # but columns / index for join have same dtype
@@ -1110,15 +1186,23 @@ def test_join_by_index_patterns(how, shuffle):
             ddl = dd.from_pandas(pdl, lpart)
             ddr = dd.from_pandas(pdr, rpart)
 
-            assert_eq(ddl.join(ddr, how=how, shuffle=shuffle), pdl.join(pdr, how=how))
-            assert_eq(ddr.join(ddl, how=how, shuffle=shuffle), pdr.join(pdl, how=how))
+            assert_eq(
+                ddl.join(ddr, how=how, shuffle=shuffle_method), pdl.join(pdr, how=how)
+            )
+            assert_eq(
+                ddr.join(ddl, how=how, shuffle=shuffle_method), pdr.join(pdl, how=how)
+            )
 
             assert_eq(
-                ddl.join(ddr, how=how, lsuffix="l", rsuffix="r", shuffle=shuffle),
+                ddl.join(
+                    ddr, how=how, lsuffix="l", rsuffix="r", shuffle=shuffle_method
+                ),
                 pdl.join(pdr, how=how, lsuffix="l", rsuffix="r"),
             )
             assert_eq(
-                ddr.join(ddl, how=how, lsuffix="l", rsuffix="r", shuffle=shuffle),
+                ddr.join(
+                    ddl, how=how, lsuffix="l", rsuffix="r", shuffle=shuffle_method
+                ),
                 pdr.join(pdl, how=how, lsuffix="l", rsuffix="r"),
             )
 
@@ -1145,9 +1229,22 @@ def test_join_by_index_patterns(how, shuffle):
             """
 
 
+def test_join_gives_proper_divisions():
+    # https://github.com/dask/dask/issues/8113
+    df = pd.DataFrame({"a": ["a", "b", "c"]}, index=[0, 1, 2])
+    ddf = dd.from_pandas(df, npartitions=1)
+
+    right_df = pd.DataFrame({"b": [1.0, 2.0, 3.0]}, index=["a", "b", "c"])
+
+    expected = df.join(right_df, how="inner", on="a")
+    actual = ddf.join(right_df, how="inner", on="a")
+    assert actual.divisions == ddf.divisions
+
+    assert_eq(expected, actual)
+
+
 @pytest.mark.parametrize("how", ["inner", "outer", "left", "right"])
-@pytest.mark.parametrize("shuffle", ["disk", "tasks"])
-def test_merge_by_multiple_columns(how, shuffle):
+def test_merge_by_multiple_columns(how, shuffle_method):
     # warnings here from pandas
     pdf1l = pd.DataFrame(
         {
@@ -1207,8 +1304,12 @@ def test_merge_by_multiple_columns(how, shuffle):
             ddl = dd.from_pandas(pdl, lpart)
             ddr = dd.from_pandas(pdr, rpart)
 
-            assert_eq(ddl.join(ddr, how=how, shuffle=shuffle), pdl.join(pdr, how=how))
-            assert_eq(ddr.join(ddl, how=how, shuffle=shuffle), pdr.join(pdl, how=how))
+            assert_eq(
+                ddl.join(ddr, how=how, shuffle=shuffle_method), pdl.join(pdr, how=how)
+            )
+            assert_eq(
+                ddr.join(ddl, how=how, shuffle=shuffle_method), pdr.join(pdl, how=how)
+            )
 
             assert_eq(
                 dd.merge(
@@ -1217,7 +1318,7 @@ def test_merge_by_multiple_columns(how, shuffle):
                     how=how,
                     left_index=True,
                     right_index=True,
-                    shuffle=shuffle,
+                    shuffle=shuffle_method,
                 ),
                 pd.merge(pdl, pdr, how=how, left_index=True, right_index=True),
             )
@@ -1228,27 +1329,35 @@ def test_merge_by_multiple_columns(how, shuffle):
                     how=how,
                     left_index=True,
                     right_index=True,
-                    shuffle=shuffle,
+                    shuffle=shuffle_method,
                 ),
                 pd.merge(pdr, pdl, how=how, left_index=True, right_index=True),
             )
 
             # hash join
             list_eq(
-                dd.merge(ddl, ddr, how=how, left_on="a", right_on="d", shuffle=shuffle),
+                dd.merge(
+                    ddl, ddr, how=how, left_on="a", right_on="d", shuffle=shuffle_method
+                ),
                 pd.merge(pdl, pdr, how=how, left_on="a", right_on="d"),
             )
             list_eq(
-                dd.merge(ddl, ddr, how=how, left_on="b", right_on="e", shuffle=shuffle),
+                dd.merge(
+                    ddl, ddr, how=how, left_on="b", right_on="e", shuffle=shuffle_method
+                ),
                 pd.merge(pdl, pdr, how=how, left_on="b", right_on="e"),
             )
 
             list_eq(
-                dd.merge(ddr, ddl, how=how, left_on="d", right_on="a", shuffle=shuffle),
+                dd.merge(
+                    ddr, ddl, how=how, left_on="d", right_on="a", shuffle=shuffle_method
+                ),
                 pd.merge(pdr, pdl, how=how, left_on="d", right_on="a"),
             )
             list_eq(
-                dd.merge(ddr, ddl, how=how, left_on="e", right_on="b", shuffle=shuffle),
+                dd.merge(
+                    ddr, ddl, how=how, left_on="e", right_on="b", shuffle=shuffle_method
+                ),
                 pd.merge(pdr, pdl, how=how, left_on="e", right_on="b"),
             )
 
@@ -1259,7 +1368,7 @@ def test_merge_by_multiple_columns(how, shuffle):
                     how=how,
                     left_on=["a", "b"],
                     right_on=["d", "e"],
-                    shuffle=shuffle,
+                    shuffle=shuffle_method,
                 ),
                 pd.merge(pdl, pdr, how=how, left_on=["a", "b"], right_on=["d", "e"]),
             )
@@ -1307,12 +1416,14 @@ def test_cheap_inner_merge_with_pandas_object():
     b = pd.DataFrame({"x": [1, 2, 3, 4], "z": list("abda")})
 
     dc = da.merge(b, on="x", how="inner")
+    assert not hlg_layer_topological(dc.dask, -1).is_materialized()
     assert all("shuffle" not in k[0] for k in dc.dask)
 
     list_eq(da.merge(b, on="x", how="inner"), a.merge(b, on="x", how="inner"))
 
 
-def test_cheap_single_partition_merge():
+@pytest.mark.parametrize("flip", [False, True])
+def test_cheap_single_partition_merge(flip):
     a = pd.DataFrame(
         {"x": [1, 2, 3, 4, 5, 6], "y": list("abdabd")}, index=[10, 20, 30, 40, 50, 60]
     )
@@ -1321,11 +1432,15 @@ def test_cheap_single_partition_merge():
     b = pd.DataFrame({"x": [1, 2, 3, 4], "z": list("abda")})
     bb = dd.from_pandas(b, npartitions=1, sort=False)
 
-    cc = aa.merge(bb, on="x", how="inner")
+    pd_inputs = (b, a) if flip else (a, b)
+    inputs = (bb, aa) if flip else (aa, bb)
+
+    cc = dd.merge(*inputs, on="x", how="inner")
+    assert not hlg_layer_topological(cc.dask, -1).is_materialized()
     assert all("shuffle" not in k[0] for k in cc.dask)
     assert len(cc.dask) == len(aa.dask) * 2 + len(bb.dask)
 
-    list_eq(aa.merge(bb, on="x", how="inner"), a.merge(b, on="x", how="inner"))
+    list_eq(cc, pd.merge(*pd_inputs, on="x", how="inner"))
 
 
 def test_cheap_single_partition_merge_divisions():
@@ -1338,6 +1453,8 @@ def test_cheap_single_partition_merge_divisions():
     bb = dd.from_pandas(b, npartitions=1, sort=False)
 
     actual = aa.merge(bb, on="x", how="inner")
+    assert not hlg_layer_topological(actual.dask, -1).is_materialized()
+
     assert not actual.known_divisions
     assert_divisions(actual)
 
@@ -1347,21 +1464,27 @@ def test_cheap_single_partition_merge_divisions():
 
 
 @pytest.mark.parametrize("how", ["left", "right"])
-def test_cheap_single_parition_merge_left_right(how):
+@pytest.mark.parametrize("flip", [False, True])
+def test_cheap_single_parition_merge_left_right(how, flip):
     a = pd.DataFrame({"x": range(8), "z": list("ababbdda")}, index=range(8))
     aa = dd.from_pandas(a, npartitions=1)
 
     b = pd.DataFrame({"x": [1, 2, 3, 4], "z": list("abda")}, index=range(4))
     bb = dd.from_pandas(b, npartitions=1)
 
-    actual = aa.merge(bb, left_index=True, right_on="x", how=how)
-    expected = a.merge(b, left_index=True, right_on="x", how=how)
+    pd_inputs = (b, a) if flip else (a, b)
+    inputs = (bb, aa) if flip else (aa, bb)
 
+    actual = dd.merge(*inputs, left_index=True, right_on="x", how=how)
+    expected = pd.merge(*pd_inputs, left_index=True, right_on="x", how=how)
+
+    assert not hlg_layer_topological(actual.dask, -1).is_materialized()
     assert_eq(actual, expected)
 
-    actual = aa.merge(bb, left_on="x", right_index=True, how=how)
-    expected = a.merge(b, left_on="x", right_index=True, how=how)
+    actual = dd.merge(*inputs, left_on="x", right_index=True, how=how)
+    expected = pd.merge(*pd_inputs, left_on="x", right_index=True, how=how)
 
+    assert not hlg_layer_topological(actual.dask, -1).is_materialized()
     assert_eq(actual, expected)
 
 
@@ -1382,14 +1505,16 @@ def test_cheap_single_partition_merge_on_index():
     # for empty joins.
     expected.index = expected.index.astype("int64")
 
-    assert actual.known_divisions
+    assert not hlg_layer_topological(actual.dask, -1).is_materialized()
+    assert not actual.known_divisions
     assert_eq(actual, expected)
 
     actual = bb.merge(aa, right_index=True, left_on="x", how="inner")
     expected = b.merge(a, right_index=True, left_on="x", how="inner")
     expected.index = expected.index.astype("int64")
 
-    assert actual.known_divisions
+    assert not hlg_layer_topological(actual.dask, -1).is_materialized()
+    assert not actual.known_divisions
     assert_eq(actual, expected)
 
 
@@ -1405,15 +1530,14 @@ def test_merge_maintains_columns():
     assert tuple(merged.columns) == ("D", "C", "B", "A", "G", "H", "I")
 
 
-@pytest.mark.parametrize("shuffle", ["disk", "tasks"])
-def test_merge_index_without_divisions(shuffle):
+def test_merge_index_without_divisions(shuffle_method):
     a = pd.DataFrame({"x": [1, 2, 3, 4, 5]}, index=[1, 2, 3, 4, 5])
     b = pd.DataFrame({"y": [1, 2, 3, 4, 5]}, index=[5, 4, 3, 2, 1])
 
     aa = dd.from_pandas(a, npartitions=3, sort=False)
     bb = dd.from_pandas(b, npartitions=2)
 
-    result = aa.join(bb, how="inner", shuffle=shuffle)
+    result = aa.join(bb, how="inner", shuffle=shuffle_method)
     expected = a.join(b, how="inner")
     assert_eq(result, expected)
 
@@ -1497,7 +1621,7 @@ def test_concat2():
         ("x", 1): pd.DataFrame({"a": [4, 5, 6], "b": [3, 2, 1]}),
         ("x", 2): pd.DataFrame({"a": [7, 8, 9], "b": [0, 0, 0]}),
     }
-    meta = make_meta({"a": "i8", "b": "i8"})
+    meta = make_meta({"a": "i8", "b": "i8"}, parent_meta=pd.DataFrame())
     a = dd.DataFrame(dsk, "x", meta, [None, None])
     dsk = {
         ("y", 0): pd.DataFrame({"a": [10, 20, 30], "b": [40, 50, 60]}),
@@ -1510,7 +1634,7 @@ def test_concat2():
         ("y", 0): pd.DataFrame({"b": [10, 20, 30], "c": [40, 50, 60]}),
         ("y", 1): pd.DataFrame({"b": [40, 50, 60], "c": [30, 20, 10]}),
     }
-    meta = make_meta({"b": "i8", "c": "i8"})
+    meta = make_meta({"b": "i8", "c": "i8"}, parent_meta=pd.DataFrame())
     c = dd.DataFrame(dsk, "y", meta, [None, None])
 
     dsk = {
@@ -1521,7 +1645,11 @@ def test_concat2():
             {"b": [40, 50, 60], "c": [30, 20, 10], "d": [90, 80, 70]}, index=[3, 4, 5]
         ),
     }
-    meta = make_meta({"b": "i8", "c": "i8", "d": "i8"}, index=pd.Index([], "i8"))
+    meta = make_meta(
+        {"b": "i8", "c": "i8", "d": "i8"},
+        index=pd.Index([], "i8"),
+        parent_meta=pd.DataFrame(),
+    )
     d = dd.DataFrame(dsk, "y", meta, [0, 3, 5])
 
     cases = [[a, b], [a, c], [a, d]]
@@ -1616,7 +1744,6 @@ def test_concat3():
         )
 
 
-@pytest.mark.filterwarnings("ignore")
 def test_concat4_interleave_partitions():
     pdf1 = pd.DataFrame(
         np.random.randn(10, 5), columns=list("ABCDE"), index=list("abcdefghij")
@@ -1664,7 +1791,6 @@ def test_concat4_interleave_partitions():
     assert msg in str(err.value)
 
 
-@pytest.mark.filterwarnings("ignore")
 def test_concat5():
     pdf1 = pd.DataFrame(
         np.random.randn(7, 5), columns=list("ABCDE"), index=list("abcdefg")
@@ -1771,7 +1897,6 @@ def test_concat5():
         (False, False, False),
     ],
 )
-@pytest.mark.filterwarnings("ignore")
 def test_concat_categorical(known, cat_index, divisions):
     frames = [
         pd.DataFrame(
@@ -1924,14 +2049,13 @@ def test_append():
     check_with_warning(ddf.a, df3.b, df.a, df3.b)
 
 
-@pytest.mark.filterwarnings("ignore")
 def test_append2():
     dsk = {
         ("x", 0): pd.DataFrame({"a": [1, 2, 3], "b": [4, 5, 6]}),
         ("x", 1): pd.DataFrame({"a": [4, 5, 6], "b": [3, 2, 1]}),
         ("x", 2): pd.DataFrame({"a": [7, 8, 9], "b": [0, 0, 0]}),
     }
-    meta = make_meta({"a": "i8", "b": "i8"})
+    meta = make_meta({"a": "i8", "b": "i8"}, parent_meta=pd.DataFrame())
     ddf1 = dd.DataFrame(dsk, "x", meta, [None, None])
 
     dsk = {
@@ -1945,7 +2069,7 @@ def test_append2():
         ("y", 0): pd.DataFrame({"b": [10, 20, 30], "c": [40, 50, 60]}),
         ("y", 1): pd.DataFrame({"b": [40, 50, 60], "c": [30, 20, 10]}),
     }
-    meta = make_meta({"b": "i8", "c": "i8"})
+    meta = make_meta({"b": "i8", "c": "i8"}, parent_meta=pd.DataFrame())
     ddf3 = dd.DataFrame(dsk, "y", meta, [None, None])
 
     assert_eq(ddf1.append(ddf2), ddf1.compute().append(ddf2.compute(), sort=False))
@@ -2078,9 +2202,10 @@ def test_merge_outer_empty():
 
     for x in range(0, k_clusters + 1):
         assert_eq(
-            dd.merge(empty_df, df[df.cluster == x], how="outer").compute(),
-            df[df.cluster == x].compute(),
+            dd.merge(empty_df, df[df.cluster == x], how="outer"),
+            df[df.cluster == x],
             check_index=False,
+            check_divisions=False,
         )
 
 
@@ -2095,7 +2220,9 @@ def test_dtype_equality_warning():
     assert len(r) == 0
 
 
-@pytest.mark.parametrize("engine", ["pandas", "cudf"])
+@pytest.mark.parametrize(
+    "engine", ["pandas", pytest.param("cudf", marks=pytest.mark.gpu)]
+)
 def test_groupby_concat_cudf(engine):
 
     # NOTE: Issue #5643 Reproducer
@@ -2143,6 +2270,30 @@ def test_groupby_concat_cudf(engine):
     assert_eq(res_dd.compute().sort_index(), res.sort_index())
 
 
+@pytest.mark.parametrize("ordered", [True, False])
+def test_concat_ignore_order(ordered):
+    pdf1 = pd.DataFrame(
+        {
+            "x": pd.Categorical(
+                ["a", "b", "c", "a"], categories=["a", "b", "c"], ordered=ordered
+            )
+        }
+    )
+    ddf1 = dd.from_pandas(pdf1, 2)
+    pdf2 = pd.DataFrame(
+        {
+            "x": pd.Categorical(
+                ["c", "b", "a"], categories=["c", "b", "a"], ordered=ordered
+            )
+        }
+    )
+    ddf2 = dd.from_pandas(pdf2, 2)
+    expected = pd.concat([pdf1, pdf2])
+    expected["x"] = expected["x"].astype("category")
+    result = dd.concat([ddf1, ddf2], ignore_order=True)
+    assert_eq(result, expected)
+
+
 def test_categorical_join():
     # https://github.com/dask/dask/issues/6134
     df = pd.DataFrame(
@@ -2164,12 +2315,8 @@ def test_categorical_join():
     assert actual_dask.join_col.dtype == "category"
 
     actual = actual_dask.compute()
-    if dd._compat.PANDAS_GT_100:
-        assert actual.join_col.dtype == "category"
-        assert assert_eq(expected, actual)
-    else:
-        assert actual.join_col.dtype == "object"
-        assert (expected.values == actual.values).all()
+    assert actual.join_col.dtype == "category"
+    assert assert_eq(expected, actual)
 
 
 def test_categorical_merge_with_columns_missing_from_left():
@@ -2220,6 +2367,127 @@ def test_categorical_merge_retains_category_dtype():
     actual_dask = df1.merge(df2, on="A")
     assert actual_dask.A.dtype == "category"
 
-    if dd._compat.PANDAS_GT_100:
-        actual = actual_dask.compute()
-        assert actual.A.dtype == "category"
+    actual = actual_dask.compute()
+    assert actual.A.dtype == "category"
+
+
+def test_categorical_merge_does_not_raise_setting_with_copy_warning():
+    # https://github.com/dask/dask/issues/7087
+    df1 = pd.DataFrame(data={"A": ["a", "b", "c"]}, index=["s", "v", "w"])
+    df2 = pd.DataFrame(data={"B": ["t", "d", "i"]}, index=["v", "w", "r"])
+
+    ddf1 = dd.from_pandas(df1, npartitions=1)
+
+    df2 = df2.astype({"B": "category"})
+    assert_eq(df1.join(df2), ddf1.join(df2))
+
+
+@pytest.mark.parametrize("how", ["inner", "left", "right"])
+@pytest.mark.parametrize("npartitions", [28, 32])
+@pytest.mark.parametrize("base", ["lg", "sm"])
+def test_merge_tasks_large_to_small(how, npartitions, base):
+
+    size_lg = 3000
+    size_sm = 300
+    npartitions_lg = 30
+    npartitions_sm = 3
+    broadcast_bias = 1.0  # Prioritize broadcast
+
+    lg = pd.DataFrame(
+        {
+            "x": np.random.choice(np.arange(100), size_lg),
+            "y": np.arange(size_lg),
+        }
+    )
+    ddf_lg = dd.from_pandas(lg, npartitions=npartitions_lg)
+
+    sm = pd.DataFrame(
+        {
+            "x": np.random.choice(np.arange(100), size_sm),
+            "y": np.arange(size_sm),
+        }
+    )
+    ddf_sm = dd.from_pandas(sm, npartitions=npartitions_sm)
+
+    if base == "lg":
+        left = lg
+        ddf_left = ddf_lg
+        right = sm
+        ddf_right = ddf_sm
+    else:
+        left = sm
+        ddf_left = ddf_sm
+        right = lg
+        ddf_right = ddf_lg
+
+    dd_result = dd.merge(
+        ddf_left,
+        ddf_right,
+        on="y",
+        how=how,
+        npartitions=npartitions,
+        broadcast=broadcast_bias,
+        shuffle="tasks",
+    )
+    pd_result = pd.merge(left, right, on="y", how=how)
+
+    # Make sure `on` dtypes match
+    dd_result["y"] = dd_result["y"].astype(np.int32)
+    pd_result["y"] = pd_result["y"].astype(np.int32)
+
+    if npartitions:
+        assert dd_result.npartitions == npartitions
+
+    assert_eq(
+        dd_result.compute().sort_values("y"),
+        pd_result.sort_values("y"),
+        check_index=False,
+    )
+
+
+@pytest.mark.parametrize("how", ["right", "inner"])
+def test_pairwise_rejects_unsupported_join_types(how):
+    base_df = dd.from_pandas(
+        pd.DataFrame({"a": [1, 2, 3], "b": [4, 5, 6]}, index=[0, 1, 3]), 3
+    )
+    dfs = [
+        dd.from_pandas(
+            pd.DataFrame({"a": [4, 5, 6], "b": [3, 2, 1]}, index=[5, 6, 8]), 3
+        ),
+        dd.from_pandas(
+            pd.DataFrame({"a": [7, 8, 9], "b": [0, 0, 0]}, index=[9, 9, 9]), 3
+        ),
+    ]
+
+    with pytest.raises(ValueError) as e:
+        base_df.join(dfs, how=how)
+    e.match("merge_multi only supports left or outer joins")
+
+
+@pytest.mark.parametrize("how", ["left", "outer"])
+@pytest.mark.parametrize("npartitions_base", [1, 2, 3])
+@pytest.mark.parametrize("npartitions_other", [1, 2, 3])
+def test_pairwise_merge_results_in_identical_output_df(
+    how, npartitions_base, npartitions_other
+):
+    dfs_to_merge = []
+    for i in range(10):
+        df = pd.DataFrame(
+            {
+                f"{i}A": [5, 6, 7, 8],
+                f"{i}B": [4, 3, 2, 1],
+            },
+            index=[0, 1, 2, 3],
+        )
+        ddf = dd.from_pandas(df, npartitions_other)
+        dfs_to_merge.append(ddf)
+
+    ddf_loop = dd.from_pandas(pd.DataFrame(index=[0, 1, 3]), npartitions_base)
+    for ddf in dfs_to_merge:
+        ddf_loop = ddf_loop.join(ddf, how=how)
+
+    ddf_pairwise = dd.from_pandas(pd.DataFrame(index=[0, 1, 3]), npartitions_base)
+
+    ddf_pairwise = ddf_pairwise.join(dfs_to_merge, how=how)
+
+    assert_eq(ddf_pairwise, ddf_loop)
