@@ -1,15 +1,14 @@
 import pytest
 
 pd = pytest.importorskip("pandas")
-import pandas.util.testing as tm
 
 import numpy as np
 
 import dask.array as da
 import dask.dataframe as dd
+from dask.dataframe._compat import PANDAS_GT_120
 from dask.dataframe.utils import assert_eq
 
-
 _BASE_UFUNCS = [
     "conj",
     "exp",
@@ -167,7 +166,7 @@ def test_ufunc_array_wrap(ufunc):
     - np.ufunc(dd.Series) => np.ndarray
     - np.ufunc(pd.Series) => np.ndarray
     """
-    if ufunc == "fix" and np.__version__ >= "1.13.0":
+    if ufunc == "fix":
         pytest.skip("fix calls floor in a way that we do not yet support")
 
     dafunc = getattr(da, ufunc)
@@ -183,10 +182,10 @@ def test_ufunc_array_wrap(ufunc):
     assert_eq(dafunc(ds), pd.Series(npfunc(s), index=s.index))
 
     assert isinstance(npfunc(ds), np.ndarray)
-    tm.assert_numpy_array_equal(npfunc(ds), npfunc(s))
+    np.testing.assert_equal(npfunc(ds), npfunc(s))
 
     assert isinstance(dafunc(s), np.ndarray)
-    tm.assert_numpy_array_equal(dafunc(s), npfunc(s))
+    np.testing.assert_array_equal(dafunc(s), npfunc(s))
 
     df = pd.DataFrame(
         {
@@ -205,10 +204,10 @@ def test_ufunc_array_wrap(ufunc):
     assert_eq(dafunc(ddf), exp)
 
     assert isinstance(npfunc(ddf), np.ndarray)
-    tm.assert_numpy_array_equal(npfunc(ddf), npfunc(df))
+    np.testing.assert_array_equal(npfunc(ddf), npfunc(df))
 
     assert isinstance(dafunc(df), np.ndarray)
-    tm.assert_numpy_array_equal(dafunc(df), npfunc(df))
+    np.testing.assert_array_equal(dafunc(df), npfunc(df))
 
 
 _UFUNCS_2ARG = [
@@ -218,7 +217,7 @@ _UFUNCS_2ARG = [
     "hypot",
     "copysign",
     "nextafter",
-    "ldexp",
+    pytest.param("ldexp", marks=[pytest.mark.filterwarnings("ignore::RuntimeWarning")]),
     pytest.param("fmod", marks=[pytest.mark.filterwarnings("ignore::RuntimeWarning")]),
     "logical_and",
     "logical_or",
@@ -453,7 +452,7 @@ def test_2args_with_array(ufunc, pandas, darray):
     assert isinstance(dafunc(dask, darray), dask_type)
     assert isinstance(dafunc(darray, dask), dask_type)
 
-    tm.assert_numpy_array_equal(
+    np.testing.assert_array_equal(
         dafunc(dask, darray).compute().values, npfunc(pandas.values, darray).compute()
     )
 
@@ -461,10 +460,10 @@ def test_2args_with_array(ufunc, pandas, darray):
     assert isinstance(npfunc(dask, darray), dask_type)
     assert isinstance(npfunc(darray, dask), dask_type)
 
-    tm.assert_numpy_array_equal(
+    np.testing.assert_array_equal(
         npfunc(dask, darray).compute().values, npfunc(pandas.values, darray.compute())
     )
-    tm.assert_numpy_array_equal(
+    np.testing.assert_array_equal(
         npfunc(darray, dask).compute().values, npfunc(darray.compute(), pandas.values)
     )
 
@@ -490,6 +489,18 @@ def test_ufunc_with_reduction(redfunc, ufunc, pandas):
     np_redfunc = getattr(np, redfunc)
     np_ufunc = getattr(np, ufunc)
 
+    if (
+        PANDAS_GT_120
+        and (redfunc == "prod")
+        and ufunc in ["conj", "square", "negative", "absolute"]
+        and isinstance(pandas, pd.DataFrame)
+    ):
+        # TODO(pandas) follow pandas behaviour?
+        # starting with pandas 1.2.0, the ufunc is applied column-wise, and therefore
+        # applied on the integer columns separately, overflowing for those columns
+        # (instead of being applied on 2D ndarray that was converted to float)
+        pytest.xfail("'prod' overflowing with integer columns in pandas 1.2.0")
+
     with pytest.warns(None):
         assert isinstance(np_redfunc(dask), (dd.DataFrame, dd.Series, dd.core.Scalar))
         assert_eq(np_redfunc(np_ufunc(dask)), np_redfunc(np_ufunc(pandas)))
