@@ -1,43 +1,40 @@
 import math
 import numbers
 import re
-import textwrap
-from collections.abc import Iterator, Mapping
-
 import sys
+import textwrap
 import traceback
+from collections.abc import Iterator, Mapping
 from contextlib import contextmanager
 
 import numpy as np
 import pandas as pd
-from pandas.api.types import (
-    is_categorical_dtype,
-    is_scalar,
-    is_sparse,
-    is_period_dtype,
-    is_datetime64tz_dtype,
-    is_interval_dtype,
-)
+from pandas.api.types import is_scalar  # noqa: F401
+from pandas.api.types import is_categorical_dtype, is_dtype_equal
 
-# include these here for compat
-from ._compat import (  # noqa: F401
-    PANDAS_GT_100,
-    PANDAS_GT_110,
-    tm,
-)
-
-from .extensions import make_array_nonempty, make_scalar
 from ..base import is_dask_collection
 from ..core import get_deps
 from ..local import get_sync
-from ..utils import asciitable, is_arraylike, Dispatch, typename
+from ..utils import is_arraylike  # noqa: F401
+from ..utils import asciitable
 from ..utils import is_dataframe_like as dask_is_dataframe_like
-from ..utils import is_series_like as dask_is_series_like
 from ..utils import is_index_like as dask_is_index_like
-
-# register pandas extension types
-from . import _dtypes  # noqa: F401
+from ..utils import is_series_like as dask_is_series_like
+from ..utils import typename
+from . import _dtypes  # noqa: F401 register pandas extension types
 from . import methods
+from ._compat import PANDAS_GT_110, PANDAS_GT_120, tm  # noqa: F401
+from .dispatch import make_meta  # noqa : F401
+from .dispatch import make_meta_obj, meta_nonempty  # noqa : F401
+from .extensions import make_scalar
+
+meta_object_types = (pd.Series, pd.DataFrame, pd.Index, pd.MultiIndex)
+try:
+    import scipy.sparse as sp
+
+    meta_object_types += (sp.spmatrix,)
+except ImportError:
+    pass
 
 
 def is_integer_na_dtype(t):
@@ -55,6 +52,18 @@ def is_integer_na_dtype(t):
     return isinstance(dtype, types)
 
 
+def is_float_na_dtype(t):
+    if not PANDAS_GT_120:
+        return False
+
+    dtype = getattr(t, "dtype", t)
+    types = (
+        pd.Float32Dtype,
+        pd.Float64Dtype,
+    )
+    return isinstance(dtype, types)
+
+
 def shard_df_on_index(df, divisions):
     """Shard a DataFrame by ranges on its index
 
@@ -138,7 +147,7 @@ def insert_meta_param_description(*args, **kwargs):
     body = textwrap.wrap(
         _META_DESCRIPTION, initial_indent=indent, subsequent_indent=indent, width=78
     )
-    descr = "{0}\n{1}".format(_META_TYPES, "\n".join(body))
+    descr = "{}\n{}".format(_META_TYPES, "\n".join(body))
     if f.__doc__:
         if "$META" in f.__doc__:
             f.__doc__ = f.__doc__.replace("$META", descr)
@@ -147,7 +156,7 @@ def insert_meta_param_description(*args, **kwargs):
             parameter_header = "Parameters\n%s----------" % indent[4:]
             first, last = re.split("Parameters\\n[ ]*----------", f.__doc__)
             parameters, rest = last.split("\n\n", 1)
-            f.__doc__ = "{0}{1}{2}\n{3}{4}\n\n{5}".format(
+            f.__doc__ = "{}{}{}\n{}{}\n\n{}".format(
                 first, parameter_header, parameters, indent[4:], descr, rest
             )
     return f
@@ -184,7 +193,7 @@ def raise_on_meta_error(funcname=None, udf=False):
             "---------\n"
             "{2}"
         )
-        msg = msg.format(" in `{0}`".format(funcname) if funcname else "", repr(e), tb)
+        msg = msg.format(f" in `{funcname}`" if funcname else "", repr(e), tb)
         raise ValueError(msg) from e
 
 
@@ -222,10 +231,10 @@ def strip_unknown_categories(x, just_drop_unknown=False):
                         if just_drop_unknown:
                             x[c].cat.remove_categories(UNKNOWN_CATEGORIES, inplace=True)
                         else:
-                            x[c].cat.set_categories([], inplace=True)
+                            x[c] = x[c].cat.set_categories([])
         elif isinstance(x, pd.Series):
             if is_categorical_dtype(x.dtype) and not has_known_categories(x):
-                x.cat.set_categories([], inplace=True)
+                x = x.cat.set_categories([])
         if isinstance(x.index, pd.CategoricalIndex) and not has_known_categories(
             x.index
         ):
@@ -257,10 +266,10 @@ def clear_known_categories(x, cols=None, index=True):
             elif not mask.loc[cols].all():
                 raise ValueError("Not all columns are categoricals")
             for c in cols:
-                x[c].cat.set_categories([UNKNOWN_CATEGORIES], inplace=True)
+                x[c] = x[c].cat.set_categories([UNKNOWN_CATEGORIES])
         elif isinstance(x, pd.Series):
             if is_categorical_dtype(x.dtype):
-                x.cat.set_categories([UNKNOWN_CATEGORIES], inplace=True)
+                x = x.cat.set_categories([UNKNOWN_CATEGORIES])
         if index and isinstance(x.index, pd.CategoricalIndex):
             x.index = x.index.set_categories([UNKNOWN_CATEGORIES])
     elif isinstance(x, pd.CategoricalIndex):
@@ -276,218 +285,6 @@ def _empty_series(name, dtype, index=None):
     return pd.Series([], dtype=dtype, name=name, index=index)
 
 
-make_meta = Dispatch("make_meta")
-
-
-@make_meta.register((pd.Series, pd.DataFrame))
-def make_meta_pandas(x, index=None):
-    return x.iloc[:0]
-
-
-@make_meta.register(pd.Index)
-def make_meta_index(x, index=None):
-    return x[0:0]
-
-
-@make_meta.register(object)
-def make_meta_object(x, index=None):
-    """Create an empty pandas object containing the desired metadata.
-
-    Parameters
-    ----------
-    x : dict, tuple, list, pd.Series, pd.DataFrame, pd.Index, dtype, scalar
-        To create a DataFrame, provide a `dict` mapping of `{name: dtype}`, or
-        an iterable of `(name, dtype)` tuples. To create a `Series`, provide a
-        tuple of `(name, dtype)`. If a pandas object, names, dtypes, and index
-        should match the desired output. If a dtype or scalar, a scalar of the
-        same dtype is returned.
-    index :  pd.Index, optional
-        Any pandas index to use in the metadata. If none provided, a
-        `RangeIndex` will be used.
-
-    Examples
-    --------
-
-    >>> make_meta([('a', 'i8'), ('b', 'O')])    # doctest: +SKIP
-    Empty DataFrame
-    Columns: [a, b]
-    Index: []
-    >>> make_meta(('a', 'f8'))                  # doctest: +SKIP
-    Series([], Name: a, dtype: float64)
-    >>> make_meta('i8')                         # doctest: +SKIP
-    1
-    """
-    if hasattr(x, "_meta"):
-        return x._meta
-    elif is_arraylike(x) and x.shape:
-        return x[:0]
-
-    if index is not None:
-        index = make_meta(index)
-
-    if isinstance(x, dict):
-        return pd.DataFrame(
-            {c: _empty_series(c, d, index=index) for (c, d) in x.items()}, index=index
-        )
-    if isinstance(x, tuple) and len(x) == 2:
-        return _empty_series(x[0], x[1], index=index)
-    elif isinstance(x, (list, tuple)):
-        if not all(isinstance(i, tuple) and len(i) == 2 for i in x):
-            raise ValueError(
-                "Expected iterable of tuples of (name, dtype), got {0}".format(x)
-            )
-        return pd.DataFrame(
-            {c: _empty_series(c, d, index=index) for (c, d) in x},
-            columns=[c for c, d in x],
-            index=index,
-        )
-    elif not hasattr(x, "dtype") and x is not None:
-        # could be a string, a dtype object, or a python type. Skip `None`,
-        # because it is implictly converted to `dtype('f8')`, which we don't
-        # want here.
-        try:
-            dtype = np.dtype(x)
-            return _scalar_from_dtype(dtype)
-        except Exception:
-            # Continue on to next check
-            pass
-
-    if is_scalar(x):
-        return _nonempty_scalar(x)
-
-    raise TypeError("Don't know how to create metadata from {0}".format(x))
-
-
-_numeric_index_types = (pd.Int64Index, pd.Float64Index, pd.UInt64Index)
-
-meta_nonempty = Dispatch("meta_nonempty")
-
-
-@meta_nonempty.register(object)
-def meta_nonempty_object(x):
-    """Create a nonempty pandas object from the given metadata.
-
-    Returns a pandas DataFrame, Series, or Index that contains two rows
-    of fake data.
-    """
-    if is_scalar(x):
-        return _nonempty_scalar(x)
-    else:
-        raise TypeError(
-            "Expected Pandas-like Index, Series, DataFrame, or scalar, "
-            "got {0}".format(typename(type(x)))
-        )
-
-
-@meta_nonempty.register(pd.DataFrame)
-def meta_nonempty_dataframe(x):
-    idx = meta_nonempty(x.index)
-    dt_s_dict = dict()
-    data = dict()
-    for i, c in enumerate(x.columns):
-        series = x.iloc[:, i]
-        dt = series.dtype
-        if dt not in dt_s_dict:
-            dt_s_dict[dt] = _nonempty_series(x.iloc[:, i], idx=idx)
-        data[i] = dt_s_dict[dt]
-    res = pd.DataFrame(data, index=idx, columns=np.arange(len(x.columns)))
-    res.columns = x.columns
-    if PANDAS_GT_100:
-        res.attrs = x.attrs
-    return res
-
-
-@meta_nonempty.register(pd.Index)
-def _nonempty_index(idx):
-    typ = type(idx)
-    if typ is pd.RangeIndex:
-        return pd.RangeIndex(2, name=idx.name)
-    elif typ in _numeric_index_types:
-        return typ([1, 2], name=idx.name)
-    elif typ is pd.Index:
-        return pd.Index(["a", "b"], name=idx.name)
-    elif typ is pd.DatetimeIndex:
-        start = "1970-01-01"
-        # Need a non-monotonic decreasing index to avoid issues with
-        # partial string indexing see https://github.com/dask/dask/issues/2389
-        # and https://github.com/pandas-dev/pandas/issues/16515
-        # This doesn't mean `_meta_nonempty` should ever rely on
-        # `self.monotonic_increasing` or `self.monotonic_decreasing`
-        try:
-            return pd.date_range(
-                start=start, periods=2, freq=idx.freq, tz=idx.tz, name=idx.name
-            )
-        except ValueError:  # older pandas versions
-            data = [start, "1970-01-02"] if idx.freq is None else None
-            return pd.DatetimeIndex(
-                data, start=start, periods=2, freq=idx.freq, tz=idx.tz, name=idx.name
-            )
-    elif typ is pd.PeriodIndex:
-        return pd.period_range(
-            start="1970-01-01", periods=2, freq=idx.freq, name=idx.name
-        )
-    elif typ is pd.TimedeltaIndex:
-        start = np.timedelta64(1, "D")
-        try:
-            return pd.timedelta_range(
-                start=start, periods=2, freq=idx.freq, name=idx.name
-            )
-        except ValueError:  # older pandas versions
-            start = np.timedelta64(1, "D")
-            data = [start, start + 1] if idx.freq is None else None
-            return pd.TimedeltaIndex(
-                data, start=start, periods=2, freq=idx.freq, name=idx.name
-            )
-    elif typ is pd.CategoricalIndex:
-        if len(idx.categories) == 0:
-            data = pd.Categorical(_nonempty_index(idx.categories), ordered=idx.ordered)
-        else:
-            data = pd.Categorical.from_codes(
-                [-1, 0], categories=idx.categories, ordered=idx.ordered
-            )
-        return pd.CategoricalIndex(data, name=idx.name)
-    elif typ is pd.MultiIndex:
-        levels = [_nonempty_index(l) for l in idx.levels]
-        codes = [[0, 0] for i in idx.levels]
-        try:
-            return pd.MultiIndex(levels=levels, codes=codes, names=idx.names)
-        except TypeError:  # older pandas versions
-            return pd.MultiIndex(levels=levels, labels=codes, names=idx.names)
-
-    raise TypeError(
-        "Don't know how to handle index of type {0}".format(typename(type(idx)))
-    )
-
-
-hash_object_dispatch = Dispatch("hash_object_dispatch")
-
-
-@hash_object_dispatch.register((pd.DataFrame, pd.Series, pd.Index))
-def hash_object_pandas(
-    obj, index=True, encoding="utf8", hash_key=None, categorize=True
-):
-    return pd.util.hash_pandas_object(
-        obj, index=index, encoding=encoding, hash_key=hash_key, categorize=categorize
-    )
-
-
-group_split_dispatch = Dispatch("group_split_dispatch")
-
-
-@group_split_dispatch.register((pd.DataFrame, pd.Series, pd.Index))
-def group_split_pandas(df, c, k, ignore_index=False):
-    indexer, locations = pd._libs.algos.groupsort_indexer(
-        c.astype(np.int64, copy=False), k
-    )
-    df2 = df.take(indexer)
-    locations = locations.cumsum()
-    parts = [
-        df2.iloc[a:b].reset_index(drop=True) if ignore_index else df2.iloc[a:b]
-        for a, b in zip(locations[:-1], locations[1:])
-    ]
-    return dict(zip(range(k), parts))
-
-
 _simple_fake_mapping = {
     "b": np.bool_(True),
     "V": np.void(b" "),
@@ -509,20 +306,7 @@ def _scalar_from_dtype(dtype):
         o = _simple_fake_mapping[dtype.kind]
         return o.astype(dtype) if dtype.kind in ("m", "M") else o
     else:
-        raise TypeError("Can't handle dtype: {0}".format(dtype))
-
-
-@make_scalar.register(np.dtype)
-def _(dtype):
-    return _scalar_from_dtype(dtype)
-
-
-@make_scalar.register(pd.Timestamp)
-@make_scalar.register(pd.Timedelta)
-@make_scalar.register(pd.Period)
-@make_scalar.register(pd.Interval)
-def _(x):
-    return x
+        raise TypeError(f"Can't handle dtype: {dtype}")
 
 
 def _nonempty_scalar(x):
@@ -533,54 +317,7 @@ def _nonempty_scalar(x):
         dtype = x.dtype if hasattr(x, "dtype") else np.dtype(type(x))
         return make_scalar(dtype)
 
-    raise TypeError("Can't handle meta of type '{0}'".format(typename(type(x))))
-
-
-@meta_nonempty.register(pd.Series)
-def _nonempty_series(s, idx=None):
-    # TODO: Use register dtypes with make_array_nonempty
-    if idx is None:
-        idx = _nonempty_index(s.index)
-    dtype = s.dtype
-    if len(s) > 0:
-        # use value from meta if provided
-        data = [s.iloc[0]] * 2
-    elif is_datetime64tz_dtype(dtype):
-        entry = pd.Timestamp("1970-01-01", tz=dtype.tz)
-        data = [entry, entry]
-    elif is_categorical_dtype(dtype):
-        if len(s.cat.categories):
-            data = [s.cat.categories[0]] * 2
-            cats = s.cat.categories
-        else:
-            data = _nonempty_index(s.cat.categories)
-            cats = s.cat.categories[:0]
-        data = pd.Categorical(data, categories=cats, ordered=s.cat.ordered)
-    elif is_integer_na_dtype(dtype):
-        data = pd.array([1, None], dtype=dtype)
-    elif is_period_dtype(dtype):
-        # pandas 0.24.0+ should infer this to be Series[Period[freq]]
-        freq = dtype.freq
-        data = [pd.Period("2000", freq), pd.Period("2001", freq)]
-    elif is_sparse(dtype):
-        entry = _scalar_from_dtype(dtype.subtype)
-        if PANDAS_GT_100:
-            data = pd.array([entry, entry], dtype=dtype)
-        else:
-            data = pd.SparseArray([entry, entry], dtype=dtype)
-    elif is_interval_dtype(dtype):
-        entry = _scalar_from_dtype(dtype.subtype)
-        data = pd.array([entry, entry], dtype=dtype)
-    elif type(dtype) in make_array_nonempty._lookup:
-        data = make_array_nonempty(dtype)
-    else:
-        entry = _scalar_from_dtype(dtype)
-        data = np.array([entry, entry], dtype=dtype)
-
-    out = pd.Series(data, name=s.name, index=idx)
-    if PANDAS_GT_100:
-        out.attrs = s.attrs
-    return out
+    raise TypeError(f"Can't handle meta of type '{typename(type(x))}'")
 
 
 def is_dataframe_like(df):
@@ -626,7 +363,7 @@ def check_meta(x, meta, funcname=None, numeric_equal=True):
             if UNKNOWN_CATEGORIES in a.categories or UNKNOWN_CATEGORIES in b.categories:
                 return True
             return a == b
-        return (a.kind in eq_types and b.kind in eq_types) or (a == b)
+        return (a.kind in eq_types and b.kind in eq_types) or is_dtype_equal(a, b)
 
     if not (
         is_dataframe_like(meta) or is_series_like(meta) or is_index_like(meta)
@@ -639,7 +376,7 @@ def check_meta(x, meta, funcname=None, numeric_equal=True):
     # Notice, we use .__class__ as opposed to type() in order to support
     # object proxies see <https://github.com/dask/dask/pull/6981>
     if x.__class__ != meta.__class__:
-        errmsg = "Expected partition of type `%s` but got `%s`" % (
+        errmsg = "Expected partition of type `{}` but got `{}`".format(
             typename(type(meta)),
             typename(type(x)),
         )
@@ -651,7 +388,7 @@ def check_meta(x, meta, funcname=None, numeric_equal=True):
             if not equal_dtypes(a, b)
         ]
         if bad_dtypes:
-            errmsg = "Partition type: `%s`\n%s" % (
+            errmsg = "Partition type: `{}`\n{}".format(
                 typename(type(meta)),
                 asciitable(["Column", "Found", "Expected"], bad_dtypes),
             )
@@ -661,7 +398,7 @@ def check_meta(x, meta, funcname=None, numeric_equal=True):
     else:
         if equal_dtypes(x.dtype, meta.dtype):
             return x
-        errmsg = "Partition type: `%s`\n%s" % (
+        errmsg = "Partition type: `{}`\n{}".format(
             typename(type(meta)),
             asciitable(["", "dtype"], [("Found", x.dtype), ("Expected", meta.dtype)]),
         )
@@ -696,11 +433,11 @@ def index_summary(idx, name=None):
     if n:
         head = idx[0]
         tail = idx[-1]
-        summary = ", {} to {}".format(head, tail)
+        summary = f", {head} to {tail}"
     else:
         summary = ""
 
-    return "{}: {} entries{}".format(name, n, summary)
+    return f"{name}: {n} entries{summary}"
 
 
 ###############################################################
@@ -763,13 +500,13 @@ def _check_dask(dsk, check_names=True, check_dtypes=True, result=None):
             if check_dtypes:
                 assert_dask_dtypes(dsk, result)
         else:
-            msg = "Unsupported dask instance {0} found".format(type(dsk))
+            msg = f"Unsupported dask instance {type(dsk)} found"
             raise AssertionError(msg)
         return result
     return dsk
 
 
-def _maybe_sort(a):
+def _maybe_sort(a, check_index: bool):
     # sort by value, then index
     try:
         if is_dataframe_like(a):
@@ -782,14 +519,14 @@ def _maybe_sort(a):
             a = a.sort_values()
     except (TypeError, IndexError, ValueError):
         pass
-    return a.sort_index()
+    return a.sort_index() if check_index else a
 
 
 def assert_eq(
     a,
     b,
     check_names=True,
-    check_dtypes=True,
+    check_dtype=True,
     check_divisions=True,
     check_index=True,
     **kwargs,
@@ -803,25 +540,28 @@ def assert_eq(
             assert at == bt, (at, bt)
     assert_sane_keynames(a)
     assert_sane_keynames(b)
-    a = _check_dask(a, check_names=check_names, check_dtypes=check_dtypes)
-    b = _check_dask(b, check_names=check_names, check_dtypes=check_dtypes)
-    if not check_index:
-        a = a.reset_index(drop=True)
-        b = b.reset_index(drop=True)
+    a = _check_dask(a, check_names=check_names, check_dtypes=check_dtype)
+    b = _check_dask(b, check_names=check_names, check_dtypes=check_dtype)
     if hasattr(a, "to_pandas"):
         a = a.to_pandas()
     if hasattr(b, "to_pandas"):
         b = b.to_pandas()
+    if isinstance(a, (pd.DataFrame, pd.Series)):
+        a = _maybe_sort(a, check_index)
+        b = _maybe_sort(b, check_index)
+    if not check_index:
+        a = a.reset_index(drop=True)
+        b = b.reset_index(drop=True)
     if isinstance(a, pd.DataFrame):
-        a = _maybe_sort(a)
-        b = _maybe_sort(b)
-        tm.assert_frame_equal(a, b, **kwargs)
+        tm.assert_frame_equal(
+            a, b, check_names=check_names, check_dtype=check_dtype, **kwargs
+        )
     elif isinstance(a, pd.Series):
-        a = _maybe_sort(a)
-        b = _maybe_sort(b)
-        tm.assert_series_equal(a, b, check_names=check_names, **kwargs)
+        tm.assert_series_equal(
+            a, b, check_names=check_names, check_dtype=check_dtype, **kwargs
+        )
     elif isinstance(a, pd.Index):
-        tm.assert_index_equal(a, b, **kwargs)
+        tm.assert_index_equal(a, b, exact=check_dtype, **kwargs)
     else:
         if a == b:
             return True
@@ -842,14 +582,15 @@ def assert_dask_graph(dask, label):
             k = k[0]
         if k.startswith(label):
             return True
-    raise AssertionError(
-        "given dask graph doesn't contain label: {label}".format(label=label)
-    )
+    raise AssertionError(f"given dask graph doesn't contain label: {label}")
 
 
 def assert_divisions(ddf):
     if not hasattr(ddf, "divisions"):
         return
+
+    assert isinstance(ddf.divisions, tuple)
+
     if not getattr(ddf, "known_divisions", False):
         return
 
@@ -881,7 +622,7 @@ def assert_sane_keynames(ddf):
         assert isinstance(k, (str, bytes))
         assert len(k) < 100
         assert " " not in k
-        assert k.split("-")[0].isidentifier()
+        assert k.split("-")[0].isidentifier(), k
 
 
 def assert_dask_dtypes(ddf, res, numeric_equal=True):
