@@ -3,19 +3,18 @@ from operator import add, mul
 import pytest
 
 from dask.callbacks import Callback
-from dask.local import get_sync
 from dask.diagnostics import ProgressBar
 from dask.diagnostics.progress import format_time
+from dask.local import get_sync
 from dask.threaded import get as get_threaded
 
-
 dsk = {"a": 1, "b": 2, "c": (add, "a", "b"), "d": (mul, "a", "b"), "e": (mul, "c", "d")}
 
 
 def check_bar_completed(capsys, width=40):
     out, err = capsys.readouterr()
     assert out.count("100% Completed") == 1
-    bar, percent, time = [i.strip() for i in out.split("\r")[-1].split("|")]
+    bar, percent, time = (i.strip() for i in out.split("\r")[-1].split("|"))
     assert bar == "[" + "#" * width + "]"
     assert percent == "100% Completed"
 
@@ -69,6 +68,7 @@ def test_format_time():
 
 def test_register(capsys):
     try:
+        assert not Callback.active
         p = ProgressBar()
         p.register()
 
