@@ -1,17 +1,17 @@
-import re
 import os
+import re
 from functools import partial
 
-from .core import istask, get_dependencies, ishashable
-from .utils import funcname, import_required, key_split, apply
-
+from .core import get_dependencies, ishashable, istask
+from .utils import apply, funcname, import_required, key_split
 
 graphviz = import_required(
     "graphviz",
-    "Drawing dask graphs requires the "
-    "`graphviz` python library and the "
-    "`graphviz` system library to be "
-    "installed.",
+    "Drawing dask graphs requires the `graphviz` python library and the "
+    "`graphviz` system library.\n\n"
+    "Please either conda or pip install as follows:\n\n"
+    "  conda install python-graphviz     # either conda install\n"
+    "  python -m pip install graphviz    # or pip install and follow installation instructions",
 )
 
 
@@ -31,13 +31,13 @@ def task_label(task):
         func = task[1]
     if hasattr(func, "funcs"):
         if len(func.funcs) > 1:
-            return "{0}(...)".format(funcname(func.funcs[0]))
+            return f"{funcname(func.funcs[0])}(...)"
         else:
             head = funcname(func.funcs[0])
     else:
         head = funcname(func)
     if any(has_sub_tasks(i) for i in task[1:]):
-        return "{0}(...)".format(head)
+        return f"{head}(...)"
     else:
         return head
 
@@ -95,7 +95,7 @@ def label(x, cache=None):
             for h in m.groups():
                 if cache is not None:
                     n = cache.get(h, len(cache))
-                    label = "#{0}".format(n)
+                    label = f"#{n}"
                     # cache will be overwritten destructively
                     cache[h] = n
                 else:
@@ -137,15 +137,15 @@ def to_graphviz(
     verbose=False,
     **kwargs,
 ):
-    if data_attributes is None:
-        data_attributes = {}
-    if function_attributes is None:
-        function_attributes = {}
-    if graph_attr is None:
-        graph_attr = {}
-
+    data_attributes = data_attributes or {}
+    function_attributes = function_attributes or {}
     graph_attr = graph_attr or {}
+    node_attr = node_attr or {}
+    edge_attr = edge_attr or {}
+
     graph_attr["rankdir"] = rankdir
+    node_attr["fontname"] = "helvetica"
+
     graph_attr.update(kwargs)
     g = graphviz.Digraph(
         graph_attr=graph_attr, node_attr=node_attr, edge_attr=edge_attr
@@ -274,7 +274,12 @@ def dot_graph(dsk, filename="mydask", format=None, **kwargs):
 
 def graphviz_to_file(g, filename, format):
     fmts = [".png", ".pdf", ".dot", ".svg", ".jpeg", ".jpg"]
-    if format is None and any(filename.lower().endswith(fmt) for fmt in fmts):
+
+    if (
+        format is None
+        and filename is not None
+        and any(filename.lower().endswith(fmt) for fmt in fmts)
+    ):
         filename, format = os.path.splitext(filename)
         format = format[1:].lower()
 
@@ -293,7 +298,7 @@ def graphviz_to_file(g, filename, format):
 
     display_cls = _get_display_cls(format)
 
-    if not filename:
+    if filename is None:
         return display_cls(data=data)
 
     full_filename = ".".join([filename, format])
