@@ -1,16 +1,16 @@
 import os
-import sys
 import signal
 import threading
+from concurrent.futures import ThreadPoolExecutor
 from multiprocessing.pool import ThreadPool
-from time import time, sleep
+from time import sleep, time
 
 import pytest
 
 import dask
 from dask.system import CPU_COUNT
 from dask.threaded import get
-from dask.utils_test import inc, add
+from dask.utils_test import add, inc
 
 
 def test_get():
@@ -55,14 +55,16 @@ def test_exceptions_rise_to_top():
     pytest.raises(ValueError, lambda: get(dsk, "y"))
 
 
-def test_reuse_pool():
-    with ThreadPool() as pool:
+@pytest.mark.parametrize("pool_typ", [ThreadPool, ThreadPoolExecutor])
+def test_reuse_pool(pool_typ):
+    with pool_typ(CPU_COUNT) as pool:
         with dask.config.set(pool=pool):
             assert get({"x": (inc, 1)}, "x") == 2
             assert get({"x": (inc, 1)}, "x") == 2
 
 
-def test_pool_kwarg():
+@pytest.mark.parametrize("pool_typ", [ThreadPool, ThreadPoolExecutor])
+def test_pool_kwarg(pool_typ):
     def f():
         sleep(0.01)
         return threading.get_ident()
@@ -70,7 +72,7 @@ def test_pool_kwarg():
     dsk = {("x", i): (f,) for i in range(30)}
     dsk["x"] = (len, (set, [("x", i) for i in range(len(dsk))]))
 
-    with ThreadPool(3) as pool:
+    with pool_typ(3) as pool:
         assert get(dsk, "x", pool=pool) == 3
 
 
@@ -147,11 +149,7 @@ def test_thread_safety():
     assert L == [1] * 20
 
 
-@pytest.mark.xfail(
-    "xdist" in sys.modules,
-    reason="This test fails intermittently when using pytest-xdist (maybe)",
-    strict=False,
-)
+@pytest.mark.slow
 def test_interrupt():
     # Windows implements `queue.get` using polling,
     # which means we can set an exception to interrupt the call to `get`.
@@ -164,20 +162,17 @@ def test_interrupt():
         def interrupt_main():
             signal.pthread_kill(main_thread, signal.SIGINT)
 
-    def long_task():
-        sleep(5)
-
-    dsk = {("x", i): (long_task,) for i in range(20)}
+    # 7 seconds is is how long the test will take when you factor in teardown.
+    # Don't set it too short or the test will become flaky on non-performing CI
+    dsk = {("x", i): (sleep, 7) for i in range(20)}
     dsk["x"] = (len, list(dsk.keys()))
-    try:
-        interrupter = threading.Timer(0.5, interrupt_main)
-        interrupter.start()
-        start = time()
+
+    # 3 seconds is how long the test will take without teardown
+    interrupter = threading.Timer(3, interrupt_main)
+    interrupter.start()
+
+    start = time()
+    with pytest.raises(KeyboardInterrupt):
         get(dsk, "x")
-    except KeyboardInterrupt:
-        pass
-    except Exception:
-        assert False, "Failed to interrupt"
     stop = time()
-    if stop - start > 4:
-        assert False, "Failed to interrupt"
+    assert stop < start + 6
