@@ -1,9 +1,11 @@
-from time import time
-import dask
-from dask import threaded, multiprocessing, local
 from random import randint
+from time import time
+
 import matplotlib.pyplot as plt
 
+import dask
+from dask import local, multiprocessing, threaded
+
 
 def noop(x):
     pass
@@ -13,7 +15,7 @@ nrepetitions = 1
 
 
 def trivial(width, height):
-    """ Embarrassingly parallel dask """
+    """Embarrassingly parallel dask"""
     d = {("x", 0, i): i for i in range(width)}
     for j in range(1, height):
         d.update({("x", j, i): (noop, ("x", j - 1, i)) for i in range(width)})
@@ -21,7 +23,7 @@ def trivial(width, height):
 
 
 def crosstalk(width, height, connections):
-    """ Natural looking dask with some inter-connections """
+    """Natural looking dask with some inter-connections"""
     d = {("x", 0, i): i for i in range(width)}
     for j in range(1, height):
         d.update(
@@ -37,7 +39,7 @@ def crosstalk(width, height, connections):
 
 
 def dense(width, height):
-    """ Full barriers between each step """
+    """Full barriers between each step"""
     d = {("x", 0, i): i for i in range(width)}
     for j in range(1, height):
         d.update(
